@@ -1,6 +1,4 @@
 /*
- *  arch-tag: Interface to audio backend
- *
  *  Copyright (C) 2003 Jorn Baayen <jorn@nl.linux.org>
  *  Copyright (C) 2003 Colin Walters <walters@verbum.org>
  *
@@ -33,7 +31,7 @@
 
 #include <glib-object.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
-#include "rb-metadata.h"
+#include <metadata/rb-metadata.h>
 
 G_BEGIN_DECLS
 
@@ -96,12 +94,6 @@ struct _RBPlayerIface
 	void		(*set_volume)		(RBPlayer *player,
 						 float volume);
 	float		(*get_volume)		(RBPlayer *player);
-	void		(*set_replaygain)	(RBPlayer *player,
-						 const char *uri,
-						 double track_gain,
-						 double track_peak,
-						 double album_gain,
-						 double album_peak);
 
 	gboolean	(*seekable)		(RBPlayer *player);
 	void		(*set_time)		(RBPlayer *player,
@@ -114,7 +106,8 @@ struct _RBPlayerIface
 	void		(*playing_stream)	(RBPlayer *player,
 						 gpointer stream_data);
 	void		(*eos)			(RBPlayer *player,
-						 gpointer stream_data);
+						 gpointer stream_data,
+						 gboolean early);
 	void		(*info)			(RBPlayer *player,
 						 gpointer stream_data,
 						 RBMetaDataField field,
@@ -137,6 +130,9 @@ struct _RBPlayerIface
 	void		(*image)		(RBPlayer *player,
 						 gpointer stream_data,
 						 GdkPixbuf *image);
+	void		(*redirect)		(RBPlayer *player,
+						 gpointer stream_data,
+						 const gchar *uri);
 };
 
 GType		rb_player_get_type   (void);
@@ -162,10 +158,6 @@ gboolean	rb_player_playing    (RBPlayer *player);
 
 void		rb_player_set_volume (RBPlayer *player, float volume);
 float		rb_player_get_volume (RBPlayer *player);
-void		rb_player_set_replaygain (RBPlayer *player,
-					  const char *uri,
-					  double track_gain, double track_peak,
-					  double album_gain, double album_peak);
 
 gboolean	rb_player_seekable   (RBPlayer *player);
 void		rb_player_set_time   (RBPlayer *player, gint64 newtime);
@@ -174,7 +166,7 @@ gint64		rb_player_get_time   (RBPlayer *player);
 gboolean	rb_player_multiple_open (RBPlayer *player);
 
 /* only to be used by subclasses */
-void	_rb_player_emit_eos (RBPlayer *player, gpointer stream_data);
+void	_rb_player_emit_eos (RBPlayer *player, gpointer stream_data, gboolean early);
 void	_rb_player_emit_info (RBPlayer *player, gpointer stream_data, RBMetaDataField field, GValue *value);
 void	_rb_player_emit_buffering (RBPlayer *player, gpointer stream_data, guint progress);
 void	_rb_player_emit_error (RBPlayer *player, gpointer stream_data, GError *error);
@@ -183,6 +175,7 @@ void	_rb_player_emit_event (RBPlayer *player, gpointer stream_data, const char *
 void	_rb_player_emit_playing_stream (RBPlayer *player, gpointer stream_data);
 void	_rb_player_emit_volume_changed (RBPlayer *player, float volume);
 void	_rb_player_emit_image (RBPlayer *player, gpointer stream_data, GdkPixbuf *image);
+void	_rb_player_emit_redirect (RBPlayer *player, gpointer stream_data, const char *uri);
 
 G_END_DECLS
 
