@@ -79,15 +79,13 @@ static gpointer rb_audiocd_load_songs (RBAudioCdSource *source);
 static void rb_audiocd_load_metadata (RBAudioCdSource *source, RhythmDB *db);
 static void rb_audiocd_load_metadata_cancel (RBAudioCdSource *source);
 
-static void metadata_gather_cb (RhythmDB *db, RhythmDBEntry *entry, RBStringValueMap *data, RBAudioCdSource *source);
-static GValue *album_artist_metadata_request_cb (RhythmDB *db, RhythmDBEntry *entry, RBAudioCdSource *source);
-static GValue *album_artist_sortname_metadata_request_cb (RhythmDB *db, RhythmDBEntry *entry, RBAudioCdSource *source);
-
+static gboolean update_artist_cb (GtkWidget *widget, GdkEventFocus *event, RBAudioCdSource *source);
+static gboolean update_artist_sort_cb (GtkWidget *widget, GdkEventFocus *event, RBAudioCdSource *source);
 static gboolean update_album_cb (GtkWidget *widget, GdkEventFocus *event, RBAudioCdSource *source);
 static gboolean update_genre_cb (GtkWidget *widget, GdkEventFocus *event, RBAudioCdSource *source);
 static gboolean update_year_cb (GtkWidget *widget, GdkEventFocus *event, RBAudioCdSource *source);
 static gboolean update_disc_number_cb (GtkWidget *widget, GdkEventFocus *event, RBAudioCdSource *source);
-#if defined(HAVE_SJ_METADATA_GETTER) && GTK_CHECK_VERSION(2,17,6)
+#if defined(HAVE_SJ_METADATA_GETTER)
 static void info_bar_response_cb (GtkInfoBar *info_bar, gint response_id, RBAudioCdSource *source);
 #endif
 
@@ -252,6 +250,25 @@ rb_audiocd_source_dispose (GObject *object)
 	G_OBJECT_CLASS (rb_audiocd_source_parent_class)->dispose (object);
 }
 
+static inline void
+force_no_spacing (GtkWidget *widget)
+{
+	static gboolean first_time = TRUE;
+	if (first_time) {
+		gtk_rc_parse_string ("\n"
+				     "   style \"audiocd-extract-header-style\"\n"
+				     "   {\n"
+				     "      GtkCheckButton::indicator-spacing=0\n"
+				     "   }\n"
+				     "\n"
+				     "   widget \"*.audiocd-extract-header\" style \"audiocd-extract-header-style\"\n"
+				     "\n");
+		first_time = FALSE;
+	}
+
+	gtk_widget_set_name (widget, "audiocd-extract-header");
+}
+
 static void
 rb_audiocd_source_constructed (GObject *object)
 {
@@ -262,7 +279,6 @@ rb_audiocd_source_constructed (GObject *object)
 	GtkTreeViewColumn *extract;
 	GtkWidget *widget;
 	GtkAction *action;
-	RhythmDB *db;
 	RBPlugin *plugin;
 	RBShell *shell;
 	char *ui_file;
@@ -317,6 +333,7 @@ rb_audiocd_source_constructed (GObject *object)
 	gtk_tree_view_column_set_clickable (extract, TRUE);
 	widget = gtk_check_button_new ();
 	g_object_set (widget, "active", TRUE, NULL);
+	force_no_spacing (widget);
 	gtk_widget_show_all (widget);
 	g_signal_connect_object (extract, "clicked", G_CALLBACK (extract_column_clicked_cb), source, 0);
 	gtk_tree_view_column_set_widget (extract, widget);
@@ -326,24 +343,15 @@ rb_audiocd_source_constructed (GObject *object)
 	/* set column width */
 	gtk_cell_renderer_get_size (renderer, GTK_WIDGET (entry_view), NULL, NULL, NULL, &toggle_width, NULL);
 	gtk_tree_view_column_set_sizing (extract, GTK_TREE_VIEW_COLUMN_FIXED);
-	gtk_tree_view_column_set_fixed_width (extract, toggle_width + 5);
+	gtk_tree_view_column_set_fixed_width (extract, toggle_width + 10);
 
 	rb_entry_view_insert_column_custom (entry_view, extract, "", "Extract", NULL, NULL, NULL, 1);
-	gtk_widget_set_tooltip_text (GTK_WIDGET (extract->button), _("Select tracks to be extracted"));
+	gtk_widget_set_tooltip_text (gtk_tree_view_column_get_widget (extract),
+	                             _("Select tracks to be extracted"));
 
 	/* hide the 'album' column */
 	gtk_tree_view_column_set_visible (rb_entry_view_get_column (entry_view, RB_ENTRY_VIEW_COL_ALBUM), FALSE);
 
-	/* handle extra metadata requests for album artist and album artist sortname */
-	db = get_db_for_source (source);
-	g_signal_connect_object (G_OBJECT (db), "entry-extra-metadata-request::" RHYTHMDB_PROP_ALBUM_ARTIST,
-				 G_CALLBACK (album_artist_metadata_request_cb), source, 0);
-	g_signal_connect_object (G_OBJECT (db), "entry-extra-metadata-request::" RHYTHMDB_PROP_ALBUM_ARTIST_SORTNAME,
-				 G_CALLBACK (album_artist_sortname_metadata_request_cb), source, 0);
-	g_signal_connect_object (G_OBJECT (db), "entry-extra-metadata-gather",
-				 G_CALLBACK (metadata_gather_cb), source, 0);
-	g_object_unref (db);
-
 	/* set up the album info widgets */
 	g_object_get (source, "plugin", &plugin, NULL);
 	ui_file = rb_plugin_find_file (plugin, "album-info.ui");
@@ -355,7 +363,7 @@ rb_audiocd_source_constructed (GObject *object)
 		RBAudioCdSourcePrivate *priv;
 		GtkWidget *table;
 		GtkBuilder *builder;
-#if defined(HAVE_SJ_METADATA_GETTER) && GTK_CHECK_VERSION(2,17,6)
+#if defined(HAVE_SJ_METADATA_GETTER)
 		GtkWidget *box;
 		char *message;
 #endif
@@ -368,7 +376,7 @@ rb_audiocd_source_constructed (GObject *object)
 		table = GTK_WIDGET (gtk_builder_get_object (builder, "album_info"));
 		g_assert (table != NULL);
 
-#if defined(HAVE_SJ_METADATA_GETTER) && GTK_CHECK_VERSION(2,17,6)
+#if defined(HAVE_SJ_METADATA_GETTER)
 		/* Info bar for non-Musicbrainz data */
 		priv->info_bar = gtk_info_bar_new_with_buttons (_("S_ubmit Album"), GTK_RESPONSE_OK,
 								_("Hide"), GTK_RESPONSE_CANCEL,
@@ -395,6 +403,8 @@ rb_audiocd_source_constructed (GObject *object)
 		priv->genre_entry = GTK_WIDGET (gtk_builder_get_object (builder, "genre_entry"));
 		priv->disc_number_entry = GTK_WIDGET (gtk_builder_get_object (builder, "disc_number_entry"));
 
+		g_signal_connect_object (priv->artist_entry, "focus-out-event", G_CALLBACK (update_artist_cb), source, 0);
+		g_signal_connect_object (priv->artist_sort_entry, "focus-out-event", G_CALLBACK (update_artist_sort_cb), source, 0);
 		g_signal_connect_object (priv->album_entry, "focus-out-event", G_CALLBACK (update_album_cb), source, 0);
 		g_signal_connect_object (priv->genre_entry, "focus-out-event", G_CALLBACK (update_genre_cb), source, 0);
 		g_signal_connect_object (priv->year_entry, "focus-out-event", G_CALLBACK (update_year_cb), source, 0);
@@ -827,13 +837,11 @@ metadata_cb (SjMetadataGetter *metadata,
 	} else
 		album = (AlbumDetails *)albums->data;
 
-#if GTK_CHECK_VERSION(2,17,6)
 	if (album->metadata_source != SOURCE_MUSICBRAINZ) {
 		priv->submit_url = sj_metadata_getter_get_submit_url (metadata);
 		if (priv->submit_url != NULL)
 			gtk_widget_show (priv->info_bar);
 	}
-#endif
 
 	if (album->metadata_source == SOURCE_FALLBACK) {
 		rb_debug ("ignoring CD metadata from fallback source");
@@ -872,6 +880,7 @@ metadata_cb (SjMetadataGetter *metadata,
 	rb_debug ("musicbrainz_albumid: %s", album->album_id);
 	rb_debug ("musicbrainz_albumartistid: %s", album->artist_id);
 	rb_debug ("album artist: %s", album->artist);
+	rb_debug ("album artist sortname: %s", album->artist_sortname);
 	rb_debug ("disc number: %d", album->disc_number);
 	rb_debug ("genre: %s", album->genre);
 
@@ -896,6 +905,8 @@ metadata_cb (SjMetadataGetter *metadata,
 		entry_set_string_prop (db, entry, RHYTHMDB_PROP_MUSICBRAINZ_ALBUMID, TRUE, album->album_id);
 		entry_set_string_prop (db, entry, RHYTHMDB_PROP_MUSICBRAINZ_ALBUMARTISTID, TRUE, album->artist_id);
 		entry_set_string_prop (db, entry, RHYTHMDB_PROP_ARTIST_SORTNAME, TRUE, track->artist_sortname);
+		entry_set_string_prop (db, entry, RHYTHMDB_PROP_ALBUM_ARTIST, TRUE, album->artist);
+		entry_set_string_prop (db, entry, RHYTHMDB_PROP_ALBUM_ARTIST_SORTNAME, TRUE, album->artist_sortname);
 
 		g_value_init (&value, G_TYPE_ULONG);
 		g_value_set_ulong (&value, track->duration);
@@ -1151,70 +1162,6 @@ impl_uri_is_source (RBSource *source, const char *uri)
 	return FALSE;
 }
 
-static void
-metadata_gather_cb (RhythmDB *db, RhythmDBEntry *entry, RBStringValueMap *data, RBAudioCdSource *source)
-{
-	RBAudioCdSourcePrivate *priv = AUDIOCD_SOURCE_GET_PRIVATE (source);
-	GValue value = {0,};
-
-	if (_rb_source_check_entry_type (RB_SOURCE (source), entry) == FALSE) {
-		return;
-	}
-
-	if (gtk_entry_get_text_length (GTK_ENTRY (priv->artist_entry)) > 0) {
-		g_value_init (&value, G_TYPE_STRING);
-		g_value_set_string (&value, gtk_entry_get_text (GTK_ENTRY (priv->artist_entry)));
-		rb_string_value_map_set (data, RHYTHMDB_PROP_ALBUM_ARTIST, &value);
-		g_value_unset (&value);
-	}
-
-	if (gtk_entry_get_text_length (GTK_ENTRY (priv->artist_sort_entry)) > 0) {
-		g_value_init (&value, G_TYPE_STRING);
-		g_value_set_string (&value, gtk_entry_get_text (GTK_ENTRY (priv->artist_sort_entry)));
-		rb_string_value_map_set (data, RHYTHMDB_PROP_ALBUM_ARTIST_SORTNAME, &value);
-		g_value_unset (&value);
-	}
-}
-
-
-static GValue *
-album_artist_metadata_request_cb (RhythmDB *db, RhythmDBEntry *entry, RBAudioCdSource *source)
-{
-	RBAudioCdSourcePrivate *priv = AUDIOCD_SOURCE_GET_PRIVATE (source);
-	GValue *value = NULL;
-
-	if (_rb_source_check_entry_type (RB_SOURCE (source), entry) == FALSE) {
-		return NULL;
-	}
-
-	if (gtk_entry_get_text_length (GTK_ENTRY (priv->artist_entry)) > 0) {
-		value = g_new0 (GValue, 1);
-		g_value_init (value, G_TYPE_STRING);
-		g_value_set_string (value, gtk_entry_get_text (GTK_ENTRY (priv->artist_entry)));
-	}
-
-	return value;
-}
-
-static GValue *
-album_artist_sortname_metadata_request_cb (RhythmDB *db, RhythmDBEntry *entry, RBAudioCdSource *source)
-{
-	RBAudioCdSourcePrivate *priv = AUDIOCD_SOURCE_GET_PRIVATE (source);
-	GValue *value = NULL;
-
-	if (_rb_source_check_entry_type (RB_SOURCE (source), entry) == FALSE) {
-		return NULL;
-	}
-
-	if (gtk_entry_get_text_length (GTK_ENTRY (priv->artist_sort_entry)) > 0) {
-		value = g_new0 (GValue, 1);
-		g_value_init (value, G_TYPE_STRING);
-		g_value_set_string (value, gtk_entry_get_text (GTK_ENTRY (priv->artist_sort_entry)));
-	}
-
-	return value;
-}
-
 static void
 update_tracks (RBAudioCdSource *source, RhythmDBPropType property, GValue *value)
 {
@@ -1242,6 +1189,20 @@ update_tracks_string (RBAudioCdSource *source, RhythmDBPropType property, const
 	g_value_unset (&v);
 }
 
+static gboolean
+update_artist_cb (GtkWidget *widget, GdkEventFocus *event, RBAudioCdSource *source)
+{
+	update_tracks_string (source, RHYTHMDB_PROP_ALBUM_ARTIST, gtk_entry_get_text (GTK_ENTRY (widget)));
+	return FALSE;
+}
+
+static gboolean
+update_artist_sort_cb (GtkWidget *widget, GdkEventFocus *event, RBAudioCdSource *source)
+{
+	update_tracks_string (source, RHYTHMDB_PROP_ALBUM_ARTIST_SORTNAME, gtk_entry_get_text (GTK_ENTRY (widget)));
+	return FALSE;
+}
+
 static gboolean
 update_album_cb (GtkWidget *widget, GdkEventFocus *event, RBAudioCdSource *source)
 {
@@ -1294,7 +1255,7 @@ update_disc_number_cb (GtkWidget *widget, GdkEventFocus *event, RBAudioCdSource
 	return FALSE;
 }
 
-#if defined(HAVE_SJ_METADATA_GETTER) && GTK_CHECK_VERSION(2,17,6)
+#if defined(HAVE_SJ_METADATA_GETTER)
 static void
 info_bar_response_cb (GtkInfoBar *info_bar, gint response_id, RBAudioCdSource *source)
 {
