@@ -1,6 +1,4 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
- *
- *  arch-tag: Implementation of last.fm station source object
  *
  *  Copyright (C) 2006 Matt Novenstern <fisxoj@gmail.com>
  *  Copyright (C) 2008 Jonathan Matthew <jonathan@d14n.org>
@@ -74,11 +72,12 @@
 #include "rb-dialog.h"
 #include "rb-debug.h"
 #include "eel-gconf-extensions.h"
-#include "gedit-message-area.h"
 #include "rb-shell-player.h"
 #include "rb-play-order.h"
 #include "rb-lastfm-play-order.h"
 
+#include "gseal-gtk-compat.h"
+
 #define LASTFM_URL "ws.audioscrobbler.com"
 #define RB_LASTFM_PLATFORM "linux"
 #define RB_LASTFM_VERSION "1.5"
@@ -746,6 +745,7 @@ set_message_area_text_and_icon (RBLastfmSource *source,
 	char      *secondary_markup;
 	GtkWidget *primary_label;
 	GtkWidget *secondary_label;
+	GtkWidget *content;
 
 	hbox_content = gtk_hbox_new (FALSE, 8);
 	gtk_widget_show (hbox_content);
@@ -767,7 +767,7 @@ set_message_area_text_and_icon (RBLastfmSource *source,
 	gtk_label_set_use_markup (GTK_LABEL (primary_label), TRUE);
 	gtk_label_set_line_wrap (GTK_LABEL (primary_label), TRUE);
 	gtk_misc_set_alignment (GTK_MISC (primary_label), 0, 0.5);
-	GTK_WIDGET_SET_FLAGS (primary_label, GTK_CAN_FOCUS);
+	gtk_widget_set_can_focus (primary_label, TRUE);
 	gtk_label_set_selectable (GTK_LABEL (primary_label), TRUE);
 
   	if (secondary_text != NULL) {
@@ -777,7 +777,7 @@ set_message_area_text_and_icon (RBLastfmSource *source,
 		g_free (secondary_markup);
 		gtk_widget_show (secondary_label);
 		gtk_box_pack_start (GTK_BOX (vbox), secondary_label, TRUE, TRUE, 0);
-		GTK_WIDGET_SET_FLAGS (secondary_label, GTK_CAN_FOCUS);
+		gtk_widget_set_can_focus (secondary_label, TRUE);
 		gtk_label_set_use_markup (GTK_LABEL (secondary_label), TRUE);
 		gtk_label_set_line_wrap (GTK_LABEL (secondary_label), TRUE);
 		gtk_label_set_selectable (GTK_LABEL (secondary_label), TRUE);
@@ -785,8 +785,8 @@ set_message_area_text_and_icon (RBLastfmSource *source,
 	}
 
 	gtk_widget_show (source->priv->message_area);
-	gedit_message_area_set_contents (GEDIT_MESSAGE_AREA (source->priv->message_area),
-					 hbox_content);
+	content = gtk_info_bar_get_content_area (GTK_INFO_BAR (source->priv->message_area));
+	gtk_container_add (GTK_CONTAINER (content), hbox_content);
 }
 
 static void
@@ -818,9 +818,9 @@ set_message_area (RBLastfmSource *source,
 }
 
 static void
-on_message_area_response (GeditMessageArea *area,
-			  int               response_id,
-			  RBLastfmSource   *source)
+on_message_area_response (GtkInfoBar     *infobar,
+			  int             response_id,
+			  RBLastfmSource *source)
 {
 	RBPlugin  *plugin;
 	GtkWidget *dialog;
@@ -841,9 +841,9 @@ show_error_message (RBLastfmSource *source,
 		return;
 	}
 
-	area = gedit_message_area_new_with_buttons (_("Account Settings"),
-						    GTK_RESPONSE_ACCEPT,
-						    NULL);
+	area = gtk_info_bar_new_with_buttons (_("Account Settings"),
+					      GTK_RESPONSE_ACCEPT,
+					      NULL);
 	set_message_area (source, area);
 	set_message_area_text_and_icon (source,
 					"gtk-dialog-error",
@@ -1189,7 +1189,7 @@ impl_receive_drag (RBSource *asource, GtkSelectionData *selection_data)
 	char *title = NULL;
 	RBLastfmSource *source = RB_LASTFM_SOURCE (asource);
 
-	uri = (char *)selection_data->data;
+	uri = (char *) gtk_selection_data_get_data (selection_data);
 	rb_debug ("parsing uri %s", uri);
 
 	if (strstr (uri, "lastfm://") == NULL)
