@@ -37,6 +37,7 @@
 #include "rb-debug.h"
 #include "rb-plugin.h"
 #include "rb-file-helpers.h"
+#include "rb-util.h"
 
 #define PLAYLIST_SAVE_TIMEOUT	1
 
@@ -62,6 +63,14 @@ enum {
 	PROP_PLAYER_SOURCE
 };
 
+typedef struct {
+	RBGenericPlayerPlaylistSource *source;
+#if TOTEM_PL_PARSER_CHECK_VERSION(2,29,1)
+	TotemPlPlaylist *playlist;
+#endif
+	TotemPlParserType playlist_type;
+} SavePlaylistData;
+
 
 static void
 impl_save_to_xml (RBPlaylistSource *source, xmlNodePtr node)
@@ -69,13 +78,64 @@ impl_save_to_xml (RBPlaylistSource *source, xmlNodePtr node)
 	/* do nothing; just to prevent weirdness */
 }
 
+#if TOTEM_PL_PARSER_CHECK_VERSION(2,29,1)
+
+static void
+set_field_from_property (TotemPlPlaylist *playlist,
+			 TotemPlPlaylistIter *iter,
+			 RhythmDBEntry *entry,
+			 RhythmDBPropType property,
+			 const char *field)
+{
+	const char *value;
+
+	value = rhythmdb_entry_get_string (entry, property);
+	if (value != NULL) {
+		totem_pl_playlist_set (playlist, iter, field, value, NULL);
+	}
+}
+
+static gboolean
+save_playlist_foreach (GtkTreeModel *model,
+		       GtkTreePath *path,
+		       GtkTreeIter *iter,
+		       SavePlaylistData *data)
+{
+	RBGenericPlayerPlaylistSourcePrivate *priv = GET_PRIVATE (data->source);
+	RhythmDBEntry *entry;
+	TotemPlPlaylistIter pl_iter;
+	const char *host_uri;
+	char *uri;
+
+	entry = rhythmdb_query_model_iter_to_entry (RHYTHMDB_QUERY_MODEL (model), iter);
+	if (entry == NULL) {
+		return FALSE;
+	}
+
+	host_uri = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_LOCATION);
+	uri = rb_generic_player_source_uri_to_playlist_uri (priv->player_source, host_uri, data->playlist_type);
+
+	totem_pl_playlist_append (data->playlist, &pl_iter);
+	totem_pl_playlist_set (data->playlist, &pl_iter, TOTEM_PL_PARSER_FIELD_URI, uri, NULL);
+	set_field_from_property (data->playlist, &pl_iter, entry, RHYTHMDB_PROP_ARTIST, TOTEM_PL_PARSER_FIELD_AUTHOR);
+	set_field_from_property (data->playlist, &pl_iter, entry, RHYTHMDB_PROP_GENRE, TOTEM_PL_PARSER_FIELD_GENRE);
+	set_field_from_property (data->playlist, &pl_iter, entry, RHYTHMDB_PROP_ALBUM, TOTEM_PL_PARSER_FIELD_ALBUM);
+	set_field_from_property (data->playlist, &pl_iter, entry, RHYTHMDB_PROP_TITLE, TOTEM_PL_PARSER_FIELD_TITLE);
+	rhythmdb_entry_unref (entry);
+
+	g_free (uri);
+	return FALSE;
+}
+
+#else
+
 static void
 save_playlist_entry (GtkTreeModel *model, GtkTreeIter *iter,
 		     char **uri, char **title,
 		     gboolean *custom_title,
-		     RBGenericPlayerPlaylistSource *source)
+		     SavePlaylistData *data)
 {
-	RBGenericPlayerPlaylistSourcePrivate *priv = GET_PRIVATE (source);
+	RBGenericPlayerPlaylistSourcePrivate *priv = GET_PRIVATE (data->source);
 	RhythmDBEntry *entry;
 	const char *host_uri;
 
@@ -86,11 +146,14 @@ save_playlist_entry (GtkTreeModel *model, GtkTreeIter *iter,
 	}
 
 	host_uri = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_LOCATION);
-	*uri = rb_generic_player_source_uri_to_playlist_uri (priv->player_source, host_uri);
+	*uri = rb_generic_player_source_uri_to_playlist_uri (priv->player_source, host_uri, data->playlist_type);
 	*title = rhythmdb_entry_dup_string (entry, RHYTHMDB_PROP_TITLE);
 	*custom_title = TRUE;
+	rhythmdb_entry_unref (entry);
 }
 
+#endif
+
 /* this probably belongs more in totem than here */
 static const char *
 playlist_format_extension (TotemPlParserType playlist_type)
@@ -121,9 +184,12 @@ save_playlist (RBGenericPlayerPlaylistSource *source)
 	TotemPlParserType playlist_type;
 	RhythmDBQueryModel *query_model;
 	char *name;
-	char *temp_uri;
+	char *temp_path;
 	GError *error = NULL;
 	RBGenericPlayerPlaylistSourcePrivate *priv = GET_PRIVATE (source);
+	GFile *file;
+	gboolean result;
+	SavePlaylistData data;
 
 	priv->save_playlist_id = 0;
 	playlist_type = rb_generic_player_source_get_playlist_format (priv->player_source);
@@ -172,42 +238,61 @@ save_playlist (RBGenericPlayerPlaylistSource *source)
 		g_object_unref (dir);
 	}
 
-	temp_uri = g_strdup_printf ("%s%06X", priv->playlist_path, g_random_int_range (0, 0xFFFFFF));
+	temp_path = g_strdup_printf ("%s%06X", priv->playlist_path, g_random_int_range (0, 0xFFFFFF));
+	file = g_file_new_for_path (temp_path);
 
 	parser = totem_pl_parser_new ();
+	data.source = source;
+	data.playlist_type = playlist_type;
+#if TOTEM_PL_PARSER_CHECK_VERSION(2,29,1)
+	data.playlist = totem_pl_playlist_new ();
+
+	gtk_tree_model_foreach (GTK_TREE_MODEL (query_model),
+				(GtkTreeModelForeachFunc) save_playlist_foreach,
+				&data);
+	if (rb_debug_matches ("totem_pl_parser_save", "totem-pl-parser.c")) {
+		g_object_set (parser, "debug", TRUE, NULL);
+	}
+
+	result = totem_pl_parser_save (parser, data.playlist, file, name, playlist_type, &error);
+	g_object_unref (data.playlist);
+	data.playlist = NULL;
+#else
 	if (rb_debug_matches ("totem_pl_parser_write_with_title", "totem-pl-parser.c")) {
 		g_object_set (parser, "debug", TRUE, NULL);
 	}
-	if (totem_pl_parser_write_with_title (parser,
-					      GTK_TREE_MODEL (query_model),
-					      (TotemPlParserIterFunc) save_playlist_entry,
-					      temp_uri,
-					      name,
-					      playlist_type,
-					      source,
-					      &error) == FALSE) {
+
+	result = totem_pl_parser_write_with_title (parser,
+						   GTK_TREE_MODEL (query_model),
+						   (TotemPlParserIterFunc) save_playlist_entry,
+						   temp_path,
+						   name,
+						   playlist_type,
+						   &data,
+						   &error);
+#endif
+	if (result == FALSE) {
 		/* XXX report this more usefully */
-		g_warning ("Playlist save failed: %s", error->message);
+		g_warning ("Playlist save failed: %s", error ? error->message : "<no error>");
 	} else {
 		GFile *dest;
-		GFile *src;
 
 		dest = g_file_new_for_path (priv->playlist_path);
-		src = g_file_new_for_path (temp_uri);
-		g_file_move (src, dest, G_FILE_COPY_OVERWRITE | G_FILE_COPY_NO_FALLBACK_FOR_MOVE, NULL, NULL, NULL, &error);
+		g_file_move (file, dest, G_FILE_COPY_OVERWRITE | G_FILE_COPY_NO_FALLBACK_FOR_MOVE, NULL, NULL, NULL, &error);
 		if (error != NULL) {
 			/* XXX report this more usefully */
-			g_warning ("Replacing playlist failed: %s", error->message);
+			g_warning ("moving %s => %s failed: %s", temp_path, priv->playlist_path, error->message);
 		}
 
 		g_object_unref (dest);
-		g_object_unref (src);
 	}
 
 	g_clear_error (&error);
 	g_free (name);
-	g_free (temp_uri);
+	g_free (temp_path);
 	g_object_unref (query_model);
+	g_object_unref (parser);
+	g_object_unref (file);
 
 	return FALSE;
 }
@@ -341,20 +426,31 @@ impl_mark_dirty (RBPlaylistSource *source)
 
 RBSource *
 rb_generic_player_playlist_source_new (RBShell *shell,
-				       RBGenericPlayerSource *source,
+				       RBGenericPlayerSource *player_source,
 				       const char *playlist_file,
 				       const char *device_root,
 				       RhythmDBEntryType entry_type)
 {
-	return RB_SOURCE (g_object_new (RB_TYPE_GENERIC_PLAYER_PLAYLIST_SOURCE,
-					"shell", shell,
-					"is-local", FALSE,
-					"entry-type", entry_type,
-					"source-group", RB_SOURCE_GROUP_DEVICES,
-					"player-source", source,
-					"playlist-path", playlist_file,
-					"device-root", device_root,
-					NULL));
+	RBSource *source;
+	source = RB_SOURCE (g_object_new (RB_TYPE_GENERIC_PLAYER_PLAYLIST_SOURCE,
+					  "shell", shell,
+					  "is-local", FALSE,
+					  "entry-type", entry_type,
+					  "source-group", RB_SOURCE_GROUP_DEVICES,
+					  "player-source", player_source,
+					  "playlist-path", playlist_file,
+					  "device-root", device_root,
+					  NULL));
+
+	if (load_playlist (RB_GENERIC_PLAYER_PLAYLIST_SOURCE (source)) == FALSE) {
+		rb_debug ("playlist didn't parse; killing the source");
+		if (g_object_is_floating (source))
+			g_object_ref_sink (source);
+		g_object_unref (source);
+		return NULL;
+	}
+
+	return source;
 }
 
 void
@@ -383,24 +479,6 @@ rb_generic_player_playlist_source_init (RBGenericPlayerPlaylistSource *source)
 {
 }
 
-static GObject *
-impl_constructor (GType type, guint n_construct_properties, GObjectConstructParam *construct_properties)
-{
-	RBGenericPlayerPlaylistSource *source;
-
-	source = RB_GENERIC_PLAYER_PLAYLIST_SOURCE (G_OBJECT_CLASS (rb_generic_player_playlist_source_parent_class) -> constructor (type, n_construct_properties, construct_properties));
-
-	if (load_playlist (source) == FALSE) {
-		rb_debug ("playlist didn't parse; killing the source");
-		if (g_object_is_floating (source))
-			g_object_ref_sink (source);
-		g_object_unref (source);
-		return NULL;
-	}
-
-	return G_OBJECT (source);
-}
-
 static void
 impl_dispose (GObject *object)
 {
@@ -478,41 +556,6 @@ impl_show_popup (RBSource *source)
 	return TRUE;
 }
 
-static gboolean
-impl_can_move_to_trash (RBSource *source)
-{
-	RBGenericPlayerPlaylistSourcePrivate *priv = GET_PRIVATE (source);
-	RBEntryView *songs;
-	GList *sel;
-	gboolean ret;
-
-	songs = rb_source_get_entry_view (source);
-	sel = rb_entry_view_get_selected_entries (songs);
-
-	ret = rb_generic_player_source_can_trash_entries (priv->player_source, sel);
-
-	g_list_foreach (sel, (GFunc) rhythmdb_entry_unref, NULL);
-	g_list_free (sel);
-
-	return ret;
-}
-
-static void
-impl_move_to_trash (RBSource *source)
-{
-	RBGenericPlayerPlaylistSourcePrivate *priv = GET_PRIVATE (source);
-	RBEntryView *songs;
-	GList *sel;
-
-	songs = rb_source_get_entry_view (source);
-	sel = rb_entry_view_get_selected_entries (songs);
-
-	rb_generic_player_source_trash_or_delete_entries (priv->player_source, sel, FALSE);
-
-	g_list_foreach (sel, (GFunc) rhythmdb_entry_unref, NULL);
-	g_list_free (sel);
-}
-
 static void
 rb_generic_player_playlist_source_class_init (RBGenericPlayerPlaylistSourceClass *klass)
 {
@@ -520,15 +563,13 @@ rb_generic_player_playlist_source_class_init (RBGenericPlayerPlaylistSourceClass
 	RBSourceClass *source_class = RB_SOURCE_CLASS (klass);
 	RBPlaylistSourceClass *playlist_class = RB_PLAYLIST_SOURCE_CLASS (klass);
 
-	object_class->constructor = impl_constructor;
 	object_class->dispose = impl_dispose;
 	object_class->finalize = impl_finalize;
 	object_class->get_property = impl_get_property;
 	object_class->set_property = impl_set_property;
 
 	source_class->impl_show_popup = impl_show_popup;
-	source_class->impl_can_move_to_trash = impl_can_move_to_trash;
-	source_class->impl_move_to_trash = impl_move_to_trash;
+	source_class->impl_can_move_to_trash = (RBSourceFeatureFunc) rb_false_function;
 
 	playlist_class->impl_save_contents_to_xml = impl_save_to_xml;
 	playlist_class->impl_mark_dirty = impl_mark_dirty;
