@@ -1,6 +1,4 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
- *
- *  arch-tag: Implementation of generic audio player source object
  *
  *  Copyright (C) 2004 James Livingston  <doclivingston@gmail.com>
  *
@@ -53,6 +51,9 @@
 #include "rb-plugin.h"
 #include "rhythmdb-import-job.h"
 #include "rb-import-errors-source.h"
+#include "rb-builder-helpers.h"
+#include "rb-sync-settings.h"
+#include "rb-missing-plugins.h"
 
 static void impl_constructed (GObject *object);
 static void impl_dispose (GObject *object);
@@ -70,11 +71,9 @@ static void load_songs (RBGenericPlayerSource *source);
 
 static gboolean impl_show_popup (RBSource *source);
 static void impl_delete_thyself (RBSource *source);
-static gboolean impl_can_move_to_trash (RBSource *source);
 static gboolean impl_can_paste (RBSource *source);
 static gboolean impl_can_delete (RBSource *source);
 static void impl_delete (RBSource *source);
-static void impl_move_to_trash (RBSource *source);
 static void impl_get_status (RBSource *source, char **text, char **progress_text, float *progress);
 
 static GList* impl_get_mime_types (RBRemovableMediaSource *source);
@@ -82,13 +81,25 @@ static char* impl_build_dest_uri (RBRemovableMediaSource *source,
 				  RhythmDBEntry *entry,
 				  const char *mimetype,
 				  const char *extension);
+static guint64 impl_get_capacity (RBMediaPlayerSource *source);
+static guint64 impl_get_free_space (RBMediaPlayerSource *source);
+static void impl_get_entries (RBMediaPlayerSource *source, const char *category, GHashTable *map);
+static void impl_delete_entries (RBMediaPlayerSource *source,
+				 GList *entries,
+				 RBMediaPlayerSourceDeleteCallback callback,
+				 gpointer data,
+				 GDestroyNotify destroy_data);
+static void impl_show_properties (RBMediaPlayerSource *source, GtkWidget *info_box, GtkWidget *notebook);
+static void impl_add_playlist (RBMediaPlayerSource *source, char *name, GList *entries);
+static void impl_remove_playlists (RBMediaPlayerSource *source);
 
 static char *default_get_mount_path (RBGenericPlayerSource *source);
 static void default_load_playlists (RBGenericPlayerSource *source);
 static char * default_uri_from_playlist_uri (RBGenericPlayerSource *source,
 					     const char *uri);
 static char * default_uri_to_playlist_uri (RBGenericPlayerSource *source,
-					   const char *uri);
+					   const char *uri,
+					   TotemPlParserType playlist_type);
 
 enum
 {
@@ -120,14 +131,15 @@ typedef struct
 
 } RBGenericPlayerSourcePrivate;
 
-RB_PLUGIN_DEFINE_TYPE(RBGenericPlayerSource, rb_generic_player_source, RB_TYPE_REMOVABLE_MEDIA_SOURCE)
-#define GENERIC_PLAYER_SOURCE_GET_PRIVATE(o)   (G_TYPE_INSTANCE_GET_PRIVATE ((o), RB_TYPE_GENERIC_PLAYER_SOURCE, RBGenericPlayerSourcePrivate))
+RB_PLUGIN_DEFINE_TYPE(RBGenericPlayerSource, rb_generic_player_source, RB_TYPE_MEDIA_PLAYER_SOURCE)
+#define GET_PRIVATE(o)   (G_TYPE_INSTANCE_GET_PRIVATE ((o), RB_TYPE_GENERIC_PLAYER_SOURCE, RBGenericPlayerSourcePrivate))
 
 static void
 rb_generic_player_source_class_init (RBGenericPlayerSourceClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 	RBSourceClass *source_class = RB_SOURCE_CLASS (klass);
+	RBMediaPlayerSourceClass *mps_class = RB_MEDIA_PLAYER_SOURCE_CLASS (klass);
 	RBRemovableMediaSourceClass *rms_class = RB_REMOVABLE_MEDIA_SOURCE_CLASS (klass);
 
 	object_class->set_property = impl_set_property;
@@ -140,11 +152,18 @@ rb_generic_player_source_class_init (RBGenericPlayerSourceClass *klass)
 	source_class->impl_delete_thyself = impl_delete_thyself;
 	source_class->impl_can_delete = impl_can_delete;
 	source_class->impl_delete = impl_delete;
-	source_class->impl_can_move_to_trash = impl_can_move_to_trash;
-	source_class->impl_move_to_trash = impl_move_to_trash;
+	source_class->impl_can_move_to_trash = (RBSourceFeatureFunc) rb_false_function;
 	source_class->impl_can_paste = impl_can_paste;
 	source_class->impl_get_status = impl_get_status;
 
+	mps_class->impl_get_entries = impl_get_entries;
+	mps_class->impl_get_capacity = impl_get_capacity;
+	mps_class->impl_get_free_space = impl_get_free_space;
+	mps_class->impl_delete_entries = impl_delete_entries;
+	mps_class->impl_show_properties = impl_show_properties;
+	mps_class->impl_add_playlist = impl_add_playlist;
+	mps_class->impl_remove_playlists = impl_remove_playlists;
+
 	rms_class->impl_build_dest_uri = impl_build_dest_uri;
 	rms_class->impl_get_mime_types = impl_get_mime_types;
 	rms_class->impl_should_paste = rb_removable_media_source_should_paste_no_duplicate;
@@ -192,6 +211,7 @@ impl_constructed (GObject *object)
 {
 	RBGenericPlayerSource *source;
 	RBGenericPlayerSourcePrivate *priv;
+	RhythmDBEntryType entry_type;
 	GMount *mount;
 	char **playlist_formats;
 	char *mount_name;
@@ -203,13 +223,19 @@ impl_constructed (GObject *object)
 	RB_CHAIN_GOBJECT_METHOD (rb_generic_player_source_parent_class, constructed, object);
 	source = RB_GENERIC_PLAYER_SOURCE (object);
 
-	priv = GENERIC_PLAYER_SOURCE_GET_PRIVATE (source);
+	priv = GET_PRIVATE (source);
 
-	g_object_get (source, "shell", &shell, NULL);
+	g_object_get (source,
+		      "shell", &shell,
+		      "entry-type", &entry_type,
+		      NULL);
 
 	g_object_get (shell, "db", &priv->db, NULL);
 	
-	priv->import_errors = rb_import_errors_source_new (shell, priv->error_type);
+	priv->import_errors = rb_import_errors_source_new (shell,
+							   priv->error_type,
+							   entry_type,
+							   priv->ignore_type);
 
 	g_object_unref (shell);
 
@@ -234,21 +260,19 @@ impl_constructed (GObject *object)
 
 	g_object_get (priv->device_info, "playlist-formats", &playlist_formats, NULL);
 	if (playlist_formats != NULL && g_strv_length (playlist_formats) > 0) {
-		RhythmDBEntryType entry_type;
-
-		g_object_get (source, "entry-type", &entry_type, NULL);
 		entry_type->has_playlists = TRUE;
-		g_boxed_free (RHYTHMDB_TYPE_ENTRY_TYPE, entry_type);
 	}
 	g_strfreev (playlist_formats);
+	g_boxed_free (RHYTHMDB_TYPE_ENTRY_TYPE, entry_type);
 
+        rb_media_player_source_load (RB_MEDIA_PLAYER_SOURCE (source));
 	load_songs (source);
 }
 
 static void
 impl_set_property (GObject *object, guint prop_id, const GValue *value, GParamSpec *pspec)
 {
-	RBGenericPlayerSourcePrivate *priv = GENERIC_PLAYER_SOURCE_GET_PRIVATE (object);
+	RBGenericPlayerSourcePrivate *priv = GET_PRIVATE (object);
 
 	switch (prop_id) {
 	case PROP_IGNORE_ENTRY_TYPE:
@@ -269,7 +293,7 @@ impl_set_property (GObject *object, guint prop_id, const GValue *value, GParamSp
 static void
 impl_get_property (GObject *object, guint prop_id, GValue *value, GParamSpec *pspec)
 {
-	RBGenericPlayerSourcePrivate *priv = GENERIC_PLAYER_SOURCE_GET_PRIVATE (object);
+	RBGenericPlayerSourcePrivate *priv = GET_PRIVATE (object);
 
 	switch (prop_id) {
 	case PROP_IGNORE_ENTRY_TYPE:
@@ -290,7 +314,7 @@ impl_get_property (GObject *object, guint prop_id, GValue *value, GParamSpec *ps
 static void
 impl_dispose (GObject *object)
 {
-	RBGenericPlayerSourcePrivate *priv = GENERIC_PLAYER_SOURCE_GET_PRIVATE (object);
+	RBGenericPlayerSourcePrivate *priv = GET_PRIVATE (object);
 
 	if (priv->load_playlists_id != 0) {
 		g_source_remove (priv->load_playlists_id);
@@ -333,11 +357,11 @@ impl_finalize (GObject *object)
 	RBGenericPlayerSourcePrivate *priv;
 
 	g_return_if_fail (RB_IS_GENERIC_PLAYER_SOURCE (object));
-	priv = GENERIC_PLAYER_SOURCE_GET_PRIVATE (object);
+	priv = GET_PRIVATE (object);
 }
 
 RBRemovableMediaSource *
-rb_generic_player_source_new (RBShell *shell, GMount *mount, MPIDDevice *device_info)
+rb_generic_player_source_new (RBPlugin *plugin, RBShell *shell, GMount *mount, MPIDDevice *device_info)
 {
 	RBGenericPlayerSource *source;
 	RhythmDBEntryType entry_type;
@@ -370,6 +394,7 @@ rb_generic_player_source_new (RBShell *shell, GMount *mount, MPIDDevice *device_
 	g_free (path);
 
 	source = RB_GENERIC_PLAYER_SOURCE (g_object_new (RB_TYPE_GENERIC_PLAYER_SOURCE,
+							 "plugin", plugin,
 							 "entry-type", entry_type,
 							 "ignore-entry-type", ignore_type,
 							 "error-entry-type", error_type,
@@ -389,7 +414,7 @@ impl_delete_thyself (RBSource *source)
 {
 	GList *pl;
 	GList *p;
-	RBGenericPlayerSourcePrivate *priv = GENERIC_PLAYER_SOURCE_GET_PRIVATE (source);
+	RBGenericPlayerSourcePrivate *priv = GET_PRIVATE (source);
 
 	/* take a copy of the list first, as playlist_deleted_cb modifies priv->playlists */
 	pl = g_list_copy (priv->playlists);
@@ -413,7 +438,7 @@ static void
 import_complete_cb (RhythmDBImportJob *job, int total, RBGenericPlayerSource *source)
 {
 	RBGenericPlayerSourceClass *klass = RB_GENERIC_PLAYER_SOURCE_GET_CLASS (source);
-	RBGenericPlayerSourcePrivate *priv = GENERIC_PLAYER_SOURCE_GET_PRIVATE (source);
+	RBGenericPlayerSourcePrivate *priv = GET_PRIVATE (source);
 	RBShell *shell;
 
 	GDK_THREADS_ENTER ();
@@ -442,13 +467,15 @@ import_status_changed_cb (RhythmDBImportJob *job, int total, int imported, RBGen
 static void
 load_songs (RBGenericPlayerSource *source)
 {
-	RBGenericPlayerSourcePrivate *priv = GENERIC_PLAYER_SOURCE_GET_PRIVATE (source);
+	RBGenericPlayerSourcePrivate *priv = GET_PRIVATE (source);
 	RhythmDBEntryType entry_type;
 	char **audio_folders;
 	char *mount_path;
 
 	mount_path = rb_generic_player_source_get_mount_path (source);
-	g_object_get (source, "entry-type", &entry_type, NULL);
+	g_object_get (source,
+		      "entry-type", &entry_type,
+		      NULL);
 
 	/* if we have a set of folders on the device containing audio files,
 	 * load only those folders, otherwise add the whole volume.
@@ -491,7 +518,7 @@ rb_generic_player_source_get_mount_path (RBGenericPlayerSource *source)
 static char *
 default_get_mount_path (RBGenericPlayerSource *source)
 {
-	RBGenericPlayerSourcePrivate *priv = GENERIC_PLAYER_SOURCE_GET_PRIVATE (source);
+	RBGenericPlayerSourcePrivate *priv = GET_PRIVATE (source);
 
 	if (priv->mount_path == NULL) {
 		GMount *mount;
@@ -540,86 +567,17 @@ impl_show_popup (RBSource *source)
 	return TRUE;
 }
 
-gboolean
-rb_generic_player_source_can_trash_entries (RBGenericPlayerSource *source,
-					    GList *entries)
-{
-	RBGenericPlayerSourcePrivate *priv = GENERIC_PLAYER_SOURCE_GET_PRIVATE (source);
-	GList *tem;
-	gboolean ret;
-
-	if (priv->read_only != FALSE)
-		return FALSE;
-
-	ret = FALSE;
-
-	for (tem = entries; tem != NULL; tem = tem->next) {
-		RhythmDBEntry *entry;
-		const char *uri;
-		GFile *file;
-		GFileInfo *info;
-
-		entry = tem->data;
-		uri = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_LOCATION);
-		file = g_file_new_for_uri (uri);
-		info = g_file_query_info (file,
-					  G_FILE_ATTRIBUTE_ACCESS_CAN_TRASH,
-					  G_FILE_QUERY_INFO_NONE,
-					  NULL,
-					  NULL);
-		g_object_unref (file);
-		if (info == NULL) {
-			ret = FALSE;
-			break;
-		}
-		ret = g_file_info_get_attribute_boolean (info, G_FILE_ATTRIBUTE_ACCESS_CAN_TRASH);
-		g_object_unref (info);
-		if (ret == FALSE)
-			break;
-	}
-
-	return ret;
-}
-
-static gboolean
-impl_can_move_to_trash (RBSource *source)
-{
-	RBEntryView *view;
-	GList *sel;
-	gboolean ret;
-
-
-	view = rb_source_get_entry_view (source);
-	sel = rb_entry_view_get_selected_entries (view);
-
-	ret = rb_generic_player_source_can_trash_entries (RB_GENERIC_PLAYER_SOURCE (source),
-							  sel);
-
-	g_list_foreach (sel, (GFunc)rhythmdb_entry_unref, NULL);
-	g_list_free (sel);
-
-	return ret;
-}
-
 static void
 impl_get_status (RBSource *source, char **text, char **progress_text, float *progress)
 {
-	RBGenericPlayerSourcePrivate *priv = GENERIC_PLAYER_SOURCE_GET_PRIVATE (source);
+	RBGenericPlayerSourcePrivate *priv = GET_PRIVATE (source);
 
 	/* get default status text first */
 	RB_SOURCE_CLASS (rb_generic_player_source_parent_class)->impl_get_status (source, text, progress_text, progress);
 
 	/* override with bits of import status */
 	if (priv->import_job != NULL) {
-		int total;
-		int imported;
-
-		total = rhythmdb_import_job_get_total (priv->import_job);
-		imported = rhythmdb_import_job_get_imported (priv->import_job);
-
-		g_free (*progress_text);
-		*progress_text = g_strdup_printf (_("Importing (%d/%d)"), imported, total);
-		*progress = ((float)imported / (float)total);
+		_rb_source_set_import_status (source, priv->import_job, progress_text, progress);
 	}
 }
 
@@ -628,7 +586,7 @@ impl_get_status (RBSource *source, char **text, char **progress_text, float *pro
 static void
 playlist_deleted_cb (RBSource *playlist, RBGenericPlayerSource *source)
 {
-	RBGenericPlayerSourcePrivate *priv = GENERIC_PLAYER_SOURCE_GET_PRIVATE (source);
+	RBGenericPlayerSourcePrivate *priv = GET_PRIVATE (source);
 	GList *p;
 
 	p = g_list_find (priv->playlists, playlist);
@@ -643,7 +601,7 @@ rb_generic_player_source_add_playlist (RBGenericPlayerSource *source,
 				       RBShell *shell,
 				       RBSource *playlist)
 {
-	RBGenericPlayerSourcePrivate *priv = GENERIC_PLAYER_SOURCE_GET_PRIVATE (source);
+	RBGenericPlayerSourcePrivate *priv = GET_PRIVATE (source);
 	g_object_ref (playlist);
 	priv->playlists = g_list_prepend (priv->playlists, playlist);
 
@@ -673,19 +631,30 @@ default_uri_from_playlist_uri (RBGenericPlayerSource *source, const char *uri)
 }
 
 static char *
-default_uri_to_playlist_uri (RBGenericPlayerSource *source, const char *uri)
+default_uri_to_playlist_uri (RBGenericPlayerSource *source, const char *uri, TotemPlParserType playlist_type)
 {
 	char *mount_uri;
 	char *playlist_uri;
 
-	mount_uri = rb_generic_player_source_get_mount_path (source);
-	if (g_str_has_prefix (uri, mount_uri) == FALSE) {
-		rb_debug ("uri %s is not under device mount uri %s", uri, mount_uri);
-		return NULL;
-	}
+	switch (playlist_type) {
+	case TOTEM_PL_PARSER_IRIVER_PLA:
+		/* we need absolute paths within the device filesystem for this format */
+		mount_uri = rb_generic_player_source_get_mount_path (source);
+		if (g_str_has_prefix (uri, mount_uri) == FALSE) {
+			rb_debug ("uri %s is not under device mount uri %s", uri, mount_uri);
+			return NULL;
+		}
 
-	playlist_uri = g_strdup_printf ("file://%s", uri + strlen (mount_uri));
-	return playlist_uri;
+		playlist_uri = g_strdup_printf ("file://%s", uri + strlen (mount_uri));
+		return playlist_uri;
+
+	case TOTEM_PL_PARSER_M3U_DOS:
+	case TOTEM_PL_PARSER_M3U:
+	case TOTEM_PL_PARSER_PLS:
+	default:
+		/* leave the URI as-is, so we end up with relative paths in the playlist file */
+		return g_strdup (uri);
+	}
 }
 
 char *
@@ -697,11 +666,11 @@ rb_generic_player_source_uri_from_playlist_uri (RBGenericPlayerSource *source, c
 }
 
 char *
-rb_generic_player_source_uri_to_playlist_uri (RBGenericPlayerSource *source, const char *uri)
+rb_generic_player_source_uri_to_playlist_uri (RBGenericPlayerSource *source, const char *uri, TotemPlParserType playlist_type)
 {
 	RBGenericPlayerSourceClass *klass = RB_GENERIC_PLAYER_SOURCE_GET_CLASS (source);
 
-	return klass->impl_uri_to_playlist_uri (source, uri);
+	return klass->impl_uri_to_playlist_uri (source, uri, playlist_type);
 }
 
 static void
@@ -746,7 +715,7 @@ visit_playlist_dirs (GFile *file,
 	char *uri;
 	RhythmDBEntry *entry;
 	RhythmDBEntryType entry_type;
-	RBGenericPlayerSourcePrivate *priv = GENERIC_PLAYER_SOURCE_GET_PRIVATE (source);
+	RBGenericPlayerSourcePrivate *priv = GET_PRIVATE (source);
 
 	if (dir) {
 		return TRUE;
@@ -790,7 +759,7 @@ visit_playlist_dirs (GFile *file,
 static void
 default_load_playlists (RBGenericPlayerSource *source)
 {
-	RBGenericPlayerSourcePrivate *priv = GENERIC_PLAYER_SOURCE_GET_PRIVATE (source);
+	RBGenericPlayerSourcePrivate *priv = GET_PRIVATE (source);
 	char *mount_path;
 	char *playlist_path;
 	char *full_playlist_path;
@@ -798,7 +767,7 @@ default_load_playlists (RBGenericPlayerSource *source)
 
 	mount_path = rb_generic_player_source_get_mount_path (source);
 
-	g_object_get (priv->device_info, "playlist-path", &playlist_path, NULL);
+	playlist_path = rb_generic_player_source_get_playlist_path (RB_GENERIC_PLAYER_SOURCE (source));
 	if (playlist_path) {
 
 		/* If the device only supports a single playlist, just load that */
@@ -815,9 +784,6 @@ default_load_playlists (RBGenericPlayerSource *source)
 		}
 
 		/* Otherwise, limit the search to the device's playlist folder */
-		if (g_str_has_suffix (playlist_path, "/%File")) {
-			playlist_path[strlen (playlist_path) - strlen("/%File")] = '\0';
-		}
 		full_playlist_path = rb_uri_append_path (mount_path, playlist_path);
 		rb_debug ("constructed playlist search path %s", full_playlist_path);
 	} else {
@@ -843,7 +809,7 @@ default_load_playlists (RBGenericPlayerSource *source)
 static gboolean
 impl_can_paste (RBSource *source)
 {
-	RBGenericPlayerSourcePrivate *priv = GENERIC_PLAYER_SOURCE_GET_PRIVATE (source);
+	RBGenericPlayerSourcePrivate *priv = GET_PRIVATE (source);
 
 	return (priv->read_only == FALSE);
 }
@@ -851,7 +817,7 @@ impl_can_paste (RBSource *source)
 static gboolean
 impl_can_delete (RBSource *source)
 {
-	RBGenericPlayerSourcePrivate *priv = GENERIC_PLAYER_SOURCE_GET_PRIVATE (source);
+	RBGenericPlayerSourcePrivate *priv = GET_PRIVATE (source);
 
 	return (priv->read_only == FALSE);
 }
@@ -859,7 +825,7 @@ impl_can_delete (RBSource *source)
 static gboolean
 can_delete_directory (RBGenericPlayerSource *source, GFile *dir)
 {
-	RBGenericPlayerSourcePrivate *priv = GENERIC_PLAYER_SOURCE_GET_PRIVATE (source);
+	RBGenericPlayerSourcePrivate *priv = GET_PRIVATE (source);
 	gboolean result;
 	GMount *mount;
 	GFile *root;
@@ -900,11 +866,9 @@ can_delete_directory (RBGenericPlayerSource *source, GFile *dir)
 }
 
 void
-rb_generic_player_source_trash_or_delete_entries (RBGenericPlayerSource *source,
-						  GList *entries,
-						  gboolean _delete)
+rb_generic_player_source_delete_entries (RBGenericPlayerSource *source, GList *entries)
 {
-	RBGenericPlayerSourcePrivate *priv = GENERIC_PLAYER_SOURCE_GET_PRIVATE (source);
+	RBGenericPlayerSourcePrivate *priv = GET_PRIVATE (source);
 	GList *tem;
 
 	if (priv->read_only != FALSE)
@@ -919,10 +883,7 @@ rb_generic_player_source_trash_or_delete_entries (RBGenericPlayerSource *source,
 		entry = tem->data;
 		uri = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_LOCATION);
 		file = g_file_new_for_uri (uri);
-		if (_delete)
-			g_file_delete (file, NULL, NULL);
-		else
-			g_file_trash (file, NULL, NULL);
+		g_file_delete (file, NULL, NULL);
 
 		/* now walk up the directory structure and delete empty dirs
 		 * until we reach the root or one of the device's audio folders.
@@ -957,22 +918,6 @@ rb_generic_player_source_trash_or_delete_entries (RBGenericPlayerSource *source,
 	rhythmdb_commit (priv->db);
 }
 
-static void
-impl_move_to_trash (RBSource *source)
-{
-	RBEntryView *view;
-	GList *sel;
-
-	view = rb_source_get_entry_view (source);
-	sel = rb_entry_view_get_selected_entries (view);
-
-	rb_generic_player_source_trash_or_delete_entries (RB_GENERIC_PLAYER_SOURCE (source),
-							  sel,
-							  FALSE);
-	g_list_foreach (sel, (GFunc)rhythmdb_entry_unref, NULL);
-	g_list_free (sel);
-}
-
 static void
 impl_delete (RBSource *source)
 {
@@ -982,9 +927,7 @@ impl_delete (RBSource *source)
 	view = rb_source_get_entry_view (source);
 	sel = rb_entry_view_get_selected_entries (view);
 
-	rb_generic_player_source_trash_or_delete_entries (RB_GENERIC_PLAYER_SOURCE (source),
-							  sel,
-							  TRUE);
+	rb_generic_player_source_delete_entries (RB_GENERIC_PLAYER_SOURCE (source), sel);
 	g_list_foreach (sel, (GFunc)rhythmdb_entry_unref, NULL);
 	g_list_free (sel);
 }
@@ -1009,7 +952,7 @@ sanitize_path (const char *str)
 static GList *
 impl_get_mime_types (RBRemovableMediaSource *source)
 {
-	RBGenericPlayerSourcePrivate *priv = GENERIC_PLAYER_SOURCE_GET_PRIVATE (source);
+	RBGenericPlayerSourcePrivate *priv = GET_PRIVATE (source);
 	GList *list = NULL;
 	char **output_formats;
 	char **mime;
@@ -1028,7 +971,8 @@ impl_build_dest_uri (RBRemovableMediaSource *source,
 		     const char *mimetype,
 		     const char *extension)
 {
-	RBGenericPlayerSourcePrivate *priv = GENERIC_PLAYER_SOURCE_GET_PRIVATE (source);
+	RBGenericPlayerSourcePrivate *priv = GET_PRIVATE (source);
+	const char *in_artist;
 	char *artist, *album, *title;
 	gulong track_number, disc_number;
 	const char *folders;
@@ -1047,7 +991,11 @@ impl_build_dest_uri (RBRemovableMediaSource *source,
 		ext = g_strdup ("");
 	}
 
-	artist = sanitize_path (rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_ARTIST));
+	in_artist = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_ALBUM_ARTIST);
+	if (in_artist[0] == '\0') {
+		in_artist = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_ARTIST);
+	}
+	artist = sanitize_path (in_artist);
 	album = sanitize_path (rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_ALBUM));
 	title = sanitize_path (rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_TITLE));
 
@@ -1140,7 +1088,7 @@ strv_contains (char **strv, const char *s)
 void
 rb_generic_player_source_set_supported_formats (RBGenericPlayerSource *source, TotemPlParser *parser)
 {
-	RBGenericPlayerSourcePrivate *priv = GENERIC_PLAYER_SOURCE_GET_PRIVATE (source);
+	RBGenericPlayerSourcePrivate *priv = GET_PRIVATE (source);
 	char **playlist_formats;
 	const char *check[] = { "audio/x-mpegurl", "audio/x-scpls", "audio/x-iriver-pla" };
 
@@ -1148,7 +1096,7 @@ rb_generic_player_source_set_supported_formats (RBGenericPlayerSource *source, T
 	if (playlist_formats != NULL && g_strv_length (playlist_formats) > 0) {
 		int i;
 		for (i = 0; i < G_N_ELEMENTS (check); i++) {
-			if (strv_contains (playlist_formats, check[i])) {
+			if (strv_contains (playlist_formats, check[i]) == FALSE) {
 				totem_pl_parser_add_ignored_mimetype (parser, check[i]);
 			}
 		}
@@ -1161,7 +1109,7 @@ rb_generic_player_source_set_supported_formats (RBGenericPlayerSource *source, T
 TotemPlParserType
 rb_generic_player_source_get_playlist_format (RBGenericPlayerSource *source)
 {
-	RBGenericPlayerSourcePrivate *priv = GENERIC_PLAYER_SOURCE_GET_PRIVATE (source);
+	RBGenericPlayerSourcePrivate *priv = GET_PRIVATE (source);
 	TotemPlParserType result;
 	char **playlist_formats;
 
@@ -1170,7 +1118,7 @@ rb_generic_player_source_get_playlist_format (RBGenericPlayerSource *source)
 	if (playlist_formats == NULL || g_strv_length (playlist_formats) == 0 || strv_contains (playlist_formats, "audio/x-scpls")) {
 		result = TOTEM_PL_PARSER_PLS;
 	} else if (strv_contains (playlist_formats, "audio/x-mpegurl")) {
-		result = TOTEM_PL_PARSER_M3U;
+		result = TOTEM_PL_PARSER_M3U_DOS;
 	} else if (strv_contains (playlist_formats, "audio/x-iriver-pla")) {
 		result = TOTEM_PL_PARSER_IRIVER_PLA;
 	} else {
@@ -1185,10 +1133,245 @@ rb_generic_player_source_get_playlist_format (RBGenericPlayerSource *source)
 char *
 rb_generic_player_source_get_playlist_path (RBGenericPlayerSource *source)
 {
-	RBGenericPlayerSourcePrivate *priv = GENERIC_PLAYER_SOURCE_GET_PRIVATE (source);
+	RBGenericPlayerSourcePrivate *priv = GET_PRIVATE (source);
 	char *path;
 
 	g_object_get (priv->device_info, "playlist-path", &path, NULL);
+	if (g_str_has_suffix (path, "%File")) {
+		path[strlen (path) - strlen("%File")] = '\0';
+	}
 	return path;
 }
 
+static guint64
+get_fs_property (RBGenericPlayerSource *source, const char *attr)
+{
+	char *mountpoint;
+	GFile *root;
+	GFileInfo *info;
+	guint64 value = 0;
+
+	mountpoint = rb_generic_player_source_get_mount_path (source);
+	root = g_file_new_for_uri (mountpoint);
+	g_free (mountpoint);
+
+	info = g_file_query_filesystem_info (root, attr, NULL, NULL);
+	g_object_unref (root);
+	if (info != NULL) {
+		if (g_file_info_has_attribute (info, attr)) {
+			value = g_file_info_get_attribute_uint64 (info, attr);
+		}
+		g_object_unref (info);
+	}
+	return value;
+}
+static guint64
+impl_get_capacity (RBMediaPlayerSource *source)
+{
+	return get_fs_property (RB_GENERIC_PLAYER_SOURCE (source), G_FILE_ATTRIBUTE_FILESYSTEM_SIZE);
+}
+
+static guint64
+impl_get_free_space (RBMediaPlayerSource *source)
+{
+	return get_fs_property (RB_GENERIC_PLAYER_SOURCE (source), G_FILE_ATTRIBUTE_FILESYSTEM_FREE);
+}
+
+static void
+impl_get_entries (RBMediaPlayerSource *source,
+		  const char *category,
+		  GHashTable *map)
+{
+	RhythmDBQueryModel *model;
+	GtkTreeIter iter;
+	gboolean podcast;
+
+	/* we don't have anything else to distinguish podcasts from regular
+	 * tracks, so just use the genre.
+	 */
+	podcast = (g_str_equal (category, SYNC_CATEGORY_PODCAST));
+
+	g_object_get (source, "base-query-model", &model, NULL);
+	if (gtk_tree_model_get_iter_first (GTK_TREE_MODEL (model), &iter) == FALSE) {
+		g_object_unref (model);
+		return;
+	}
+
+	do {
+		RhythmDBEntry *entry;
+		const char *genre;
+		entry = rhythmdb_query_model_iter_to_entry (model, &iter);
+		genre = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_GENRE);
+		if (g_str_equal (genre, "Podcast") == podcast) {
+			_rb_media_player_source_add_to_map (map, entry);
+		}
+	} while (gtk_tree_model_iter_next (GTK_TREE_MODEL (model), &iter));
+
+	g_object_unref (model);
+}
+
+static void
+impl_delete_entries (RBMediaPlayerSource *source,
+		     GList *entries,
+		     RBMediaPlayerSourceDeleteCallback callback,
+		     gpointer callback_data,
+		     GDestroyNotify destroy_data)
+{
+	rb_generic_player_source_delete_entries (RB_GENERIC_PLAYER_SOURCE (source), entries);
+
+	if (callback) {
+		callback (source, callback_data);
+	}
+	if (destroy_data) {
+		destroy_data (callback_data);
+	}
+}
+
+
+static void
+impl_show_properties (RBMediaPlayerSource *source, GtkWidget *info_box, GtkWidget *notebook)
+{
+	RBGenericPlayerSourcePrivate *priv = GET_PRIVATE (source);
+	RhythmDBQueryModel *model;
+	GtkBuilder *builder;
+	GtkWidget *widget;
+	GString *str;
+	char *device_name;
+	char *builder_file;
+	char *vendor_name;
+	char *model_name;
+	char *serial_id;
+	RBPlugin *plugin;
+	char *text;
+	GList *output_formats;
+	GList *t;
+
+	g_object_get (source, "plugin", &plugin, NULL);
+	builder_file = rb_plugin_find_file (plugin, "generic-player-info.ui");
+	g_object_unref (plugin);
+
+	if (builder_file == NULL) {
+		g_warning ("Couldn't find generic-player-info.ui");
+		return;
+	}
+
+	builder = rb_builder_load (builder_file, NULL);
+	g_free (builder_file);
+
+	if (builder == NULL) {
+		rb_debug ("Couldn't load generic-player-info.ui");
+		return;
+	}
+
+	/* 'basic' tab stuff */
+
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "generic-player-basic-info"));
+	gtk_box_pack_start (GTK_BOX (info_box), widget, TRUE, TRUE, 0);
+
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "entry-device-name"));
+	g_object_get (source, "name", &device_name, NULL);
+	gtk_entry_set_text (GTK_ENTRY (widget), device_name);
+	g_free (device_name);
+	/* don't think we can support this..
+	g_signal_connect (widget, "focus-out-event",
+			  (GCallback)rb_mtp_source_name_changed_cb, source);
+			  */
+
+	g_object_get (source, "base-query-model", &model, NULL);
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "num-tracks"));
+	text = g_strdup_printf ("%d", gtk_tree_model_iter_n_children (GTK_TREE_MODEL (model), NULL));
+	gtk_label_set_text (GTK_LABEL (widget), text);
+	g_free (text);
+	g_object_unref (model);
+
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "num-playlists"));
+	text = g_strdup_printf ("%d", g_list_length (priv->playlists));
+	gtk_label_set_text (GTK_LABEL (widget), text);
+	g_free (text);
+
+	/* 'advanced' tab stuff */
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "generic-player-advanced-tab"));
+	gtk_notebook_append_page (GTK_NOTEBOOK (notebook), widget, gtk_label_new (_("Advanced")));
+
+	g_object_get (priv->device_info,
+		      "model", &model_name,
+		      "vendor", &vendor_name,
+		      "serial", &serial_id,
+		      NULL);
+
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "label-model-value"));
+	gtk_label_set_text (GTK_LABEL (widget), model_name);
+
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "label-manufacturer-value"));
+	gtk_label_set_text (GTK_LABEL (widget), vendor_name);
+
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "label-serial-number-value"));
+	gtk_label_set_text (GTK_LABEL (widget), serial_id);
+
+	g_free (model_name);
+	g_free (vendor_name);
+	g_free (serial_id);
+
+	str = g_string_new ("");
+	output_formats = rb_removable_media_source_get_format_descriptions (RB_REMOVABLE_MEDIA_SOURCE (source));
+	for (t = output_formats; t != NULL; t = t->next) {
+		if (t != output_formats) {
+			g_string_append (str, "\n");
+		}
+		g_string_append (str, t->data);
+	}
+	rb_list_deep_free (output_formats);
+
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "audio-format-list"));
+	gtk_label_set_text (GTK_LABEL (widget), str->str);
+	g_string_free (str, TRUE);
+
+	g_object_unref (builder);
+}
+
+static void
+impl_add_playlist (RBMediaPlayerSource *source, char *name, GList *entries)
+{
+	RBSource *playlist;
+	RhythmDBEntryType entry_type;
+	RBShell *shell;
+	GList *i;
+
+	g_object_get (source,
+		      "shell", &shell,
+		      "entry-type", &entry_type,
+		      NULL);
+
+	playlist = rb_generic_player_playlist_source_new (shell, RB_GENERIC_PLAYER_SOURCE (source), NULL, NULL, entry_type);
+	g_boxed_free (RHYTHMDB_TYPE_ENTRY_TYPE, entry_type);
+
+	rb_generic_player_source_add_playlist (RB_GENERIC_PLAYER_SOURCE (source),
+					       shell,
+					       playlist);
+	g_object_set (playlist, "name", name, NULL);
+
+	for (i = entries; i != NULL; i = i->next) {
+		rb_static_playlist_source_add_entry (RB_STATIC_PLAYLIST_SOURCE (playlist),
+						     i->data,
+						     -1);
+	}
+
+	g_object_unref (shell);
+}
+
+static void
+impl_remove_playlists (RBMediaPlayerSource *source)
+{
+	RBGenericPlayerSourcePrivate *priv = GET_PRIVATE (source);
+	GList *playlists;
+	GList *t;
+
+	playlists = g_list_copy (priv->playlists);
+	for (t = playlists; t != NULL; t = t->next) {
+		RBGenericPlayerPlaylistSource *p = RB_GENERIC_PLAYER_PLAYLIST_SOURCE (t->data);
+		rb_generic_player_playlist_delete_from_player (p);
+		rb_source_delete_thyself (RB_SOURCE (p));
+	}
+
+	g_list_free (playlists);
+}
