@@ -46,6 +46,9 @@ typedef struct {
 		OPEN_DEVICE = 1,
 		CLOSE_DEVICE,
 		SET_DEVICE_NAME,
+		THREAD_CALLBACK,
+
+		CREATE_FOLDER,
 
 		ADD_TO_ALBUM,
 		REMOVE_FROM_ALBUM,
@@ -65,6 +68,7 @@ typedef struct {
 	char *filename;
 	GdkPixbuf *image;
 	char *name;
+	char **path;
 
 	gpointer callback;
 	gpointer user_data;
@@ -78,6 +82,9 @@ task_name (RBMtpThreadTask *task)
 	case OPEN_DEVICE:	return g_strdup ("open device");
 	case CLOSE_DEVICE:	return g_strdup ("close device");
 	case SET_DEVICE_NAME:	return g_strdup_printf ("set device name to %s", task->name);
+	case THREAD_CALLBACK:	return g_strdup ("thread callback");
+
+	case CREATE_FOLDER:	return g_strdup_printf ("create folder %s", task->path[g_strv_length (task->path)-1]);
 
 	case ADD_TO_ALBUM:	return g_strdup_printf ("add track %u to album %s", task->track_id, task->album);
 	case REMOVE_FROM_ALBUM:	return g_strdup_printf ("remove track %u from album %s", task->track_id, task->album);
@@ -112,6 +119,7 @@ destroy_task (RBMtpThreadTask *task)
 	g_free (task->album);
 	g_free (task->filename);
 	g_free (task->name);
+	g_strfreev (task->path);
 
 	if (task->image) {
 		g_object_unref (task->image);
@@ -139,19 +147,98 @@ static void
 open_device (RBMtpThread *thread, RBMtpThreadTask *task)
 {
 	RBMtpOpenCallback cb = task->callback;
+	int retry;
 
 	/* open the device */
-	thread->device = LIBMTP_Open_Raw_Device (task->raw_device);
-	if (thread->device == NULL) {
-		/* hm. any error information we can give here? */
-		/* maybe retry a few times since apparently that can help? */
-		cb (NULL, task->user_data);
-		return;
+	rb_debug ("attempting to open device");
+	for (retry = 0; retry < 5; retry++) {
+		if (retry > 0) {
+			/* sleep a while before trying again */
+			g_usleep (G_USEC_PER_SEC);
+		}
+
+		thread->device = LIBMTP_Open_Raw_Device (task->raw_device);
+		if (thread->device != NULL) {
+			break;
+		}
+
+		rb_debug ("attempt %d failed..", retry+1);
 	}
 
 	cb (thread->device, task->user_data);
 }
 
+static void
+create_folder (RBMtpThread *thread, RBMtpThreadTask *task)
+{
+	RBMtpCreateFolderCallback cb = task->callback;
+	LIBMTP_folder_t *folders;
+	LIBMTP_folder_t *f;
+	LIBMTP_folder_t *target = NULL;
+	uint32_t folder_id;
+	uint32_t storage_id;
+	int i;
+
+	folders = LIBMTP_Get_Folder_List (thread->device);
+	if (folders == NULL) {
+		rb_debug ("unable to get folder list");
+		rb_mtp_thread_report_errors (thread, FALSE);
+		cb (0, task->user_data);
+		return;
+	}
+
+	/* first find the default music folder */
+	f = LIBMTP_Find_Folder (folders, thread->device->default_music_folder);
+	if (f == NULL) {
+		rb_debug ("unable to find default music folder");
+		cb (0, task->user_data);
+		LIBMTP_destroy_folder_t (folders);
+		return;
+	}
+	storage_id = f->storage_id;
+	folder_id = f->folder_id;
+
+	/* descend through the folder tree, following the path */
+	i = 0;
+	while (task->path[i] != NULL) {
+
+		/* look for a folder at this level with the same name as the
+		 * next path component
+		 */
+		target = f->child;
+		while (target != NULL) {
+			if (g_strcmp0 (target->name, task->path[i]) == 0) {
+				rb_debug ("found path element %d: %s", i, target->name);
+				break;
+			}
+			target = target->sibling;
+		}
+
+		if (target == NULL) {
+			rb_debug ("path element %d (%s) not found", i, task->path[i]);
+			break;
+		}
+		f = target;
+		folder_id = f->folder_id;
+		i++;
+	}
+
+	/* now create any path elements that don't already exist */
+	while (task->path[i] != NULL) {
+		folder_id = LIBMTP_Create_Folder (thread->device, task->path[i], folder_id, storage_id);
+		if (folder_id == 0) {
+			rb_debug ("couldn't create path element %d: %s", i, task->path[i]);
+			rb_mtp_thread_report_errors (thread, FALSE);
+			break;
+		}
+		rb_debug ("created path element %d: %s with folder ID %u", i, task->path[i], folder_id);
+		i++;
+	}
+
+	cb (folder_id, task->user_data);
+	LIBMTP_destroy_folder_t (folders);
+}
+
 static LIBMTP_album_t *
 add_track_to_album (RBMtpThread *thread, const char *album_name, uint32_t track_id, uint32_t storage_id, gboolean *new_album)
 {
@@ -507,9 +594,15 @@ upload_track (RBMtpThread *thread, RBMtpThreadTask *task)
 	if (LIBMTP_Send_Track_From_File (thread->device, task->filename, task->track, NULL, NULL)) {
 		stack = LIBMTP_Get_Errorstack (thread->device);
 		rb_debug ("unable to send track: %s", stack->error_text);
-		error = g_error_new (RB_MTP_THREAD_ERROR, RB_MTP_THREAD_ERROR_SEND_TRACK,
-				     _("Unable to send file to MTP device: %s"),
-				     stack->error_text);
+
+		if (stack->errornumber == LIBMTP_ERROR_STORAGE_FULL) {
+			error = g_error_new (RB_MTP_THREAD_ERROR, RB_MTP_THREAD_ERROR_NO_SPACE,
+					     _("No space left on MTP device"));
+		} else {
+			error = g_error_new (RB_MTP_THREAD_ERROR, RB_MTP_THREAD_ERROR_SEND_TRACK,
+					     _("Unable to send file to MTP device: %s"),
+					     stack->error_text);
+		}
 		LIBMTP_Clear_Errorstack (thread->device);
 		task->track->item_id = 0;		/* is this actually an invalid item ID? */
 	}
@@ -538,6 +631,17 @@ run_task (RBMtpThread *thread, RBMtpThreadTask *task)
 		}
 		break;
 
+	case THREAD_CALLBACK:
+		{
+			RBMtpThreadCallback cb = (RBMtpThreadCallback)task->callback;
+			cb (thread->device, task->user_data);
+		}
+		break;
+
+	case CREATE_FOLDER:
+		create_folder (thread, task);
+		break;
+
 	case ADD_TO_ALBUM:
 		add_track_to_album_and_update (thread, task);
 		break;
@@ -623,6 +727,21 @@ rb_mtp_thread_set_device_name (RBMtpThread *thread, const char *name)
 	queue_task (thread, task);
 }
 
+void
+rb_mtp_thread_create_folder (RBMtpThread *thread,
+			     const char **path,
+			     RBMtpCreateFolderCallback func,
+			     gpointer data,
+			     GDestroyNotify destroy_data)
+{
+	RBMtpThreadTask *task = create_task (CREATE_FOLDER);
+	task->path = g_strdupv ((char **)path);
+	task->callback = func;
+	task->user_data = data;
+	task->destroy_data = destroy_data;
+	queue_task (thread, task);
+}
+
 void
 rb_mtp_thread_add_to_album (RBMtpThread *thread, LIBMTP_track_t *track, const char *album)
 {
@@ -708,6 +827,19 @@ rb_mtp_thread_download_track (RBMtpThread *thread,
 	queue_task (thread, task);
 }
 
+void
+rb_mtp_thread_queue_callback (RBMtpThread *thread,
+			      RBMtpThreadCallback func,
+			      gpointer data,
+			      GDestroyNotify destroy_data)
+{
+	RBMtpThreadTask *task = create_task (THREAD_CALLBACK);
+	task->callback = func;
+	task->user_data = data;
+	task->destroy_data = destroy_data;
+	queue_task (thread, task);
+}
+
 void
 rb_mtp_thread_report_errors (RBMtpThread *thread, gboolean use_dialog)
 {
@@ -747,7 +879,9 @@ impl_finalize (GObject *object)
 
 	g_hash_table_destroy (thread->albums);
 
-	LIBMTP_Release_Device (thread->device);
+	if (thread->device != NULL) {
+		LIBMTP_Release_Device (thread->device);
+	}
 
 	G_OBJECT_CLASS (rb_mtp_thread_parent_class)->finalize (object);
 }
@@ -795,9 +929,9 @@ rb_mtp_thread_error_get_type (void)
 
 	if (etype == 0)	{
 		static const GEnumValue values[] = {
-			ENUM_ENTRY (RB_MTP_THREAD_ERROR_NO_SPACE, "Not enough space to download track"),
-			ENUM_ENTRY (RB_MTP_THREAD_ERROR_TEMPFILE, "Unable to create temporary file"),
-			ENUM_ENTRY (RB_MTP_THREAD_ERROR_GET_TRACK, "Unable to retrieve track"),
+			ENUM_ENTRY (RB_MTP_THREAD_ERROR_NO_SPACE, "no-space"),
+			ENUM_ENTRY (RB_MTP_THREAD_ERROR_TEMPFILE, "tempfile-failed"),
+			ENUM_ENTRY (RB_MTP_THREAD_ERROR_GET_TRACK, "track-get-failed"),
 			{ 0, 0, 0 }
 		};
 
