@@ -81,6 +81,7 @@
 #include <dbus/dbus-glib.h>
 
 #include "rb-vis-widget.h"
+#include "gseal-gtk-compat.h"
 
 /* preferences */
 #define CONF_VIS_PREFIX  CONF_PREFIX "/plugins/visualizer"
@@ -248,24 +249,6 @@ static const VisualizerModeName vis_mode_name[] = {
 
 RB_PLUGIN_REGISTER(RBVisualizerPlugin, rb_visualizer_plugin)
 
-static GdkCursor *
-get_blank_cursor (GdkWindow *window)
-{
-#if !GTK_CHECK_VERSION(2,16,0)
-	GdkPixmap *pixmap;
-	GdkCursor *cursor;
-	GdkColor color = {0, 0, 0, 0};
-
-	pixmap = gdk_bitmap_create_from_data (window, "\0\0\0\0\0\0\0\0", 1, 1);
-	cursor = gdk_cursor_new_from_pixmap (pixmap, pixmap, &color, &color, 0, 0);
-	g_object_unref (pixmap);
-
-	return cursor;
-#else
-	return gdk_cursor_new (GDK_BLANK_CURSOR);
-#endif
-}
-
 static void
 rb_visualizer_plugin_init (RBVisualizerPlugin *plugin)
 {
@@ -824,11 +807,14 @@ actually_hide_controls (RBVisualizerPlugin *plugin)
 		 */
 		gtk_widget_grab_focus (plugin->vis_widget);
 
-		if (GTK_WIDGET_REALIZED (plugin->vis_widget)) {
+		if (gtk_widget_get_realized (plugin->vis_widget)) {
+			GdkWindow *window;
 			GdkCursor *cursor;
 
-			cursor = get_blank_cursor (plugin->vis_widget->window);
-			gdk_window_set_cursor (plugin->vis_widget->window, cursor);
+			window = gtk_widget_get_window (plugin->vis_widget);
+
+			cursor = gdk_cursor_new (GDK_BLANK_CURSOR);
+			gdk_window_set_cursor (window, cursor);
 			gdk_cursor_unref (cursor);
 		}
 		/* fall through */
@@ -883,8 +869,9 @@ show_controls (RBVisualizerPlugin *plugin, gboolean play_controls_only)
 		case FULLSCREEN:
 			gtk_widget_show (plugin->play_control_widget);
 			gtk_widget_show (plugin->disable_button);
-			if (GTK_WIDGET_REALIZED (plugin->vis_widget)) {
-				gdk_window_set_cursor (plugin->vis_widget->window, NULL);
+			if (gtk_widget_get_realized (plugin->vis_widget)) {
+				gdk_window_set_cursor (gtk_widget_get_window (plugin->vis_widget),
+						       NULL);
 			}
 			break;
 		case EXTERNAL_WINDOW:
@@ -1020,7 +1007,7 @@ resize_vis_window (RBVisualizerPlugin *plugin, int quality, gboolean resize_down
 	float ratio;
 	gboolean update = FALSE;
 
-	if (GTK_WIDGET_REALIZED (plugin->vis_window) == FALSE) {
+	if (gtk_widget_get_realized (plugin->vis_window) == FALSE) {
 		rb_debug ("window isn't realized yet; trying in size-request instead");
 		if (plugin->vis_window_size_request_id == 0) {
 			plugin->vis_window_size_request_id =
@@ -1240,7 +1227,9 @@ update_window (RBVisualizerPlugin *plugin, VisualizerMode mode, int screen, int
 
 		rb_shell_add_widget (plugin->shell,
 				     plugin->vis_box,
-				     RB_SHELL_UI_LOCATION_MAIN_BOTTOM);
+				     RB_SHELL_UI_LOCATION_MAIN_BOTTOM,
+				     FALSE,
+				     TRUE);
 
 		if (plugin->xoverlay != NULL) {
 			gst_x_overlay_set_xwindow_id (plugin->xoverlay, plugin->remote_window);
@@ -1646,6 +1635,7 @@ impl_activate (RBPlugin *plugin,
 	} else if (RB_IS_PLAYER_GST_TEE (pi->player)) {
 		GstElement *videoscale;
 		GstElement *colorspace;
+		GstElement *queue;
 		GstPad *pad;
 
 		rb_debug ("using tee-based visualization");
@@ -1668,11 +1658,14 @@ impl_activate (RBPlugin *plugin,
 		pi->capsfilter = gst_element_factory_make ("capsfilter", NULL);
 		colorspace = gst_element_factory_make ("ffmpegcolorspace", NULL);
 		videoscale = gst_element_factory_make ("videoscale", NULL);
+		queue = gst_element_factory_make ("queue", NULL);
+
+		g_object_set (queue, "max-size-buffers", 3, "max-size-bytes", 0, "max-size-time", (gint64) 0, NULL);
 
 		gst_bin_add_many (GST_BIN (pi->visualizer),
-				  pi->identity, pi->capsfilter, colorspace, videoscale, pi->video_sink,
+				  pi->identity, pi->capsfilter, queue, colorspace, videoscale, pi->video_sink,
 				  NULL);
-		gst_element_link_many (pi->capsfilter, colorspace, videoscale, pi->video_sink, NULL);
+		gst_element_link_many (pi->capsfilter, queue, colorspace, videoscale, pi->video_sink, NULL);
 		/* leave identity unlinked until we have a visualizer element */
 
 		pad = gst_element_get_static_pad (pi->identity, "sink");
@@ -1709,7 +1702,7 @@ impl_activate (RBPlugin *plugin,
 
 	if (pi->vis_shell == NULL) {
 		pi->vis_shell = gtk_vbox_new (FALSE, 0);
-		rb_shell_add_widget (pi->shell, pi->vis_shell, RB_SHELL_UI_LOCATION_MAIN_NOTEBOOK);
+		rb_shell_add_widget (pi->shell, pi->vis_shell, RB_SHELL_UI_LOCATION_MAIN_NOTEBOOK, FALSE, TRUE);
 	}
 
 	if (pi->vis_window == NULL) {
