@@ -1,6 +1,4 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
- *
- *  arch-tag: Headfile of podcast properties dialog
  *
  *  Copyright (C) 2005 Renato Araujo Oliveira Filho <renato.filho@indt.org>
  *
@@ -35,16 +33,20 @@
 
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
-#include <glade/glade.h>
 #include <glib.h>
 
+#if defined(WITH_WEBKIT)
+#include <webkit/webkit.h>
+#endif
+
 #include "rb-podcast-properties-dialog.h"
 #include "rb-file-helpers.h"
-#include "rb-glade-helpers.h"
+#include "rb-builder-helpers.h"
 #include "rb-dialog.h"
 #include "rb-rating.h"
 #include "rb-util.h"
 #include "rb-cut-and-paste-code.h"
+#include "rb-debug.h"
 
 static void rb_podcast_properties_dialog_class_init (RBPodcastPropertiesDialogClass *klass);
 static void rb_podcast_properties_dialog_init (RBPodcastPropertiesDialog *dialog);
@@ -98,6 +100,7 @@ struct RBPodcastPropertiesDialogPrivate
 	GtkWidget   *rating;
 	GtkWidget   *date;
 	GtkWidget   *description;
+	GtkWidget   *description_window;
 
 	GtkWidget   *close_button;
 };
@@ -111,6 +114,29 @@ enum
 
 G_DEFINE_TYPE (RBPodcastPropertiesDialog, rb_podcast_properties_dialog, GTK_TYPE_DIALOG)
 
+#if defined(WITH_WEBKIT)
+/* list of HTML-ish strings that we search for to distinguish plain text from HTML podcast
+ * descriptions.  we don't really have anything else to go on - regular content type
+ * sniffing only works for proper HTML documents, but these are just tiny fragments, usually
+ * with some simple formatting tags.  if we find any of these in a podcast description,
+ * we'll display it as HTML rather than text.
+ */
+static const char *html_clues[] = {
+	"<a ",
+	"<b>",
+	"<i>",
+	"<ul>",
+	"<br",
+	"<img ",
+	"&lt;",
+	"&gt;",
+	"&amp;",
+	"&quo;",
+	"&#8",
+	"&#x"
+};
+#endif
+
 static void
 rb_podcast_properties_dialog_class_init (RBPodcastPropertiesDialogClass *klass)
 {
@@ -133,10 +159,74 @@ rb_podcast_properties_dialog_class_init (RBPodcastPropertiesDialogClass *klass)
 	g_type_class_add_private (klass, sizeof (RBPodcastPropertiesDialogPrivate));
 }
 
+#if defined(WITH_WEBKIT)
+
+static WebKitNavigationResponse
+navigation_requested_cb (WebKitWebView *web_view,
+			 WebKitWebFrame *frame,
+			 WebKitNetworkRequest *request,
+			 RBPodcastPropertiesDialog *dialog)
+{
+	const char *uri;
+	GError *error = NULL;
+
+	uri = webkit_network_request_get_uri (request);
+	gtk_show_uri (gtk_widget_get_screen (GTK_WIDGET (dialog)), uri, GDK_CURRENT_TIME, &error);
+	if (error != NULL) {
+		rb_error_dialog (NULL, _("Unable to display requested URI"), "%s", error->message);
+		g_error_free (error);
+	}
+
+	return WEBKIT_NAVIGATION_RESPONSE_IGNORE;
+}
+
+static void
+set_webkit_settings (WebKitWebView *view)
+{
+	WebKitWebSettings *settings;
+
+	settings = webkit_web_settings_new ();
+	g_object_set (settings,
+		      "enable-scripts", FALSE,
+		      "enable-plugins", FALSE,
+		      NULL);
+	webkit_web_view_set_settings (view, settings);
+}
+
+static void
+set_webkit_font_from_gtk_style (WebKitWebView *view)
+{
+	WebKitWebSettings *settings;
+	GtkStyle *style;
+	int font_size;
+	const char *font_family;
+
+	style = gtk_widget_get_style (GTK_WIDGET (view));
+	settings = webkit_web_view_get_settings (view);
+
+	font_size = pango_font_description_get_size (style->font_desc);
+	if (pango_font_description_get_size_is_absolute (style->font_desc) == FALSE)
+		font_size /= PANGO_SCALE;
+
+	font_family = pango_font_description_get_family (style->font_desc);
+
+	rb_debug ("setting font settings: %s / %d", font_family, font_size);
+	g_object_set (settings,
+		      "default-font-size", font_size,
+		      "default-monospace-font-size", font_size,
+		      "sans-serif-font-family", font_family,
+		      "monospace-font-family", font_family,
+		      NULL);
+}
+#endif
+
 static void
 rb_podcast_properties_dialog_init (RBPodcastPropertiesDialog *dialog)
 {
-	GladeXML *xml;
+	GtkWidget  *content_area;
+	GtkWidget  *bin;
+	GtkWidget  *widget;
+	GtkBuilder *builder;
 	AtkObject *lobj, *robj;
 
 	dialog->priv = G_TYPE_INSTANCE_GET_PRIVATE (dialog,
@@ -148,65 +238,89 @@ rb_podcast_properties_dialog_init (RBPodcastPropertiesDialog *dialog)
 				 G_CALLBACK (rb_podcast_properties_dialog_response_cb),
 				 dialog, 0);
 
+	content_area = gtk_dialog_get_content_area (GTK_DIALOG (dialog));
+
 	gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
 	gtk_container_set_border_width (GTK_CONTAINER (dialog), 5);
-	gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (dialog)->vbox), 2);
+	gtk_box_set_spacing (GTK_BOX (content_area), 2);
 
 	gtk_dialog_set_default_response (GTK_DIALOG (dialog),
 					 GTK_RESPONSE_OK);
 
-	xml = rb_glade_xml_new ("podcast-properties.glade",
-				"podcastproperties",
-				dialog);
-	glade_xml_signal_autoconnect (xml);
+	builder = rb_builder_load ("podcast-properties.ui", dialog);
 
-	gtk_container_add (GTK_CONTAINER (GTK_DIALOG (dialog)->vbox),
-			   glade_xml_get_widget (xml, "podcastproperties"));
+	gtk_container_add (GTK_CONTAINER (content_area),
+			   GTK_WIDGET (gtk_builder_get_object (builder, "podcastproperties")));
 	dialog->priv->close_button = gtk_dialog_add_button (GTK_DIALOG (dialog),
 							    GTK_STOCK_CLOSE,
 							    GTK_RESPONSE_CLOSE);
 	gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_CLOSE);
 
-	/* get the widgets from the XML */
-	dialog->priv->title = glade_xml_get_widget (xml, "titleLabel");
-	dialog->priv->feed = glade_xml_get_widget (xml, "feedLabel");
-	dialog->priv->duration = glade_xml_get_widget (xml, "durationLabel");
-	dialog->priv->location = glade_xml_get_widget (xml, "locationLabel");
-	dialog->priv->download_location = glade_xml_get_widget (xml, "downloadLocationLabel");
-	dialog->priv->lastplayed = glade_xml_get_widget (xml, "lastplayedLabel");
-	dialog->priv->playcount = glade_xml_get_widget (xml, "playcountLabel");
-	dialog->priv->bitrate = glade_xml_get_widget (xml, "bitrateLabel");
-	dialog->priv->date = glade_xml_get_widget (xml, "dateLabel");
-	dialog->priv->description = glade_xml_get_widget (xml, "descriptionLabel");
+	/* get the widgets from the builder */
+	dialog->priv->title = GTK_WIDGET (gtk_builder_get_object (builder, "titleLabel"));
+	dialog->priv->feed = GTK_WIDGET (gtk_builder_get_object (builder, "feedLabel"));
+	dialog->priv->duration = GTK_WIDGET (gtk_builder_get_object (builder, "durationLabel"));
+	dialog->priv->location = GTK_WIDGET (gtk_builder_get_object (builder, "locationLabel"));
+	dialog->priv->download_location = GTK_WIDGET (gtk_builder_get_object (builder, "downloadLocationLabel"));
+	dialog->priv->lastplayed = GTK_WIDGET (gtk_builder_get_object (builder, "lastplayedLabel"));
+	dialog->priv->playcount = GTK_WIDGET (gtk_builder_get_object (builder, "playcountLabel"));
+	dialog->priv->bitrate = GTK_WIDGET (gtk_builder_get_object (builder, "bitrateLabel"));
+	dialog->priv->date = GTK_WIDGET (gtk_builder_get_object (builder, "dateLabel"));
+#if defined(WITH_WEBKIT)
+	dialog->priv->description = webkit_web_view_new ();
+	set_webkit_settings (WEBKIT_WEB_VIEW (dialog->priv->description));
+	set_webkit_font_from_gtk_style (WEBKIT_WEB_VIEW (dialog->priv->description));
 
-	rb_glade_boldify_label (xml, "titleDescLabel");
-	rb_glade_boldify_label (xml, "feedDescLabel");
-	rb_glade_boldify_label (xml, "locationDescLabel");
-	rb_glade_boldify_label (xml, "downloadLocationDescLabel");
-	rb_glade_boldify_label (xml, "durationDescLabel");
-	rb_glade_boldify_label (xml, "ratingDescLabel");
-	rb_glade_boldify_label (xml, "lastplayedDescLabel");
-	rb_glade_boldify_label (xml, "playcountDescLabel");
-	rb_glade_boldify_label (xml, "bitrateDescLabel");
-	rb_glade_boldify_label (xml, "dateDescLabel");
-	rb_glade_boldify_label (xml, "descriptionDescLabel");
+	g_signal_connect_object (dialog->priv->description,
+				 "navigation-requested",
+				 G_CALLBACK (navigation_requested_cb),
+				 dialog,
+				 0);
+#else
+	dialog->priv->description = gtk_label_new (NULL);
+	gtk_label_set_line_wrap (GTK_LABEL (dialog->priv->description), TRUE);
+#endif
+	/* add relationship between the description label and the description widget */
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "descriptionDescLabel"));
+	gtk_label_set_mnemonic_widget (GTK_LABEL (widget), dialog->priv->description);
+	lobj = gtk_widget_get_accessible (widget);
+	robj = gtk_widget_get_accessible (dialog->priv->description);
+	atk_object_add_relationship (lobj, ATK_RELATION_LABEL_FOR, robj);
+	atk_object_add_relationship (robj, ATK_RELATION_LABELLED_BY, lobj);
+
+	bin = GTK_WIDGET (gtk_builder_get_object (builder, "descriptionViewport"));
+	gtk_container_add (GTK_CONTAINER (bin), dialog->priv->description);
+
+	dialog->priv->description_window = GTK_WIDGET (gtk_builder_get_object (builder, "descriptionWindow"));
+
+	rb_builder_boldify_label (builder, "titleDescLabel");
+	rb_builder_boldify_label (builder, "feedDescLabel");
+	rb_builder_boldify_label (builder, "locationDescLabel");
+	rb_builder_boldify_label (builder, "downloadLocationDescLabel");
+	rb_builder_boldify_label (builder, "durationDescLabel");
+	rb_builder_boldify_label (builder, "ratingDescLabel");
+	rb_builder_boldify_label (builder, "lastplayedDescLabel");
+	rb_builder_boldify_label (builder, "playcountDescLabel");
+	rb_builder_boldify_label (builder, "bitrateDescLabel");
+	rb_builder_boldify_label (builder, "dateDescLabel");
+	rb_builder_boldify_label (builder, "descriptionDescLabel");
 
 	dialog->priv->rating = GTK_WIDGET (rb_rating_new ());
 	g_signal_connect_object (dialog->priv->rating,
 				 "rated",
 				 G_CALLBACK (rb_podcast_properties_dialog_rated_cb),
 				 G_OBJECT (dialog), 0);
-	gtk_container_add (GTK_CONTAINER (glade_xml_get_widget (xml, "ratingVBox")),
+	gtk_container_add (GTK_CONTAINER (gtk_builder_get_object (builder, "ratingVBox")),
 			   dialog->priv->rating);
 
 	/* add relationship between the rating label and the rating widget */
-	lobj = gtk_widget_get_accessible (glade_xml_get_widget (xml, "ratingDescLabel"));
+	lobj = gtk_widget_get_accessible (GTK_WIDGET (gtk_builder_get_object (builder, "ratingDescLabel")));
 	robj = gtk_widget_get_accessible (dialog->priv->rating);
 	
 	atk_object_add_relationship (lobj, ATK_RELATION_LABEL_FOR, robj);
 	atk_object_add_relationship (robj, ATK_RELATION_LABELLED_BY, lobj);
 
-	g_object_unref (G_OBJECT (xml));
+	g_object_unref (builder);
 }
 
 static void
@@ -521,13 +635,56 @@ rb_podcast_properties_dialog_update_date (RBPodcastPropertiesDialog *dialog)
 	g_free (time);
 }
 
+#if defined(WITH_WEBKIT)
+static gboolean
+update_scrollbar_policy_cb (WebKitWebFrame *frame, RBPodcastPropertiesDialog *dialog)
+{
+	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (dialog->priv->description_window),
+					webkit_web_frame_get_horizontal_scrollbar_policy (frame),
+					webkit_web_frame_get_vertical_scrollbar_policy (frame));
+	return TRUE;
+}
+
+#endif
+
 static void
 rb_podcast_properties_dialog_update_description (RBPodcastPropertiesDialog *dialog)
 {
+#if defined(WITH_WEBKIT)
+	WebKitWebFrame *frame;
 	const char *str;
+	int i;
+	gboolean loaded = FALSE;
+	str = rhythmdb_entry_get_string (dialog->priv->current_entry, RHYTHMDB_PROP_DESCRIPTION);
+	for (i = 0; i < G_N_ELEMENTS (html_clues); i++) {
+		if (g_strstr_len (str, -1, html_clues[i]) != NULL) {
+			webkit_web_view_load_html_string (WEBKIT_WEB_VIEW (dialog->priv->description),
+							  str,
+							  "");
+			loaded = TRUE;
+		}
+	}
 
+	if (loaded == FALSE) {
+		webkit_web_view_load_string (WEBKIT_WEB_VIEW (dialog->priv->description),
+					     str,
+					     "text/plain",
+					     "utf-8",
+					     "");
+	}
+
+	/* ensure scrollbar policy for the frame matches the viewport */
+	frame = webkit_web_view_get_main_frame (WEBKIT_WEB_VIEW (dialog->priv->description));
+	g_signal_connect_object (frame,
+				 "scrollbars-policy-changed",
+				 G_CALLBACK (update_scrollbar_policy_cb),
+				 dialog, 0);
+	update_scrollbar_policy_cb (frame, dialog);
+#else
+	const char *str;
 	str = rhythmdb_entry_get_string (dialog->priv->current_entry, RHYTHMDB_PROP_DESCRIPTION);
 	gtk_label_set_text (GTK_LABEL (dialog->priv->description), str);
+#endif
 }
 
 static char *
