@@ -1,6 +1,4 @@
- /*
- *  arch-tag: Header for RhythmDB private bits
- *
+/*
  *  Copyright (C) 2004 Colin Walters <walters@rhythmbox.org>
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -30,9 +28,9 @@
 #ifndef RHYTHMDB_PRIVATE_H
 #define RHYTHMDB_PRIVATE_H
 
-#include "rhythmdb.h"
-#include "rb-refstring.h"
-#include "rb-metadata.h"
+#include <rhythmdb/rhythmdb.h>
+#include <rhythmdb/rb-refstring.h>
+#include <metadata/rb-metadata.h>
 
 G_BEGIN_DECLS
 
@@ -69,11 +67,7 @@ enum {
 struct RhythmDBEntry_ {
 	/* internal bits */
 	guint flags;
-#if (GLIB_MAJOR_VERSION >= 2 && GLIB_MINOR_VERSION >= 10)
 	volatile gint refcount;
-#else
-	gint refcount;
-#endif
 	void *data;
 	RhythmDBEntryType type;
 	guint id;
@@ -82,21 +76,21 @@ struct RhythmDBEntry_ {
 	RBRefString *title;
 	RBRefString *artist;
 	RBRefString *album;
+	RBRefString *album_artist;
 	RBRefString *genre;
+	RBRefString *comment;
 	RBRefString *musicbrainz_trackid;
 	RBRefString *musicbrainz_artistid;
 	RBRefString *musicbrainz_albumid;
 	RBRefString *musicbrainz_albumartistid;
 	RBRefString *artist_sortname;
 	RBRefString *album_sortname;
+	RBRefString *album_artist_sortname;
 	gulong tracknum;
 	gulong discnum;
 	gulong duration;
 	gulong bitrate;
-	double track_gain;
-	double track_peak;
-	double album_gain;
-	double album_peak;
+	double bpm;
 	GDate date;
 
 	/* filesystem */
@@ -129,11 +123,6 @@ struct _RhythmDBPrivate
 	gint read_counter;
 
 	RBMetaData *metadata;
-	gboolean metadata_blocked;
-	GMutex *metadata_lock;
-	GCond *metadata_cond;
-
-	xmlChar **column_xml_names;
 
 	RBRefString *empty_string;
 	RBRefString *octet_stream_str;
@@ -150,6 +139,8 @@ struct _RhythmDBPrivate
 	GList *outstanding_stats;
 	GMutex *stat_mutex;
 	gboolean stat_thread_running;
+	int stat_thread_count;
+	int stat_thread_done;
 
 	GVolumeMonitor *volume_monitor;
 	GHashTable *monitored_directories;
@@ -206,8 +197,6 @@ typedef struct
 		RHYTHMDB_EVENT_THREAD_EXITED,
 		RHYTHMDB_EVENT_DB_SAVED,
 		RHYTHMDB_EVENT_QUERY_COMPLETE,
-		RHYTHMDB_EVENT_FILE_CREATED_OR_MODIFIED,
-		RHYTHMDB_EVENT_FILE_DELETED,
 		RHYTHMDB_EVENT_ENTRY_SET
 	} type;
 	RBRefString *uri;
