@@ -115,8 +115,8 @@ static void rhythmdb_hash_tree_foreach (RhythmDB *adb,
 					gpointer data);
 
 /* Update both of those! */
-#define RHYTHMDB_TREE_XML_VERSION "1.6"
-#define RHYTHMDB_TREE_XML_VERSION_INT 160
+#define RHYTHMDB_TREE_XML_VERSION "1.7"
+#define RHYTHMDB_TREE_XML_VERSION_INT 170
 
 static void destroy_tree_property (RhythmDBTreeProperty *prop);
 static RhythmDBTreeProperty *get_or_create_album (RhythmDBTree *db, RhythmDBTreeProperty *artist,
@@ -334,6 +334,7 @@ struct RhythmDBTreeLoadContext
 	guint canonicalise_uris : 1;
 	guint reload_all_metadata : 1;
 	guint update_podcasts : 1;
+	guint update_local_mountpoints : 1;
 };
 
 /* Returns the version as an int, multiplied by 100,
@@ -342,11 +343,12 @@ static int
 version_to_int (const char *version)
 {
 	float ver;
+	char *eptr;
 
-	if (sscanf (version, "%f", &ver) != 1) {
+	ver = g_ascii_strtod (version, &eptr);
+	if (eptr == version) {
 		return (int) (1.0 * 100);
 	}
-
 	return (int)roundf(ver * 100);
 }
 
@@ -394,6 +396,10 @@ rhythmdb_tree_parser_start_element (struct RhythmDBTreeLoadContext *ctx,
 					case 150:
 						rb_debug ("Upgrade Podcasts remote vs. local locations");
 						ctx->update_podcasts = TRUE;
+					case 160:
+						rb_debug ("reloading all file metadata to get sortnames, album artist, comments, bpm and updating mountpoints");
+						ctx->reload_all_metadata = TRUE;
+						ctx->update_local_mountpoints = TRUE;
 					case RHYTHMDB_TREE_XML_VERSION_INT:
 						/* current version */
 						break;
@@ -535,6 +541,28 @@ rhythmdb_tree_parser_end_element (struct RhythmDBTreeLoadContext *ctx,
 				ctx->entry->mountpoint = tmp;
 			}
 		}
+		if (ctx->entry->type == RHYTHMDB_ENTRY_TYPE_SONG) {
+			/* Since we now care about mountpoints for all local entries, not just
+			 * those on things that actually get mounted and unmounted, we need to
+			 * ensure they're all correct.
+			 */
+			if (ctx->update_local_mountpoints) {
+				const char *loc = rb_refstring_get (ctx->entry->location);
+				if (loc == NULL || g_str_has_prefix (loc, "file:///")) {
+					char *nmp;
+					nmp = rb_uri_get_mount_point (loc);
+					if (ctx->entry->mountpoint != NULL) {
+						rb_refstring_unref (ctx->entry->mountpoint);
+						ctx->entry->mountpoint = NULL;
+					}
+
+					if (nmp != NULL) {
+						ctx->entry->mountpoint = rb_refstring_new (nmp);
+						g_free (nmp);
+					}
+				}
+			}
+		}
 
 		if (ctx->entry->location != NULL && rb_refstring_get (ctx->entry->location)[0] != '\0') {
 			RhythmDBEntry *entry;
@@ -972,9 +1000,15 @@ save_entry (RhythmDBTree *db,
 		case RHYTHMDB_PROP_ARTIST:
 			save_entry_string(ctx, elt_name, rb_refstring_get (entry->artist));
 			break;
+		case RHYTHMDB_PROP_ALBUM_ARTIST:
+			save_entry_string_if_set(ctx, elt_name, rb_refstring_get (entry->album_artist));
+			break;
 		case RHYTHMDB_PROP_GENRE:
 			save_entry_string(ctx, elt_name, rb_refstring_get (entry->genre));
 			break;
+		case RHYTHMDB_PROP_COMMENT:
+			save_entry_string_if_set(ctx, elt_name, rb_refstring_get (entry->comment));
+			break;
 		case RHYTHMDB_PROP_MUSICBRAINZ_TRACKID:
 			save_entry_string_if_set (ctx, elt_name, rb_refstring_get (entry->musicbrainz_trackid));
 			break;
@@ -993,6 +1027,9 @@ save_entry (RhythmDBTree *db,
 		case RHYTHMDB_PROP_ALBUM_SORTNAME:
 			save_entry_string_if_set (ctx, elt_name, rb_refstring_get (entry->album_sortname));
 			break;
+		case RHYTHMDB_PROP_ALBUM_ARTIST_SORTNAME:
+			save_entry_string_if_set (ctx, elt_name, rb_refstring_get (entry->album_artist_sortname));
+			break;
 		case RHYTHMDB_PROP_TRACK_NUMBER:
 			save_entry_ulong (ctx, elt_name, entry->tracknum, FALSE);
 			break;
@@ -1014,6 +1051,9 @@ save_entry (RhythmDBTree *db,
 		case RHYTHMDB_PROP_LOCATION:
 			save_entry_string(ctx, elt_name, rb_refstring_get (entry->location));
 			break;
+		case RHYTHMDB_PROP_BPM:
+			save_entry_double(ctx, elt_name, entry->bpm);
+			break;
 		case RHYTHMDB_PROP_MOUNTPOINT:
 			save_entry_string_if_set (ctx, elt_name, rb_refstring_get (entry->mountpoint));
 			break;
@@ -1100,14 +1140,18 @@ save_entry (RhythmDBTree *db,
 		case RHYTHMDB_PROP_GENRE_SORT_KEY:
 		case RHYTHMDB_PROP_ARTIST_SORT_KEY:
 		case RHYTHMDB_PROP_ALBUM_SORT_KEY:
+		case RHYTHMDB_PROP_ALBUM_ARTIST_SORT_KEY:
 		case RHYTHMDB_PROP_ARTIST_SORTNAME_SORT_KEY:
 		case RHYTHMDB_PROP_ALBUM_SORTNAME_SORT_KEY:
+		case RHYTHMDB_PROP_ALBUM_ARTIST_SORTNAME_SORT_KEY:
 		case RHYTHMDB_PROP_TITLE_FOLDED:
 		case RHYTHMDB_PROP_GENRE_FOLDED:
 		case RHYTHMDB_PROP_ARTIST_FOLDED:
 		case RHYTHMDB_PROP_ALBUM_FOLDED:
+		case RHYTHMDB_PROP_ALBUM_ARTIST_FOLDED:
 		case RHYTHMDB_PROP_ARTIST_SORTNAME_FOLDED:
 		case RHYTHMDB_PROP_ALBUM_SORTNAME_FOLDED:
+		case RHYTHMDB_PROP_ALBUM_ARTIST_SORTNAME_FOLDED:
 		case RHYTHMDB_PROP_LAST_PLAYED_STR:
 		case RHYTHMDB_PROP_PLAYBACK_ERROR:
 		case RHYTHMDB_PROP_FIRST_SEEN_STR:
@@ -1949,6 +1993,9 @@ evaluate_conjunctive_subquery (RhythmDBTree *dbtree,
 		case RHYTHMDB_QUERY_PROP_EQUALS:
 			RHYTHMDB_PROPERTY_COMPARE (!=)
 			break;
+		case RHYTHMDB_QUERY_PROP_NOT_EQUAL:
+			RHYTHMDB_PROPERTY_COMPARE (==)
+			break;
 		case RHYTHMDB_QUERY_PROP_GREATER:
 			RHYTHMDB_PROPERTY_COMPARE (<)
 			break;
@@ -1958,6 +2005,7 @@ evaluate_conjunctive_subquery (RhythmDBTree *dbtree,
 		case RHYTHMDB_QUERY_END:
 		case RHYTHMDB_QUERY_DISJUNCTION:
 		case RHYTHMDB_QUERY_PROP_YEAR_EQUALS:
+		case RHYTHMDB_QUERY_PROP_YEAR_NOT_EQUAL:
 		case RHYTHMDB_QUERY_PROP_YEAR_LESS:
 		case RHYTHMDB_QUERY_PROP_YEAR_GREATER:
 			g_assert_not_reached ();
