@@ -34,9 +34,9 @@
 #include <stdarg.h>
 #include <libxml/tree.h>
 
-#include "rb-refstring.h"
-#include "rb-string-value-map.h"
-#include "rhythmdb-query-results.h"
+#include <rhythmdb/rb-refstring.h>
+#include <lib/rb-string-value-map.h>
+#include <rhythmdb/rhythmdb-query-results.h>
 
 G_BEGIN_DECLS
 
@@ -128,6 +128,7 @@ typedef enum
 
 	/* general */
 	RHYTHMDB_QUERY_PROP_EQUALS,
+	RHYTHMDB_QUERY_PROP_NOT_EQUAL,
 
 	/* string */
 	RHYTHMDB_QUERY_PROP_LIKE,
@@ -143,6 +144,7 @@ typedef enum
 	RHYTHMDB_QUERY_PROP_CURRENT_TIME_WITHIN,
 	RHYTHMDB_QUERY_PROP_CURRENT_TIME_NOT_WITHIN,
 	RHYTHMDB_QUERY_PROP_YEAR_EQUALS,
+	RHYTHMDB_QUERY_PROP_YEAR_NOT_EQUAL,
 	RHYTHMDB_QUERY_PROP_YEAR_GREATER,
 	RHYTHMDB_QUERY_PROP_YEAR_LESS,
 } RhythmDBQueryType;
@@ -216,6 +218,17 @@ typedef enum
 	RHYTHMDB_PROP_ALBUM_SORTNAME_SORT_KEY,
 	RHYTHMDB_PROP_ALBUM_SORTNAME_FOLDED,
 
+	RHYTHMDB_PROP_COMMENT,
+
+	RHYTHMDB_PROP_ALBUM_ARTIST,
+	RHYTHMDB_PROP_ALBUM_ARTIST_SORT_KEY,
+	RHYTHMDB_PROP_ALBUM_ARTIST_FOLDED,
+	RHYTHMDB_PROP_ALBUM_ARTIST_SORTNAME,
+	RHYTHMDB_PROP_ALBUM_ARTIST_SORTNAME_SORT_KEY,
+	RHYTHMDB_PROP_ALBUM_ARTIST_SORTNAME_FOLDED,
+
+	RHYTHMDB_PROP_BPM,
+
 	RHYTHMDB_NUM_PROPERTIES
 } RhythmDBPropType;
 
@@ -232,8 +245,6 @@ enum {
 #define RHYTHMDB_PROP_STREAM_SONG_ALBUM		"rb:stream-song-album"
 #define RHYTHMDB_PROP_COVER_ART			"rb:coverArt"
 #define RHYTHMDB_PROP_COVER_ART_URI		"rb:coverArt-uri"
-#define RHYTHMDB_PROP_ALBUM_ARTIST		"rb:album-artist"
-#define RHYTHMDB_PROP_ALBUM_ARTIST_SORTNAME	"rb:album-artist-sortname"
 
 GType rhythmdb_query_type_get_type (void);
 GType rhythmdb_prop_type_get_type (void);
@@ -248,6 +259,9 @@ typedef struct {
 	RhythmDBQuery *subquery;
 } RhythmDBQueryData;
 
+GType rhythmdb_entry_change_get_type (void);
+#define RHYTHMDB_TYPE_ENTRY_CHANGE (rhythmdb_entry_change_get_type ())
+
 typedef struct {
 	RhythmDBPropType prop;
 	GValue old;
@@ -289,7 +303,7 @@ struct _RhythmDBClass
 
 	/* signals */
 	void	(*entry_added)		(RhythmDB *db, RhythmDBEntry *entry);
-	void	(*entry_changed)	(RhythmDB *db, RhythmDBEntry *entry, GSList *changes); /* list of RhythmDBEntryChanges */
+	void	(*entry_changed)	(RhythmDB *db, RhythmDBEntry *entry, GValueArray *changes); /* array of RhythmDBEntryChanges */
 	void	(*entry_deleted)	(RhythmDB *db, RhythmDBEntry *entry);
 	void	(*entry_keyword_added)	(RhythmDB *db, RhythmDBEntry *entry, RBRefString *keyword);
 	void	(*entry_keyword_removed)(RhythmDB *db, RhythmDBEntry *entry, RBRefString *keyword);
@@ -473,9 +487,9 @@ RhythmDBQuery *	rhythmdb_query_copy			(RhythmDBQuery *array);
 void		rhythmdb_query_preprocess		(RhythmDB *db, RhythmDBQuery *query);
 
 void		rhythmdb_query_serialize		(RhythmDB *db, RhythmDBQuery *query,
-							 xmlNodePtr node);
+							 xmlNodePtr parent);
 
-RhythmDBQuery *	rhythmdb_query_deserialize		(RhythmDB *db, xmlNodePtr node);
+RhythmDBQuery *	rhythmdb_query_deserialize		(RhythmDB *db, xmlNodePtr parent);
 
 char *		rhythmdb_query_to_string		(RhythmDB *db, RhythmDBQuery *query);
 
@@ -492,6 +506,7 @@ RBStringValueMap* rhythmdb_entry_gather_metadata	(RhythmDB *db, RhythmDBEntry *e
 void		rhythmdb_emit_entry_extra_metadata_notify (RhythmDB *db, RhythmDBEntry *entry, const gchar *property_name, const GValue *metadata);
 
 gboolean	rhythmdb_is_busy			(RhythmDB *db);
+void		rhythmdb_get_progress_info		(RhythmDB *db, char **text, float *progress);
 char *		rhythmdb_compute_status_normal		(gint n_songs, glong duration,
 							 guint64 size,
 							 const char *singular,
