@@ -29,9 +29,9 @@
 #ifndef __RB_SHELL_H
 #define __RB_SHELL_H
 
-#include "rb-source.h"
-#include "rhythmdb.h"
-#include "rb-song-info.h"
+#include <sources/rb-source.h>
+#include <rhythmdb/rhythmdb.h>
+#include <widgets/rb-song-info.h>
 
 G_BEGIN_DECLS
 
@@ -67,16 +67,18 @@ typedef enum
 GType rb_shell_ui_location_get_type (void);
 #define RB_TYPE_SHELL_UI_LOCATION	(rb_shell_ui_location_get_type())
 
-typedef struct RBShellPrivate RBShellPrivate;
+typedef struct _RBShell RBShell;
+typedef struct _RBShellClass RBShellClass;
+typedef struct _RBShellPrivate RBShellPrivate;
 
-typedef struct
+struct _RBShell
 {
         GObject parent;
 
 	RBShellPrivate *priv;
-} RBShell;
+};
 
-typedef struct
+struct _RBShellClass
 {
         GObjectClass parent_class;
 
@@ -85,13 +87,14 @@ typedef struct
 	void	 (*visibility_changed)	(RBShell *shell, gboolean visible);
 	void	 (*create_song_info)	(RBShell *shell, RBSongInfo *song_info, gboolean multi);
 	void	 (*removable_media_scan_finished) (RBShell *shell);
-} RBShellClass;
+};
 
 GType		rb_shell_get_type	(void);
 
 RBShell *	rb_shell_new		(gboolean no_registration,
 					 gboolean no_update,
 					 gboolean dry_run,
+					 gboolean autostarted,
 					 char *rhythmdb,
 					 char *playlists);
 
