@@ -1,6 +1,4 @@
 /*
- *  arch-tag: Header for search entry/browse toggle container
- *
  *  Copyright (C) 2003 Colin Walters <walters@verbum.org>
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -29,7 +27,7 @@
 
 #include <gtk/gtk.h>
 
-#include "rb-source.h"
+#include <sources/rb-source.h>
 
 #ifndef __RB_SOURCE_HEADER_H
 #define __RB_SOURCE_HEADER_H
