@@ -52,6 +52,8 @@
 #include <glib/gi18n.h>
 #include <glib-object.h>
 
+#include "rb-track-transfer-batch.h"
+#include "rb-track-transfer-queue.h"
 #include <profiles/gnome-media-profiles.h>
 #include <profiles/audio-profile-choose.h>
 
@@ -63,8 +65,9 @@
 #include "rb-util.h"
 #include "eel-gconf-extensions.h"
 #include "rb-library-source.h"
-#include "rb-removable-media-manager.h"
 #include "rb-auto-playlist-source.h"
+#include "rb-encoder.h"
+#include "rb-missing-plugins.h"
 
 static void rb_library_source_class_init (RBLibrarySourceClass *klass);
 static void rb_library_source_init (RBLibrarySource *source);
@@ -751,8 +754,10 @@ rb_library_source_path_changed_cb (GtkComboBox *box, RBLibrarySource *source)
 	gint index;
 
 	index = gtk_combo_box_get_active (box);
-	path = (index >= 0) ? library_layout_paths[index].path : "";
-	eel_gconf_set_string (CONF_LIBRARY_LAYOUT_PATH, path);
+	if (index >= 0) {
+		path = library_layout_paths[index].path;
+		eel_gconf_set_string (CONF_LIBRARY_LAYOUT_PATH, path);
+	}
 }
 
 static void
@@ -762,8 +767,10 @@ rb_library_source_filename_changed_cb (GtkComboBox *box, RBLibrarySource *source
 	gint index;
 
 	index = gtk_combo_box_get_active (box);
-	filename = (index >= 0) ? library_layout_filenames[index].path : "";
-	eel_gconf_set_string (CONF_LIBRARY_LAYOUT_FILENAME, filename);
+	if (index >= 0) {
+		filename = library_layout_filenames[index].path;
+		eel_gconf_set_string (CONF_LIBRARY_LAYOUT_FILENAME, filename);
+	}
 }
 
 static void
@@ -833,6 +840,11 @@ sanitize_pattern (const char *pat)
  * %aA -- album artist (lowercase)
  * %as -- album artist sortname
  * %aS -- album artist sortname (lowercase)
+ * %ay -- album release year
+ * %an -- album disc number
+ * %aN -- album disc number, zero padded
+ * %ag -- album genre
+ * %aG -- album genre (lowercase)
  * %tn -- track number (i.e 8)
  * %tN -- track number, zero padded (i.e 08)
  * %tt -- track title
@@ -850,28 +862,27 @@ filepath_parse_pattern (RhythmDB *db,
 	const char *p;
 	char *temp;
 	GString *s;
-	GValue *value;
 	RBRefString *albumartist;
+	RBRefString *albumartist_sort;
 
 	if (pattern == NULL || pattern[0] == 0)
 		return g_strdup (" ");
 
-	/* request album artist (this is sort of temporary) */
-	value = rhythmdb_entry_request_extra_metadata (db, entry, RHYTHMDB_PROP_ALBUM_ARTIST);
-	if (value != NULL) {
-		albumartist = rb_refstring_new (g_value_get_string (value));
-		g_value_unset (value);
-		g_free (value);
-	} else {
+	/* figure out album artist - use the plain artist field if not specified */
+	albumartist = rhythmdb_entry_get_refstring (entry, RHYTHMDB_PROP_ALBUM_ARTIST);
+	if (albumartist == NULL || g_strcmp0 (rb_refstring_get (albumartist), "") == 0) {
 		albumartist = rhythmdb_entry_get_refstring (entry, RHYTHMDB_PROP_ARTIST);
 	}
+	albumartist_sort = rhythmdb_entry_get_refstring (entry, RHYTHMDB_PROP_ALBUM_ARTIST_SORTNAME);
+	if (albumartist_sort == NULL || g_strcmp0 (rb_refstring_get (albumartist_sort), "") == 0) {
+		albumartist_sort = rhythmdb_entry_get_refstring (entry, RHYTHMDB_PROP_ARTIST_SORTNAME);
+	}
 
 	s = g_string_new (NULL);
 
 	p = pattern;
 	while (*p) {
 		char *string = NULL;
-		char *t;
 
 		/* If not a % marker, copy and continue */
 		if (*p != '%') {
@@ -906,12 +917,25 @@ filepath_parse_pattern (RhythmDB *db,
 				string = sanitize_path (rb_refstring_get_folded (albumartist));
 				break;
 			case 's':
-				string = sanitize_path (rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_ARTIST_SORTNAME));
+				string = sanitize_path (rb_refstring_get (albumartist_sort));
 				break;
 			case 'S':
-				t = g_utf8_strdown (rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_ARTIST_SORTNAME), -1);
-				string = sanitize_path (t);
-				g_free (t);
+				string = sanitize_path (rb_refstring_get_folded (albumartist_sort));
+				break;
+			case 'y':
+				string = g_strdup_printf ("%u", (guint)rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_YEAR));
+				break;
+			case 'n':
+				string = g_strdup_printf ("%u", (guint)rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_DISC_NUMBER));
+				break;
+			case 'N':
+				string = g_strdup_printf ("%02u", (guint)rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_DISC_NUMBER));
+				break;
+			case 'g':
+				string = sanitize_path (rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_GENRE));
+				break;
+			case 'G':
+				string = sanitize_path (rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_GENRE_FOLDED));
 				break;
 			default:
 				string = g_strdup_printf ("%%a%c", *p);
@@ -940,9 +964,7 @@ filepath_parse_pattern (RhythmDB *db,
 				string = sanitize_path (rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_ARTIST_SORTNAME));
 				break;
 			case 'S':
-				t = g_utf8_strdown (rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_ARTIST_SORTNAME), -1);
-				string = sanitize_path (t);
-				g_free (t);
+				string = sanitize_path (rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_ARTIST_SORTNAME_FOLDED));
 				break;
 			case 'n':
 				/* Track number */
@@ -1039,7 +1061,8 @@ rb_library_source_layout_path_changed (GConfClient *client,
 				       RBLibrarySource *source)
 {
 	char *value;
-	int i = 0;
+	int active;
+	int i;
 
 	g_return_if_fail (entry != NULL);
 	g_return_if_fail (strcmp (entry->key, CONF_LIBRARY_LAYOUT_PATH) == 0);
@@ -1054,12 +1077,16 @@ rb_library_source_layout_path_changed (GConfClient *client,
 		return;
 	}
 
-	while (library_layout_paths[i].path && strcmp (library_layout_paths[i].path, value) != 0) {
-		i++;
+	active = -1;
+	for (i = 0; library_layout_paths[i].path != NULL; i++) {
+		if (strcmp (library_layout_paths[i].path, value) == 0) {
+			active = i;
+			break;
+		}
 	}
 
 	g_free (value);
-	gtk_combo_box_set_active (GTK_COMBO_BOX (source->priv->layout_path_menu), i);
+	gtk_combo_box_set_active (GTK_COMBO_BOX (source->priv->layout_path_menu), active);
 
 	layout_example_label_update (source);
 }
@@ -1071,7 +1098,8 @@ rb_library_source_layout_filename_changed (GConfClient *client,
 					   RBLibrarySource *source)
 {
 	char *value;
-	int i = 0;
+	int active;
+	int i;
 
 	g_return_if_fail (entry != NULL);
 	g_return_if_fail (strcmp (entry->key, CONF_LIBRARY_LAYOUT_FILENAME) == 0);
@@ -1086,12 +1114,16 @@ rb_library_source_layout_filename_changed (GConfClient *client,
 		return;
 	}
 
-	while (library_layout_filenames[i].path && strcmp (library_layout_filenames[i].path, value) != 0) {
-		i++;
+	active = -1;
+	for (i = 0; library_layout_filenames[i].path != NULL; i++) {
+		if (strcmp (library_layout_filenames[i].path, value) == 0) {
+			active = i;
+			break;
+		}
 	}
-
 	g_free (value);
-	gtk_combo_box_set_active (GTK_COMBO_BOX (source->priv->layout_filename_menu), i);
+
+	gtk_combo_box_set_active (GTK_COMBO_BOX (source->priv->layout_filename_menu), active);
 
 	layout_example_label_update (source);
 }
@@ -1106,7 +1138,7 @@ rb_library_source_layout_filename_changed (GConfClient *client,
  * Stolen from Sound-Juicer
  */
 static char*
-build_filename (RBLibrarySource *source, RhythmDBEntry *entry)
+build_filename (RBLibrarySource *source, RhythmDBEntry *entry, const char *extension)
 {
 	GFile *library_location;
 	GFile *dir;
@@ -1115,19 +1147,16 @@ build_filename (RBLibrarySource *source, RhythmDBEntry *entry)
 	char *realpath;
 	char *filename;
 	char *string = NULL;
-	char *extension = NULL;
 	char *tmp;
 	GSList *list;
 	char *layout_path;
 	char *layout_filename;
-	char *preferred_format;
 
 	list = eel_gconf_get_string_list (CONF_LIBRARY_LOCATION);
 	layout_path = eel_gconf_get_string (CONF_LIBRARY_LAYOUT_PATH);
 	layout_filename = eel_gconf_get_string (CONF_LIBRARY_LAYOUT_FILENAME);
-	preferred_format = eel_gconf_get_string (CONF_LIBRARY_PREFERRED_FORMAT);
 
-	if (list == NULL || layout_path == NULL || layout_filename == NULL || preferred_format == NULL) {
+	if (list == NULL || layout_path == NULL || layout_filename == NULL) {
 		/* emit warning */
 		rb_debug ("Could not retrieve settings from GConf");
 		goto out;
@@ -1144,33 +1173,6 @@ build_filename (RBLibrarySource *source, RhythmDBEntry *entry)
 	g_object_unref (library_location);
 	g_free (realpath);
 
-	if (g_str_has_prefix (rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_MIMETYPE), "audio/x-raw")) {
-		GMAudioProfile *profile;
-		profile = gm_audio_profile_lookup (preferred_format);
-		if (profile)
-			extension = g_strdup (gm_audio_profile_get_extension (profile));
-	}
-
-	if (extension == NULL) {
-		const char *uri;
-		const char *loc;
-		char *tmp;
-
-		/* use the old extension. strip anything after a '?' for http/daap/etc */
-		uri = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_LOCATION);
-		loc = g_utf8_strrchr (uri, -1, '.');
-		if (loc == NULL)
-			loc = g_utf8_strrchr (uri, -1, '/');
-		if (loc == NULL)
-			loc = uri;
-
-		extension = g_strdup (loc + 1);
-
-		tmp = g_utf8_strchr (extension, -1, '?');
-		if (tmp)
-			*tmp = '\0';
-	}
-
 	realfile = filepath_parse_pattern (source->priv->db, layout_filename, entry);
 	if (extension) {
 		filename = g_strdup_printf ("%s.%s", realfile, extension);
@@ -1181,7 +1183,6 @@ build_filename (RBLibrarySource *source, RhythmDBEntry *entry)
 
 	dest = g_file_resolve_relative_path (dir, filename);
 	g_object_unref (dir);
-	g_free (extension);
 	g_free (filename);
 
 	string = g_file_get_uri (dest);
@@ -1190,7 +1191,6 @@ build_filename (RBLibrarySource *source, RhythmDBEntry *entry)
 	rb_slist_deep_free (list);
 	g_free (layout_path);
 	g_free (layout_filename);
-	g_free (preferred_format);
 
 	return string;
 }
@@ -1220,17 +1220,62 @@ impl_can_paste (RBSource *asource)
 	return can_paste;
 }
 
-static void
-completed_cb (RhythmDBEntry *entry, const char *dest, guint64 dest_size, GError *error, RBLibrarySource *source)
+static char *
+get_dest_uri_cb (RBTrackTransferBatch *batch,
+		 RhythmDBEntry *entry,
+		 const char *mediatype,
+		 const char *extension,
+		 RBLibrarySource *source)
 {
-	if (error == NULL) {
-		rhythmdb_add_uri (source->priv->db, dest);
-	} else {
-		if (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_EXISTS)) {
+	char *dest;
+	char *sane_dest;
+
+	dest = build_filename (source, entry, extension);
+	if (dest == NULL) {
+		rb_debug ("could not create destination path for entry");
+		return NULL;
+	}
+
+	sane_dest = rb_sanitize_uri_for_filesystem (dest);
+	g_free (dest);
+	rb_debug ("destination URI for %s is %s",
+		  rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_LOCATION),
+		  sane_dest);
+	return sane_dest;
+}
+
+static void
+track_done_cb (RBTrackTransferBatch *batch,
+	       RhythmDBEntry *entry,
+	       const char *dest,
+	       guint64 dest_size,
+	       const char *dest_mediatype,
+	       GError *error,
+	       RBLibrarySource *source)
+{
+	if (error != NULL) {
+		/* probably want to cancel the batch on some errors:
+		 * - out of disk space / read only
+		 * - source has vanished (hmm, how would we know?)
+		 *
+		 * and we probably want to do something intelligent about some other errors:
+		 * - encoder pipeline errors?  hmm.
+		 */
+		if (g_error_matches (error, RB_ENCODER_ERROR, RB_ENCODER_ERROR_OUT_OF_SPACE) ||
+		    g_error_matches (error, RB_ENCODER_ERROR, RB_ENCODER_ERROR_DEST_READ_ONLY)) {
+			rb_debug ("fatal transfer error: %s", error->message);
+			rb_track_transfer_batch_cancel (batch);
+			rb_error_dialog (NULL, _("Error transferring track"), "%s", error->message);
+		} else if (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_EXISTS)) {
 			rb_debug ("not displaying 'file exists' error for %s", dest);
 		} else {
 			rb_error_dialog (NULL, _("Error transferring track"), "%s", error->message);
 		}
+	} else {
+		/* could probably do something smarter here to avoid
+		 * re-reading tags etc.
+		 */
+		rhythmdb_add_uri (source->priv->db, dest);
 	}
 }
 
@@ -1238,11 +1283,13 @@ static void
 impl_paste (RBSource *asource, GList *entries)
 {
 	RBLibrarySource *source = RB_LIBRARY_SOURCE (asource);
-	RBRemovableMediaManager *rm_mgr;
+	RBTrackTransferQueue *xferq;
 	GList *l;
 	GSList *sl;
 	RBShell *shell;
 	RhythmDBEntryType source_entry_type;
+	RBTrackTransferBatch *batch;
+	gboolean start_batch = FALSE;
 
 	if (impl_can_paste (asource) == FALSE) {
 		g_warning ("RBLibrarySource impl_paste called when gconf keys unset");
@@ -1255,15 +1302,17 @@ impl_paste (RBSource *asource, GList *entries)
 		      "shell", &shell,
 		      "entry-type", &source_entry_type,
 		      NULL);
-	g_object_get (shell, "removable-media-manager", &rm_mgr, NULL);
+	g_object_get (shell, "track-transfer-queue", &xferq, NULL);
 	g_object_unref (shell);
 
+	batch = rb_track_transfer_batch_new (NULL, NULL, NULL, RB_SOURCE (source));
+	g_signal_connect_object (batch, "get-dest-uri", G_CALLBACK (get_dest_uri_cb), source, 0);
+	g_signal_connect_object (batch, "track-done", G_CALLBACK (track_done_cb), source, 0);
+
 	for (l = entries; l != NULL; l = g_list_next (l)) {
 		RhythmDBEntry *entry = (RhythmDBEntry *)l->data;
 		RhythmDBEntryType entry_type;
 		RBSource *source_source;
-		char *dest;
-		char *sane_dest;
 
 		rb_debug ("pasting entry %s", rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_LOCATION));
 
@@ -1280,22 +1329,18 @@ impl_paste (RBSource *asource, GList *entries)
 			continue;
 		}
 
-		dest = build_filename (source, entry);
-		if (dest == NULL) {
-			rb_debug ("could not create destination path for entry");
-			continue;
-		}
-
-		sane_dest = rb_sanitize_uri_for_filesystem (dest);
-		g_free (dest);
-
-		rb_removable_media_manager_queue_transfer (rm_mgr, entry,
-							  sane_dest, NULL,
-							  (RBTransferCompleteCallback)completed_cb, source);
+		rb_track_transfer_batch_add (batch, entry);
+		start_batch = TRUE;
 	}
 	g_boxed_free (RHYTHMDB_TYPE_ENTRY_TYPE, source_entry_type);
 
-	g_object_unref (rm_mgr);
+	if (start_batch) {
+		rb_track_transfer_queue_start_batch (xferq, batch);
+	} else {
+		g_object_unref (batch);
+	}
+
+	g_object_unref (xferq);
 }
 
 static guint
@@ -1350,11 +1395,13 @@ maybe_create_import_job (RBLibrarySource *source)
 {
 	RhythmDBImportJob *job;
 	if (source->priv->import_jobs == NULL || source->priv->start_import_job_id == 0) {
+
 		rb_debug ("creating new import job");
 		job = rhythmdb_import_job_new (source->priv->db,
 					       RHYTHMDB_ENTRY_TYPE_SONG,
 					       RHYTHMDB_ENTRY_TYPE_IGNORE,
 					       RHYTHMDB_ENTRY_TYPE_IMPORT_ERROR);
+
 		g_signal_connect_object (job,
 					 "status-changed",
 					 G_CALLBACK (import_job_status_changed_cb),
