@@ -223,7 +223,7 @@ static void rb_podcast_source_cmd_new_podcast	 (GtkAction *action,
 						  RBPodcastSource *source);
 static void rb_podcast_source_entry_changed_cb	(RhythmDB *db,
 						 RhythmDBEntry *entry,
-						 GSList *changes,
+						 GValueArray *changes,
 						 RBPodcastSource *source);
 static void rb_podcast_source_pixbuf_clicked_cb	(RBCellRendererPixbuf *renderer,
 						 const char *path,
@@ -255,6 +255,7 @@ static void impl_get_status				(RBSource *source,
 							 float *progress);
 static guint impl_want_uri				(RBSource *source, const char *uri);
 static gboolean impl_add_uri				(RBSource *source, const char *uri, const char *title, const char *genre);
+static char *impl_get_delete_action			(RBSource *source);
 
 #define CMD_PATH_SHOW_BROWSER "/commands/ShowBrowser"
 #define CMD_PATH_CURRENT_STATION "/commands/CurrentStation"
@@ -365,7 +366,6 @@ rb_podcast_source_class_init (RBPodcastSourceClass *klass)
 	source_class->impl_can_copy = (RBSourceFeatureFunc) rb_false_function;
 	source_class->impl_can_cut = (RBSourceFeatureFunc) rb_false_function;
 	source_class->impl_can_delete = (RBSourceFeatureFunc) rb_true_function;
-	source_class->impl_can_pause = (RBSourceFeatureFunc) rb_true_function;
 	source_class->impl_delete = impl_delete;
 	source_class->impl_get_browser_key  = impl_get_browser_key;
 	source_class->impl_get_config_widget = impl_get_config_widget;
@@ -380,6 +380,7 @@ rb_podcast_source_class_init (RBPodcastSourceClass *klass)
 	source_class->impl_get_status = impl_get_status;
 	source_class->impl_want_uri = impl_want_uri;
 	source_class->impl_add_uri = impl_add_uri;
+	source_class->impl_get_delete_action = impl_get_delete_action;
 
 	g_object_class_install_property (object_class,
 					 PROP_PODCAST_MANAGER,
@@ -934,12 +935,23 @@ impl_delete (RBSource *asource)
 	if (ret == GTK_RESPONSE_CANCEL || ret == GTK_RESPONSE_DELETE_EVENT)
 		return;
 
-	rb_podcast_manager_set_remove_files (source->priv->podcast_mgr,
-					     (ret == GTK_RESPONSE_YES));
-
 	entries = rb_entry_view_get_selected_entries (source->priv->posts);
 	for (l = entries; l != NULL; l = g_list_next (l)) {
-		rhythmdb_entry_delete (source->priv->db, l->data);
+		RhythmDBEntry *entry = l->data;
+
+		rb_podcast_manager_cancel_download (source->priv->podcast_mgr, entry);
+		if (ret == GTK_RESPONSE_YES) {
+			rb_podcast_manager_delete_download (source->priv->podcast_mgr, entry);
+		}
+
+		/* set podcast entries to invisible instead of deleted so they will
+		 * not reappear after the podcast has been updated
+		 */
+		GValue v = {0,};
+		g_value_init (&v, G_TYPE_BOOLEAN);
+		g_value_set_boolean (&v, TRUE);
+		rhythmdb_entry_set (source->priv->db, entry, RHYTHMDB_PROP_HIDDEN, &v);
+		g_value_unset (&v);
 	}
 
 	g_list_foreach (entries, (GFunc)rhythmdb_entry_unref, NULL);
@@ -1826,8 +1838,8 @@ rb_podcast_source_post_feed_cell_sort_func (RhythmDBEntry *a,
 	gint ret;
 
 	/* feeds */
-	a_str = rhythmdb_entry_get_string (a, RHYTHMDB_PROP_ALBUM);
-	b_str = rhythmdb_entry_get_string (b, RHYTHMDB_PROP_ALBUM);
+	a_str = rhythmdb_entry_get_string (a, RHYTHMDB_PROP_ALBUM_SORT_KEY);
+	b_str = rhythmdb_entry_get_string (b, RHYTHMDB_PROP_ALBUM_SORT_KEY);
 
 	ret = strcmp (a_str, b_str);
 	if (ret != 0)
@@ -1840,8 +1852,8 @@ rb_podcast_source_post_feed_cell_sort_func (RhythmDBEntry *a,
 		return (a_val > b_val) ? 1 : -1;
 
 	/* titles */
-	a_str = rhythmdb_entry_get_string (a, RHYTHMDB_PROP_TITLE);
-	b_str = rhythmdb_entry_get_string (b, RHYTHMDB_PROP_TITLE);
+	a_str = rhythmdb_entry_get_string (a, RHYTHMDB_PROP_TITLE_SORT_KEY);
+	b_str = rhythmdb_entry_get_string (b, RHYTHMDB_PROP_TITLE_SORT_KEY);
 
 	ret = strcmp (a_str, b_str);
 	if (ret != 0)
@@ -2063,20 +2075,21 @@ impl_add_uri (RBSource *asource, const char *uri, const char *title, const char
 static void
 rb_podcast_source_entry_changed_cb (RhythmDB *db,
 				    RhythmDBEntry *entry,
-				    GSList *changes,
+				    GValueArray *changes,
 				    RBPodcastSource *source)
 {
 	RhythmDBEntryType entry_type;
 	gboolean feed_changed;
-	GSList *t;
+	int i;
 
 	entry_type = rhythmdb_entry_get_entry_type (entry);
 	if (entry_type != RHYTHMDB_ENTRY_TYPE_PODCAST_FEED)
 		return;
 
 	feed_changed = FALSE;
-	for (t = changes; t; t = t->next) {
-		RhythmDBEntryChange *change = t->data;
+	for (i = 0; i < changes->n_values; i++) {
+		GValue *v = g_value_array_get_nth (changes, i);
+		RhythmDBEntryChange *change = g_value_get_boxed (v);
 
 		if (change->prop == RHYTHMDB_PROP_PLAYBACK_ERROR) {
 			feed_changed = TRUE;
@@ -2140,3 +2153,8 @@ rb_podcast_source_pixbuf_clicked_cb (RBCellRendererPixbuf *renderer,
 	gtk_tree_path_free (path);
 }
 
+static char *
+impl_get_delete_action (RBSource *source)
+{
+	return g_strdup ("EditDelete");
+}
