@@ -1,6 +1,4 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
- *
- *  arch-tag: Implementation of the abstract base class of all sources
  *
  *  Copyright (C) 2002 Jorn Baayen <jorn@nl.linux.org>
  *  Copyright (C) 2003 Colin Walters <walters@gnome.org>
@@ -78,6 +76,7 @@ static void default_get_status (RBSource *source, char **text, char **progress_t
 static void default_move_to_trash (RBSource *source);
 static GList * default_get_ui_actions (RBSource *source);
 static GList * default_get_search_actions (RBSource *source);
+static char *default_get_delete_action (RBSource *source);
 
 static void rb_source_post_entry_deleted_cb (GtkTreeModel *model,
 					     RhythmDBEntry *entry,
@@ -124,6 +123,8 @@ struct _RBSourcePrivate
 	RBSourceGroup *source_group;
 	RBPlugin *plugin;
 	RBSourceSearchType search_type;
+
+	gboolean deleted;
 };
 
 enum
@@ -176,6 +177,7 @@ rb_source_class_init (RBSourceClass *klass)
 	klass->impl_can_copy = (RBSourceFeatureFunc) rb_false_function;
 	klass->impl_can_add_to_queue = (RBSourceFeatureFunc) rb_false_function;
 	klass->impl_can_move_to_trash = (RBSourceFeatureFunc) rb_false_function;
+	klass->impl_can_pause = (RBSourceFeatureFunc) rb_true_function;
 	klass->impl_get_entry_view = default_get_entry_view;
 	klass->impl_copy = default_copy;
 	klass->impl_reset_filters = default_reset_filters;
@@ -191,6 +193,7 @@ rb_source_class_init (RBSourceClass *klass)
 	klass->impl_get_status = default_get_status;
 	klass->impl_get_ui_actions = default_get_ui_actions;
 	klass->impl_get_search_actions = default_get_search_actions;
+	klass->impl_get_delete_action = default_get_delete_action;
 	klass->impl_move_to_trash = default_move_to_trash;
 
 	/**
@@ -325,7 +328,7 @@ rb_source_class_init (RBSourceClass *klass)
 						 	      "RBPlugin",
 							      "RBPlugin instance for the plugin that created the source",
 							      RB_TYPE_PLUGIN,
-							      G_PARAM_READWRITE));
+							      G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
 	/**
 	 * RBSource:base-query-model:
 	 *
@@ -367,7 +370,6 @@ rb_source_class_init (RBSourceClass *klass)
 							    RB_TYPE_SOURCE_SEARCH_TYPE,
 							    RB_SOURCE_SEARCH_NONE,
 							    G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
-
 	/**
 	 * RBSource::deleted:
 	 * @source: the #RBSource
@@ -385,7 +387,7 @@ rb_source_class_init (RBSourceClass *klass)
 			      0);
 
 	/**
-	 * RBSource::status_changed:
+	 * RBSource::status-changed:
 	 * @source: the #RBSource
 	 *
 	 * Emitted when the source's status changes.
@@ -401,7 +403,7 @@ rb_source_class_init (RBSourceClass *klass)
 			      0);
 
 	/**
-	 * RBSource::filter_changed:
+	 * RBSource::filter-changed:
 	 * @source: the #RBSource
 	 *
 	 * Fires when the user changes the filter, either by changing the
@@ -544,8 +546,10 @@ queue_update_visibility (RBSource *source)
 /**
  * rb_source_set_hidden_when_empty:
  * @source: a #RBSource
- * @hidden: if TRUE, the source should not be displayed in
- *   the source list when there are no entries in it
+ * @hidden: if TRUE, automatically hide the source
+ *
+ * Enables or disables automatic hiding of the source when
+ * there are no entries in it.
  */
 void
 rb_source_set_hidden_when_empty (RBSource *source,
@@ -764,8 +768,9 @@ default_get_browser_key (RBSource *source)
  * rb_source_get_browser_key:
  * @source: a #RBSource
  *
- * Return value: the GConf key that determines browser visibility
- * for this source (allocated)
+ * Gets the GConf key controlling browser visibility
+ *
+ * Return value: the GConf key name (allocated)
  */
 char *
 rb_source_get_browser_key (RBSource *source)
@@ -779,6 +784,8 @@ rb_source_get_browser_key (RBSource *source)
  * rb_source_can_browse:
  * @source: a #RBSource
  *
+ * Determines whether the source has a browser
+ *
  * Return value: TRUE if this source has a browser
  */
 gboolean
@@ -875,6 +882,8 @@ rb_source_update_play_statistics (RBSource *source,
  * rb_source_get_entry_view:
  * @source: a #RBSource
  *
+ * Returns the entry view widget for the source.
+ *
  * Return value: the #RBEntryView instance for the source
  */
 RBEntryView *
@@ -895,8 +904,10 @@ default_get_property_views (RBSource *source)
  * rb_source_get_property_views:
  * @source: a #RBSource
  *
- * Return value:  a list containing the #RBPropertyView<!-- -->s that
- * make up the browser for this source, if any.
+ * Returns a list containing the #RBPropertyView instances for the
+ * source, if any.
+ *
+ * Return value:  list of property views
  */
 GList *
 rb_source_get_property_views (RBSource *source)
@@ -916,6 +927,8 @@ default_can_rename (RBSource *source)
  * rb_source_can_rename:
  * @source: a #RBSource.
  *
+ * Determines whether the source can be renamed.
+ *
  * Return value: TRUE if this source can be renamed
  */
 gboolean
@@ -934,7 +947,9 @@ rb_source_can_rename (RBSource *source)
 /**
  * rb_source_search:
  * @source: a #RBSource
- * @text: new search text
+ * @search: the active #RBSourceSearch instance (may be NULL)
+ * @cur_text: the current search text (may be NULL)
+ * @new_text: the new search text
  *
  * Updates the source with new search text.  The source
  * should recreate the database query that feeds into the
@@ -947,6 +962,7 @@ rb_source_search (RBSource *source,
 		  const char *new_text)
 {
 	RBSourceClass *klass = RB_SOURCE_GET_CLASS (source);
+	g_assert (new_text != NULL);
 
 	/* several sources don't have a search ability */
 	if (klass->impl_search != NULL)
@@ -981,8 +997,10 @@ rb_source_get_config_widget (RBSource *source,
  * rb_source_can_cut:
  * @source: a #RBSource
  *
- * Return value: TRUE if the source supports the typical cut
+ * Determines whether the source supporst the typical cut
  * (as in cut-and-paste) operation.
+ *
+ * Return value: TRUE if cutting is supported
  */
 gboolean
 rb_source_can_cut (RBSource *source)
@@ -996,8 +1014,9 @@ rb_source_can_cut (RBSource *source)
  * rb_source_can_paste:
  * @source: a #RBSource
  *
- * Return value: TRUE if the source supports the typical paste
- * (as in cut-and-paste) operation.
+ * Determines whether the source supports paste operations.
+ *
+ * Return value: TRUE if the pasting is supported
  */
 gboolean
 rb_source_can_paste (RBSource *source)
@@ -1011,8 +1030,10 @@ rb_source_can_paste (RBSource *source)
  * rb_source_can_delete:
  * @source: a #RBSource
  *
- * Return value: TRUE if the source allows the user to delete
+ * Determines whether the source allows the user to delete
  * a selected set of entries.
+ *
+ * Return value: TRUE if deletion is supported
  */
 gboolean
 rb_source_can_delete (RBSource *source)
@@ -1031,8 +1052,10 @@ rb_source_can_delete (RBSource *source)
  * rb_source_can_move_to_trash:
  * @source: a #RBSource
  *
- * Return value: TRUE if the source allows the user to trash
+ * Determines whether the source allows the user to trash
  * the files backing a selected set of entries.
+ *
+ * Return value: TRUE if trashing is supported
  */
 gboolean
 rb_source_can_move_to_trash (RBSource *source)
@@ -1051,8 +1074,10 @@ rb_source_can_move_to_trash (RBSource *source)
  * rb_source_can_copy:
  * @source: a #RBSource
  *
- * Return value: TRUE if the source supports the copy part
+ * Determines whether the source supports the copy part
  * of a copy-and-paste operation.
+ *
+ * Return value: TRUE if copying is supported
  */
 gboolean
 rb_source_can_copy (RBSource *source)
@@ -1083,13 +1108,20 @@ rb_source_cut (RBSource *source)
 static GList *
 default_copy (RBSource *source)
 {
-	return rb_entry_view_get_selected_entries (rb_source_get_entry_view (source));
+	RBEntryView *entry_view;
+	entry_view = rb_source_get_entry_view (source);
+	if (entry_view == NULL)
+		return NULL;
+
+	return rb_entry_view_get_selected_entries (entry_view);
 }
 
 /**
  * rb_source_copy:
  * @source: a #RBSource
  *
+ * Copies the selected entries to the clipboard.
+ *
  * Return value: a list containing the currently selected entries from
  * the source.
  */
@@ -1121,8 +1153,10 @@ rb_source_paste (RBSource *source, GList *entries)
  * rb_source_can_add_to_queue:
  * @source: a #RBSource
  *
- * Return value: TRUE if this source can add the current selected
- * set of entries to the play queue
+ * Determines whether the source can add the selected entries to
+ * the play queue.
+ *
+ * Return value: TRUE if adding to the play queue is supported
  */
 gboolean
 rb_source_can_add_to_queue (RBSource *source)
@@ -1135,10 +1169,15 @@ static void
 default_add_to_queue (RBSource *source,
 		      RBSource *queue)
 {
-	RBEntryView *songs = rb_source_get_entry_view (source);
-	GList *selection = rb_entry_view_get_selected_entries (songs);
+	RBEntryView *songs;
+	GList *selection;
 	GList *iter;
 
+	songs = rb_source_get_entry_view (source);
+	if (songs == NULL)
+		return;
+
+	selection = rb_entry_view_get_selected_entries (songs);
 	if (selection == NULL)
 		return;
 
@@ -1191,8 +1230,11 @@ default_move_to_trash (RBSource *source)
 
 	g_object_get (priv->shell, "db", &db, NULL);
 
+	sel = NULL;
 	entry_view = rb_source_get_entry_view (source);
-	sel = rb_entry_view_get_selected_entries (entry_view);
+	if (entry_view != NULL) {
+		sel = rb_entry_view_get_selected_entries (entry_view);
+	}
 
 	for (tem = sel; tem != NULL; tem = tem->next) {
 		rhythmdb_entry_move_to_trash (db, (RhythmDBEntry *)tem->data);
@@ -1244,8 +1286,10 @@ rb_source_reset_filters (RBSource *source)
  * rb_source_can_show_properties:
  * @source: a #RBSource
  *
- * Return value: TRUE if the source can display a properties
+ * Determines whether the source can display a properties
  * window for the currently selected entry (or set of entries)
+ *
+ * Return value: TRUE if showing properties is supported
  */
 gboolean
 rb_source_can_show_properties (RBSource *source)
@@ -1274,8 +1318,10 @@ rb_source_song_properties (RBSource *source)
  * rb_source_try_playlist:
  * @source: a #RBSource
  *
- * Return value: TRUE if the playback URIs for entries in the source
- *  should be parsed as playlists, rather than just played.
+ * Determines whether playback URIs for entries in the source should
+ * be parsed as playlists rather than just played.
+ *
+ * Return value: TRUE to attempt playlist parsing
  */
 gboolean
 rb_source_try_playlist (RBSource *source)
@@ -1352,7 +1398,10 @@ rb_source_add_uri (RBSource *source, const char *uri, const char *title, const c
  * rb_source_can_pause:
  * @source: a #RBSource
  *
- * Return value: TRUE if playback of entries from the source can be paused.
+ * Determines whether playback of entries from the source can
+ * be paused.
+ *
+ * Return value: TRUE if pausing is supported
  */
 gboolean
 rb_source_can_pause (RBSource *source)
@@ -1378,7 +1427,10 @@ default_handle_eos (RBSource *source)
  * rb_source_handle_eos:
  * @source: a #RBSource
  *
- * Return value: how EOS events should be handled for entries from this source
+ * Determines how EOS events should be handled when playing entries
+ * from the source.
+ *
+ * Return value: EOS event handling type
  */
 RBSourceEOFType
 rb_source_handle_eos (RBSource *source)
@@ -1474,8 +1526,16 @@ void
 rb_source_delete_thyself (RBSource *source)
 {
 	RBSourceClass *klass;
+	RBSourcePrivate *priv;
 
 	g_return_if_fail (source != NULL);
+	priv = RB_SOURCE_GET_PRIVATE (source);
+	if (priv->deleted) {
+		rb_debug ("source has already been deleted");
+		return;
+	}
+	priv->deleted = TRUE;
+
 	klass = RB_SOURCE_GET_CLASS (source);
 	klass->impl_delete_thyself (source);
 	g_signal_emit (G_OBJECT (source), rb_source_signals[DELETED], 0);
@@ -1575,6 +1635,29 @@ rb_source_get_search_actions (RBSource *source)
 	return klass->impl_get_search_actions (source);
 }
 
+static char *
+default_get_delete_action (RBSource *source)
+{
+	return g_strdup ("EditRemove");
+}
+
+/**
+ * rb_source_get_delete_action:
+ * @source: a #RBSource
+ *
+ * Returns the name of the UI action to use for 'delete'.
+ * This allows the source to customise the visible action name
+ * and description to better describe what deletion actually does.
+ *
+ * Return value: allocated string holding UI action name
+ */
+char *
+rb_source_get_delete_action (RBSource *source)
+{
+	RBSourceClass *klass = RB_SOURCE_GET_CLASS (source);
+	return klass->impl_get_delete_action (source);
+}
+
 static gboolean
 _update_status_idle (RBSource *source)
 {
@@ -1636,11 +1719,16 @@ GList *
 rb_source_gather_selected_properties (RBSource *source,
 				      RhythmDBPropType prop)
 {
+	RBEntryView *entryview;
 	GList *selected, *tem;
 	GHashTable *selected_set;
 
+	entryview = rb_source_get_entry_view (source);
+	if (entryview == NULL)
+		return NULL;
+
 	selected_set = g_hash_table_new (g_str_hash, g_str_equal);
-	selected = rb_entry_view_get_selected_entries (rb_source_get_entry_view (RB_SOURCE (source)));
+	selected = rb_entry_view_get_selected_entries (entryview);
 
 	for (tem = selected; tem; tem = tem->next) {
 		RhythmDBEntry *entry = tem->data;
@@ -1658,6 +1746,26 @@ rb_source_gather_selected_properties (RBSource *source,
 	return tem;
 }
 
+static GtkActionGroup *
+find_action_group (GtkUIManager *uimanager, const char *group_name)
+{
+	GList *actiongroups;
+	GList *i;
+	actiongroups = gtk_ui_manager_get_action_groups (uimanager);
+
+	/* Don't create the action group if it's already registered */
+	for (i = actiongroups; i != NULL; i = i->next) {
+		const char *name;
+
+		name = gtk_action_group_get_name (GTK_ACTION_GROUP (i->data));
+		if (name != NULL && strcmp (name, group_name) == 0) {
+			return GTK_ACTION_GROUP (i->data);
+		}
+	}
+
+	return NULL;
+}
+
 /**
  * _rb_source_register_action_group:
  * @source: a #RBSource
@@ -1678,42 +1786,123 @@ _rb_source_register_action_group (RBSource *source,
 				  gpointer user_data)
 {
 	GtkUIManager *uimanager;
-	GList *actiongroups;
-	GList *i;
 	GtkActionGroup *group;
 
 	g_return_val_if_fail (group_name != NULL, NULL);
 
 	g_object_get (source, "ui-manager", &uimanager, NULL);
-	actiongroups = gtk_ui_manager_get_action_groups (uimanager);
-
-	/* Don't create the action group if it's already registered */
-	for (i = actiongroups; i != NULL; i = i->next) {
-		const char *name;
-
-		name = gtk_action_group_get_name (GTK_ACTION_GROUP (i->data));
-		if (name != NULL && strcmp (name, group_name) == 0) {
-			group = GTK_ACTION_GROUP (i->data);
-			/* Add a reference */
-			g_object_ref (group);
-			goto out;
+	group = find_action_group (uimanager, group_name);
+	if (group == NULL) {
+		group = gtk_action_group_new (group_name);
+		gtk_action_group_set_translation_domain (group,
+							 GETTEXT_PACKAGE);
+		if (actions != NULL) {
+			gtk_action_group_add_actions (group,
+						      actions, num_actions,
+						      user_data);
 		}
+		gtk_ui_manager_insert_action_group (uimanager, group, 0);
+	} else {
+		g_object_ref (group);
 	}
-
-	group = gtk_action_group_new (group_name);
-	gtk_action_group_set_translation_domain (group,
-						 GETTEXT_PACKAGE);
-	gtk_action_group_add_actions (group,
-				      actions, num_actions,
-				      user_data);
-	gtk_ui_manager_insert_action_group (uimanager, group, 0);
-
- out:
 	g_object_unref (uimanager);
 
 	return group;
 }
 
+typedef void (*SourceActionCallback) (GtkAction *action, RBSource *source);
+
+typedef struct {
+	SourceActionCallback callback;
+	/*RBShell *shell;*/
+	gpointer shell;
+} SourceActionData;
+
+static void
+source_action_data_destroy (SourceActionData *data)
+{
+	if (data->shell != NULL) {
+		g_object_remove_weak_pointer (G_OBJECT (data->shell), &data->shell);
+	}
+	g_slice_free (SourceActionData, data);
+}
+
+static void
+source_action_cb (GtkAction *action, SourceActionData *data)
+{
+	RBSource *source;
+
+	if (data->shell == NULL) {
+		return;
+	}
+
+	/* get current source */
+	g_object_get (G_OBJECT (data->shell), "selected-source", &source, NULL);
+	if (source != NULL) {
+		data->callback (action, source);
+		g_object_unref (source);
+	}
+}
+
+/**
+ * _rb_action_group_add_source_actions:
+ * @group: a #GtkActionGroup
+ * @shell: the #RBShell
+ * @actions: array of GtkActionEntry structures for the action group
+ * @num_actions: number of actions in the @actions array
+ *
+ * Adds actions to an action group where the action callback is
+ * called with the current selected source.  This can safely be called
+ * multiple times on the same action group.
+ */
+void
+_rb_action_group_add_source_actions (GtkActionGroup *group,
+				     GObject *shell,
+				     GtkActionEntry *actions,
+				     int num_actions)
+{
+	int i;
+	for (i = 0; i < num_actions; i++) {
+		GtkAction *action;
+		const char *label;
+		const char *tooltip;
+		SourceActionData *source_action_data;
+
+		if (gtk_action_group_get_action (group, actions[i].name) != NULL) {
+			/* action was already added */
+			continue;
+		}
+
+		label = gtk_action_group_translate_string (group, actions[i].label);
+		tooltip = gtk_action_group_translate_string (group, actions[i].tooltip);
+
+		action = gtk_action_new (actions[i].name, label, tooltip, NULL);
+		if (actions[i].stock_id != NULL) {
+			g_object_set (action, "stock-id", actions[i].stock_id, NULL);
+			if (gtk_icon_theme_has_icon (gtk_icon_theme_get_default (),
+						     actions[i].stock_id)) {
+				g_object_set (action, "icon-name", actions[i].stock_id, NULL);
+			}
+		}
+
+		if (actions[i].callback) {
+			GClosure *closure;
+			source_action_data = g_slice_new0 (SourceActionData);
+			source_action_data->callback = (SourceActionCallback) actions[i].callback;
+			source_action_data->shell = shell;
+			g_object_add_weak_pointer (shell, &source_action_data->shell);
+
+			closure = g_cclosure_new (G_CALLBACK (source_action_cb),
+						  source_action_data,
+						  (GClosureNotify) source_action_data_destroy);
+			g_signal_connect_closure (action, "activate", closure, FALSE);
+		}
+
+		gtk_action_group_add_action_with_accel (group, action, actions[i].accelerator);
+		g_object_unref (action);
+	}
+}
+
 /**
  * _rb_source_check_entry_type:
  * @source: a #RBSource
@@ -1738,6 +1927,30 @@ _rb_source_check_entry_type (RBSource *source, RhythmDBEntry *entry)
 	return ret;
 }
 
+/**
+ * _rb_source_set_import_status:
+ * @source: an #RBSource
+ * @job: a #RhythmDBImportJob
+ * @progress_text: used to return progress text
+ * @progress: used to return progress fraction
+ *
+ * Used in implementations of the get_status method to provide source
+ * status information based on a #RhythmDBImportJob.
+ */
+void
+_rb_source_set_import_status (RBSource *source, RhythmDBImportJob *job, char **progress_text, float *progress)
+{
+	int total;
+	int imported;
+
+	total = rhythmdb_import_job_get_total (job);
+	imported = rhythmdb_import_job_get_imported (job);
+
+	g_free (*progress_text);
+	*progress_text = g_strdup_printf (_("Importing (%d/%d)"), imported, total);
+	*progress = ((float)imported / (float)total);
+}
+
 /* This should really be standard. */
 #define ENUM_ENTRY(NAME, DESC) { NAME, "" #NAME "", DESC }
 
@@ -1748,10 +1961,10 @@ rb_source_eof_type_get_type (void)
 
 	if (etype == 0)	{
 		static const GEnumValue values[] = {
-			ENUM_ENTRY (RB_SOURCE_EOF_ERROR, "Display error when playing entry ends"),
-			ENUM_ENTRY (RB_SOURCE_EOF_STOP, "Stop playback when playing entry ends"),
-			ENUM_ENTRY (RB_SOURCE_EOF_RETRY, "Restart playing when playing entry ends"),
-			ENUM_ENTRY (RB_SOURCE_EOF_NEXT, "Start next entry when playing entry ends"),
+			ENUM_ENTRY (RB_SOURCE_EOF_ERROR, "error"),
+			ENUM_ENTRY (RB_SOURCE_EOF_STOP, "stop"),
+			ENUM_ENTRY (RB_SOURCE_EOF_RETRY, "retry"),
+			ENUM_ENTRY (RB_SOURCE_EOF_NEXT, "next"),
 			{ 0, 0, 0 }
 		};
 
@@ -1768,9 +1981,9 @@ rb_source_search_type_get_type (void)
 
 	if (etype == 0) {
 		static const GEnumValue values[] = {
-			ENUM_ENTRY (RB_SOURCE_SEARCH_NONE, "No search capability"),
-			ENUM_ENTRY (RB_SOURCE_SEARCH_INCREMENTAL, "Immediate incremental search"),
-			ENUM_ENTRY (RB_SOURCE_SEARCH_EXPLICIT, "Explicitly activated search"),
+			ENUM_ENTRY (RB_SOURCE_SEARCH_NONE, "none"),
+			ENUM_ENTRY (RB_SOURCE_SEARCH_INCREMENTAL, "incremental"),
+			ENUM_ENTRY (RB_SOURCE_SEARCH_EXPLICIT, "explicit"),
 			{ 0, 0, 0 }
 		};
 
