@@ -1,6 +1,4 @@
 /*
- *  arch-tag: Header for the abstract base class of all sources
- *
  *  Copyright (C) 2002 Jorn Baayen <jorn@nl.linux.org>
  *  Copyright (C) 2003 Colin Walters <walters@gnome.org>
  *
@@ -33,10 +31,11 @@
 
 #include <gtk/gtk.h>
 
-#include "rb-source-group.h"
-#include "rb-source-search.h"
-#include "rb-entry-view.h"
-#include "rb-shell-preferences.h"
+#include <sources/rb-source-group.h>
+#include <sources/rb-source-search.h>
+#include <widgets/rb-entry-view.h>
+#include <shell/rb-shell-preferences.h>
+#include <rhythmdb/rhythmdb-import-job.h>
 
 G_BEGIN_DECLS
 
@@ -53,6 +52,12 @@ typedef enum {
 	RB_SOURCE_SEARCH_EXPLICIT,
 } RBSourceSearchType;
 
+typedef struct _RBSource	RBSource;
+typedef struct _RBSourceClass	RBSourceClass;
+typedef struct _RBSourcePrivate	RBSourcePrivate;
+
+typedef void (*RBSourceActionCallback) (GtkAction *action, RBSource *source);
+
 GType rb_source_eof_type_get_type (void);
 #define RB_TYPE_SOURCE_EOF_TYPE	(rb_source_eof_type_get_type())
 
@@ -68,10 +73,6 @@ GType rb_source_search_type_get_type (void);
 
 #define RB_SOURCE_ICON_SIZE	GTK_ICON_SIZE_LARGE_TOOLBAR
 
-typedef struct _RBSource	RBSource;
-typedef struct _RBSourceClass	RBSourceClass;
-typedef struct _RBSourcePrivate	RBSourcePrivate;
-
 typedef gboolean (*RBSourceFeatureFunc) (RBSource *source);
 typedef const char * (*RBSourceStringFunc) (RBSource *source);
 
@@ -139,6 +140,7 @@ struct _RBSourceClass
 	void		(*impl_deactivate)	(RBSource *source);
 	GList *		(*impl_get_ui_actions)	(RBSource *source);
 	GList *		(*impl_get_search_actions) (RBSource *source);
+	char *		(*impl_get_delete_action) (RBSource *source);
 };
 
 GType		rb_source_get_type		(void);
@@ -209,6 +211,7 @@ void		rb_source_deactivate		(RBSource *source);
 
 GList *		rb_source_get_ui_actions	(RBSource *source);
 GList *		rb_source_get_search_actions	(RBSource *source);
+char *		rb_source_get_delete_action	(RBSource *source);
 
 GList *		rb_source_gather_selected_properties (RBSource *source, RhythmDBPropType prop);
 
@@ -223,9 +226,19 @@ GtkActionGroup *_rb_source_register_action_group (RBSource *source,
 						  GtkActionEntry *actions,
 						  int num_actions,
 						  gpointer user_data);
+void		_rb_action_group_add_source_actions (GtkActionGroup *group,
+						     GObject *shell,
+						     GtkActionEntry *actions,
+						     int num_actions);
+
 gboolean	_rb_source_check_entry_type	(RBSource *source,
 						 RhythmDBEntry *entry);
 
+void		_rb_source_set_import_status	(RBSource *source,
+						 RhythmDBImportJob *job,
+						 char **progress_text,
+						 float *progress);
+
 G_END_DECLS
 
 #endif /* __RB_SOURCE_H */
