@@ -27,6 +27,21 @@
  *
  */
 
+/**
+ * SECTION:rb-static-playlist-source
+ * @short_description: Manually defined playlist class
+ *
+ * Static playlists are not defined by a query, but instead by manually selected
+ * and ordered tracks.
+ *
+ * This class is used for static playlists built from the user's library, and is
+ * also a base class for the play queue and for playlists on devices and network
+ * shares.
+ *
+ * It has some ability to track locations that are not yet present in the database
+ * and to add them to the playlist once they are added.
+ */
+
 #include "config.h"
 
 #include <string.h>
@@ -336,6 +351,18 @@ rb_static_playlist_source_constructed (GObject *object)
 	gtk_widget_show_all (GTK_WIDGET (source));
 }
 
+/**
+ * rb_static_playlist_source_new:
+ * @shell: the #RBShell
+ * @name: the playlist name
+ * @sorting_name: the sorting name for the playlist (GConf key friendly)
+ * @local: if %TRUE, the playlist is local to the library
+ * @entry_type: type of database entries that can be added to the playlist.
+ *
+ * Creates a new static playlist source.
+ *
+ * Return value: new playlist.
+ */
 RBSource *
 rb_static_playlist_source_new (RBShell *shell, const char *name, const char *sorting_name, gboolean local, RhythmDBEntryType entry_type)
 {
@@ -389,6 +416,13 @@ rb_static_playlist_source_get_property (GObject *object,
 	}
 }
 
+/**
+ * rb_static_playlist_source_load_from_xml:
+ * @source: an #RBStaticPlaylistSource
+ * @node: XML node to load from
+ *
+ * Loads the playlist contents from the specified XML document node.
+ */
 void
 rb_static_playlist_source_load_from_xml (RBStaticPlaylistSource *source, xmlNodePtr node)
 {
@@ -410,6 +444,15 @@ rb_static_playlist_source_load_from_xml (RBStaticPlaylistSource *source, xmlNode
 	}
 }
 
+/**
+ * rb_static_playlist_source_new_from_xml:
+ * @shell: the #RBShell
+ * @node: XML node containing playlist entries
+ *
+ * Constructs a new playlist from the given XML document node.
+ *
+ * Return value: playlist read from XML
+ */
 RBSource *
 rb_static_playlist_source_new_from_xml (RBShell *shell, xmlNodePtr node)
 {
@@ -589,16 +632,19 @@ rb_static_playlist_source_browser_changed_cb (RBLibraryBrowser *browser,
 static gboolean
 impl_receive_drag (RBSource *asource, GtkSelectionData *data)
 {
+	GdkAtom type;
 	GList *list;
 	RBStaticPlaylistSource *source = RB_STATIC_PLAYLIST_SOURCE (asource);
 
-        if (data->type == gdk_atom_intern ("text/uri-list", TRUE) ||
-	    data->type == gdk_atom_intern ("application/x-rhythmbox-entry", TRUE)) {
-		list = rb_uri_list_parse ((char *)data->data);
+	type = gtk_selection_data_get_data_type (data);
+
+        if (type == gdk_atom_intern ("text/uri-list", TRUE) ||
+	    type == gdk_atom_intern ("application/x-rhythmbox-entry", TRUE)) {
+		list = rb_uri_list_parse ((char *)gtk_selection_data_get_data (data));
 		if (list == NULL)
 			return FALSE;
 
-		if (data->type == gdk_atom_intern ("text/uri-list", TRUE))
+		if (type == gdk_atom_intern ("text/uri-list", TRUE))
 			rb_static_playlist_source_add_uri_list (source, list);
 		else
 			rb_static_playlist_source_add_id_list (source, list);
@@ -742,6 +788,16 @@ _add_location_cb (GFile *file,
 	return TRUE;	
 }
 
+/**
+ * rb_static_playlist_source_add_location:
+ * @source: an #RBStaticPlaylistSource
+ * @location: location (URI) to add to the playlist
+ * @index: position at which to add the location (-1 to add at the end)
+ *
+ * If the location matches an entry in the database, the entry is added
+ * to the playlist.  Otherwise, if it identifies a directory, the contents
+ * of that directory are added.
+ */
 void
 rb_static_playlist_source_add_location (RBStaticPlaylistSource *source,
 					const char *location,
@@ -764,6 +820,14 @@ rb_static_playlist_source_add_location (RBStaticPlaylistSource *source,
 
 }
 
+/**
+ * rb_static_playlist_source_add_locations:
+ * @source: an #RBStaticPlaylistSource
+ * @locations: a #GList of strings to add
+ *
+ * Adds the locations specified in @locations to the playlist.
+ * See @rb_static_playlist_source_add_location for details.
+ */
 void
 rb_static_playlist_source_add_locations (RBStaticPlaylistSource *source,
 					 GList *locations)
@@ -776,6 +840,15 @@ rb_static_playlist_source_add_locations (RBStaticPlaylistSource *source,
 	}
 }
 
+/**
+ * rb_static_playlist_source_remove_location:
+ * @source: an #RBStaticPlaylistSource
+ * @location: location to remove
+ *
+ * Removes the specified location from the playlist.  This affects both
+ * the location map and the query model, whether an entry exists for the
+ * location or not.
+ */
 void
 rb_static_playlist_source_remove_location (RBStaticPlaylistSource *source,
 					   const char *location)
@@ -798,6 +871,14 @@ rb_static_playlist_source_remove_location (RBStaticPlaylistSource *source,
 	}
 }
 
+/**
+ * rb_static_playlist_source_add_entry:
+ * @source: an #RBStaticPlaylistSource
+ * @entry: entry to add to the playlist
+ * @index: position at which to add it (-1 to add at the end)
+ *
+ * Adds the specified entry to the playlist.
+ */
 void
 rb_static_playlist_source_add_entry (RBStaticPlaylistSource *source,
 				     RhythmDBEntry *entry,
@@ -809,6 +890,13 @@ rb_static_playlist_source_add_entry (RBStaticPlaylistSource *source,
 	rb_static_playlist_source_add_location_internal (source, location, index);
 }
 
+/**
+ * rb_static_playlist_source_remove_entry:
+ * @source: an #RBStaticPlaylistSource
+ * @entry: the entry to remove
+ *
+ * Removes the specified entry from the playlist.
+ */
 void
 rb_static_playlist_source_remove_entry (RBStaticPlaylistSource *source,
 					RhythmDBEntry *entry)
@@ -819,6 +907,14 @@ rb_static_playlist_source_remove_entry (RBStaticPlaylistSource *source,
 	rb_static_playlist_source_remove_location (source, location);
 }
 
+/**
+ * rb_static_playlist_source_move_entry:
+ * @source: an #RBStaticPlaylistSource
+ * @entry: the entry to move
+ * @index: new location for the entry
+ *
+ * Moves an entry within the playlist.
+ */
 void
 rb_static_playlist_source_move_entry (RBStaticPlaylistSource *source,
 				      RhythmDBEntry *entry,
@@ -906,4 +1002,3 @@ impl_want_uri (RBSource *source, const char *uri)
 
 	return 0;
 }
-
