@@ -54,6 +54,21 @@
 
 /**
  * RBEntryViewColumn:
+ * @RB_ENTRY_VIEW_COL_TRACK_NUMBER: the track number column
+ * @RB_ENTRY_VIEW_COL_TITLE: the title column
+ * @RB_ENTRY_VIEW_COL_ARTIST: the artist column
+ * @RB_ENTRY_VIEW_COL_ALBUM: the album column
+ * @RB_ENTRY_VIEW_COL_GENRE: the genre column
+ * @RB_ENTRY_VIEW_COL_DURATION: the duration column
+ * @RB_ENTRY_VIEW_COL_QUALITY: the quality (bitrate) column
+ * @RB_ENTRY_VIEW_COL_RATING: the rating column
+ * @RB_ENTRY_VIEW_COL_PLAY_COUNT: the play count column
+ * @RB_ENTRY_VIEW_COL_YEAR: the year (release date) column
+ * @RB_ENTRY_VIEW_COL_LAST_PLAYED: the last played time column
+ * @RB_ENTRY_VIEW_COL_FIRST_SEEN: the first seen (imported) column
+ * @RB_ENTRY_VIEW_COL_LAST_SEEN: the last seen column
+ * @RB_ENTRY_VIEW_COL_LOCATION: the location column
+ * @RB_ENTRY_VIEW_COL_ERROR: the error column
  *
  * Predefined column types to use in #RBEntryView<!-- -->s.  Use
  * #rb_entry_view_append_column to add these to an entry view.
@@ -86,6 +101,7 @@
 #include "eel-gconf-extensions.h"
 #include "rb-shell-player.h"
 #include "rb-cut-and-paste-code.h"
+#include "gseal-gtk-compat.h"
 
 static const GtkTargetEntry rb_entry_view_drag_types[] = {
 	{ "application/x-rhythmbox-entry", 0, 0 },
@@ -946,6 +962,31 @@ rb_entry_view_rating_cell_data_func (GtkTreeViewColumn *column,
 	rhythmdb_entry_unref (entry);
 }
 
+static void
+rb_entry_view_bpm_cell_data_func (GtkTreeViewColumn *column,
+				   GtkCellRenderer *renderer,
+				   GtkTreeModel *tree_model,
+				   GtkTreeIter *iter,
+				   struct RBEntryViewCellDataFuncData *data)
+{
+	RhythmDBEntry *entry;
+	char *str;
+	gdouble val;
+
+	entry = rhythmdb_query_model_iter_to_entry (data->view->priv->model, iter);
+
+	val = rhythmdb_entry_get_double (entry, data->propid);
+
+	if (val > 0.001)
+		str = g_strdup_printf ("%.2f", val);
+	else
+		str = g_strdup ("");
+
+	g_object_set (renderer, "text", str, NULL);
+	g_free (str);
+	rhythmdb_entry_unref (entry);
+}
+
 static void
 rb_entry_view_long_cell_data_func (GtkTreeViewColumn *column,
 				   GtkCellRenderer *renderer,
@@ -1332,6 +1373,9 @@ rb_entry_view_get_column (RBEntryView *view, RBEntryViewColumn coltype)
 	case RB_ENTRY_VIEW_COL_GENRE:
 		propid = RHYTHMDB_PROP_GENRE;
 		break;
+	case RB_ENTRY_VIEW_COL_COMMENT:
+		propid = RHYTHMDB_PROP_COMMENT;
+		break;
 	case RB_ENTRY_VIEW_COL_DURATION:
 		propid = RHYTHMDB_PROP_DURATION;
 		break;
@@ -1390,6 +1434,7 @@ rb_entry_view_cell_edited_cb (GtkCellRendererText *renderer,
 	case RHYTHMDB_PROP_GENRE:
 	case RHYTHMDB_PROP_ARTIST:
 	case RHYTHMDB_PROP_ALBUM:
+	case RHYTHMDB_PROP_COMMENT:
 	case RHYTHMDB_PROP_ARTIST_SORTNAME:
 	case RHYTHMDB_PROP_ALBUM_SORTNAME:
 		break;
@@ -1504,6 +1549,16 @@ rb_entry_view_append_column (RBEntryView *view,
 		key = "Genre";
 		ellipsize = TRUE;
 		break;
+	case RB_ENTRY_VIEW_COL_COMMENT:
+		propid = RHYTHMDB_PROP_COMMENT;
+		cell_data->propid = propid;
+		cell_data_func = (GtkTreeCellDataFunc) rb_entry_view_string_cell_data_func;
+		sort_propid = cell_data->propid;
+		sort_func = (GCompareDataFunc) rhythmdb_query_model_string_sort_func;
+		title = _("Comment");
+		key = "Comment";
+		ellipsize = TRUE;
+		break;
 	case RB_ENTRY_VIEW_COL_DURATION:
 		propid = RHYTHMDB_PROP_DURATION;
 		cell_data->propid = propid;
@@ -1620,6 +1675,16 @@ rb_entry_view_append_column (RBEntryView *view,
 		key = "Location";
 		ellipsize = TRUE;
 		break;
+	case RB_ENTRY_VIEW_COL_BPM:
+		propid = RHYTHMDB_PROP_BPM;
+		cell_data->propid = propid;
+		cell_data_func = (GtkTreeCellDataFunc) rb_entry_view_bpm_cell_data_func;
+		sort_func = (GCompareDataFunc) rhythmdb_query_model_double_ceiling_sort_func;
+		title = _("BPM");
+		key = "BPM";
+		strings[0] = title;
+		strings[1] = "999.99";
+		break;
 	case RB_ENTRY_VIEW_COL_ERROR:
 		propid = RHYTHMDB_PROP_PLAYBACK_ERROR;
 		cell_data->propid = RHYTHMDB_PROP_PLAYBACK_ERROR;
@@ -1647,6 +1712,7 @@ rb_entry_view_append_column (RBEntryView *view,
 		g_signal_connect_object (renderer, "edited",
 					 G_CALLBACK (rb_entry_view_cell_edited_cb),
 					 view, 0);
+		g_object_set (renderer, "single-paragraph-mode", TRUE, NULL);
 	} else {
 		g_free (cell_data);
 	}
@@ -1995,8 +2061,10 @@ harvest_entries (GtkTreeModel *model,
  * rb_entry_view_get_selected_entries:
  * @view: a #RBEntryView
  *
- * Return value: a #GList containing the currently selected entries in the view
- *  (must be freed)
+ * Gathers the selected entries from the view.
+ *
+ * Return value: a #GList of selected entries in the view
+ *  (must be freed and the entries unreffed)
  */
 GList *
 rb_entry_view_get_selected_entries (RBEntryView *view)
@@ -2092,7 +2160,9 @@ rb_entry_view_selection_changed_cb (GtkTreeSelection *selection,
  * rb_entry_view_have_selection:
  * @view: a #RBEntryView
  *
- * Return value: TRUE if one or more rows are selected in the view
+ * Determines whether there is an active selection in the view.
+ *
+ * Return value: TRUE if one or more rows are selected
  */
 gboolean
 rb_entry_view_have_selection (RBEntryView *view)
@@ -2104,6 +2174,8 @@ rb_entry_view_have_selection (RBEntryView *view)
  * rb_entry_view_have_complete_selection:
  * @view: a #RBEntryView
  *
+ * Determines whether all entries in the view are selected.
+ *
  * Return value: TRUE if all rows in the view are selected
  */
 gboolean
@@ -2289,7 +2361,7 @@ rb_entry_view_scroll_to_iter (RBEntryView *view,
 	 * view to the playing entry before the view has ever been displayed.
 	 * This will result in gtk+ warnings, so we avoid it in this case.
 	 */
-	if (!GTK_WIDGET_REALIZED (view))
+	if (!gtk_widget_get_realized (GTK_WIDGET (view)))
 		return;
 
 	path = gtk_tree_model_get_path (GTK_TREE_MODEL (view->priv->model), iter);
@@ -2307,7 +2379,10 @@ rb_entry_view_scroll_to_iter (RBEntryView *view,
  * @view: a #RBEntryView
  * @entry: a #RhythmDBEntry to check
  *
- * Return value: TRUE if the entry is present in the view and is currently visible
+ * Determines whether a specified entry is present in the view
+ * and is currently visible.
+ *
+ * Return value: TRUE if the entry is visible
  */
 gboolean
 rb_entry_view_get_entry_visible (RBEntryView *view,
@@ -2329,6 +2404,8 @@ rb_entry_view_get_entry_visible (RBEntryView *view,
  * @view: a #RBEntryView
  * @entry: a #RhythmDBEntry to check
  *
+ * Determines whether a specified entry is present in the view.
+ *
  * Return value: TRUE if the entry is present in the view
  */
 gboolean
@@ -2356,7 +2433,7 @@ rb_entry_view_entry_is_visible (RBEntryView *view,
 
 	g_return_if_fail (entry != NULL);
 
-	if (!GTK_WIDGET_REALIZED (view))
+	if (!gtk_widget_get_realized (GTK_WIDGET (view)))
 		return;
 
 	*realized = TRUE;
@@ -2635,21 +2712,23 @@ rb_entry_view_column_get_type (void)
 
 	if (etype == 0)	{
 		static const GEnumValue values[] = {
-			ENUM_ENTRY (RB_ENTRY_VIEW_COL_TRACK_NUMBER, "Track Number"),
-			ENUM_ENTRY (RB_ENTRY_VIEW_COL_TITLE, "Title"),
-			ENUM_ENTRY (RB_ENTRY_VIEW_COL_ARTIST, "Artist"),
-			ENUM_ENTRY (RB_ENTRY_VIEW_COL_ALBUM, "Album"),
-			ENUM_ENTRY (RB_ENTRY_VIEW_COL_GENRE, "Genre"),
-			ENUM_ENTRY (RB_ENTRY_VIEW_COL_DURATION, "Duration"),
-			ENUM_ENTRY (RB_ENTRY_VIEW_COL_QUALITY, "Quality"),
-			ENUM_ENTRY (RB_ENTRY_VIEW_COL_RATING, "Rating"),
-			ENUM_ENTRY (RB_ENTRY_VIEW_COL_PLAY_COUNT, "Play Count"),
-			ENUM_ENTRY (RB_ENTRY_VIEW_COL_YEAR, "Year"),
-			ENUM_ENTRY (RB_ENTRY_VIEW_COL_LAST_PLAYED, "Last Played"),
-			ENUM_ENTRY (RB_ENTRY_VIEW_COL_FIRST_SEEN, "First Seen"),
-			ENUM_ENTRY (RB_ENTRY_VIEW_COL_LAST_SEEN, "Last Seen"),
-			ENUM_ENTRY (RB_ENTRY_VIEW_COL_LOCATION, "Location"),
-			ENUM_ENTRY (RB_ENTRY_VIEW_COL_ERROR, "Error"),
+			ENUM_ENTRY (RB_ENTRY_VIEW_COL_TRACK_NUMBER, "track-number"),
+			ENUM_ENTRY (RB_ENTRY_VIEW_COL_TITLE, "title"),
+			ENUM_ENTRY (RB_ENTRY_VIEW_COL_ARTIST, "artist"),
+			ENUM_ENTRY (RB_ENTRY_VIEW_COL_ALBUM, "album"),
+			ENUM_ENTRY (RB_ENTRY_VIEW_COL_GENRE, "genre"),
+			ENUM_ENTRY (RB_ENTRY_VIEW_COL_COMMENT, "comment"),
+			ENUM_ENTRY (RB_ENTRY_VIEW_COL_DURATION, "duration"),
+			ENUM_ENTRY (RB_ENTRY_VIEW_COL_QUALITY, "quality"),
+			ENUM_ENTRY (RB_ENTRY_VIEW_COL_RATING, "rating"),
+			ENUM_ENTRY (RB_ENTRY_VIEW_COL_PLAY_COUNT, "play-count"),
+			ENUM_ENTRY (RB_ENTRY_VIEW_COL_YEAR, "year"),
+			ENUM_ENTRY (RB_ENTRY_VIEW_COL_LAST_PLAYED, "last-played"),
+			ENUM_ENTRY (RB_ENTRY_VIEW_COL_FIRST_SEEN, "first-seen"),
+			ENUM_ENTRY (RB_ENTRY_VIEW_COL_LAST_SEEN, "last-seen"),
+			ENUM_ENTRY (RB_ENTRY_VIEW_COL_LOCATION, "location"),
+			ENUM_ENTRY (RB_ENTRY_VIEW_COL_BPM, "bpm"),
+			ENUM_ENTRY (RB_ENTRY_VIEW_COL_ERROR, "error"),
 			{ 0, 0, 0 }
 		};
 
@@ -2666,9 +2745,9 @@ rb_entry_view_state_get_type (void)
 
 	if (etype == 0)	{
 		static const GEnumValue values[] = {
-			ENUM_ENTRY (RB_ENTRY_VIEW_NOT_PLAYING, "Not Playing"),
-			ENUM_ENTRY (RB_ENTRY_VIEW_PLAYING, "Playing"),
-			ENUM_ENTRY (RB_ENTRY_VIEW_PAUSED, "Paused"),
+			ENUM_ENTRY (RB_ENTRY_VIEW_NOT_PLAYING, "not-playing"),
+			ENUM_ENTRY (RB_ENTRY_VIEW_PLAYING, "playing"),
+			ENUM_ENTRY (RB_ENTRY_VIEW_PAUSED, "paused"),
 			{ 0, 0, 0 }
 		};
 
