@@ -1,6 +1,4 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
- *
- *  arch-tag: Implementation of local song properties dialog
  *
  *  Copyright (C) 2002 Olivier Martin <olive.martin@gmail.com>
  *  Copyright (C) 2003 Colin Walters <walters@verbum.org>
@@ -44,12 +42,11 @@
 
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
-#include <glade/glade.h>
 
 #include "rhythmdb.h"
 #include "rhythmdb-property-model.h"
 #include "rb-song-info.h"
-#include "rb-glade-helpers.h"
+#include "rb-builder-helpers.h"
 #include "rb-dialog.h"
 #include "rb-rating.h"
 #include "rb-preferences.h"
@@ -57,11 +54,11 @@
 #include "rb-source.h"
 #include "rb-shell.h"
 #include "rb-file-helpers.h"
+#include "rb-util.h"
 
 static void rb_song_info_class_init (RBSongInfoClass *klass);
 static void rb_song_info_init (RBSongInfo *song_info);
-static GObject *rb_song_info_constructor (GType type, guint n_construct_properties,
-					  GObjectConstructParam *construct_properties);
+static void rb_song_info_constructed (GObject *object);
 
 static void rb_song_info_show (GtkWidget *widget);
 static void rb_song_info_dispose (GObject *object);
@@ -129,12 +126,20 @@ struct RBSongInfoPrivate
 	GtkWidget   *title;
 	GtkWidget   *artist;
 	GtkWidget   *album;
+	GtkWidget   *album_artist;
 	GtkWidget   *genre;
 	GtkWidget   *track_cur;
 	GtkWidget   *disc_cur;
 	GtkWidget   *year;
+	GtkWidget   *comment;
+	GtkTextBuffer *comment_buffer;
 	GtkWidget   *playback_error_box;
 	GtkWidget   *playback_error_label;
+	GtkWidget   *bpm;
+
+	GtkWidget   *artist_sortname;
+	GtkWidget   *album_sortname;
+	GtkWidget   *album_artist_sortname;
 
 	GtkWidget   *bitrate;
 	GtkWidget   *duration;
@@ -210,7 +215,7 @@ rb_song_info_class_init (RBSongInfoClass *klass)
 
 	object_class->set_property = rb_song_info_set_property;
 	object_class->get_property = rb_song_info_get_property;
-	object_class->constructor = rb_song_info_constructor;
+	object_class->constructed = rb_song_info_constructed;
 
 	widget_class->show = rb_song_info_show;
 
@@ -334,7 +339,7 @@ rb_song_info_init (RBSongInfo *song_info)
 
 	gtk_container_set_border_width (GTK_CONTAINER (song_info), 5);
 	gtk_window_set_resizable (GTK_WINDOW (song_info), TRUE);
-	gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (song_info)->vbox), 2);
+	gtk_box_set_spacing (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (song_info))), 2);
 }
 
 static void
@@ -347,8 +352,7 @@ rb_song_info_show (GtkWidget *widget)
 }
 
 static void
-rb_song_info_construct_single (RBSongInfo *song_info, GladeXML *xml,
-			       gboolean editable)
+rb_song_info_construct_single (RBSongInfo *song_info, GtkBuilder *builder, gboolean editable)
 {
 	song_info->priv->backward = gtk_dialog_add_button (GTK_DIALOG (song_info),
 							   GTK_STOCK_GO_BACK,
@@ -371,28 +375,33 @@ rb_song_info_construct_single (RBSongInfo *song_info, GladeXML *xml,
 	gtk_window_set_title (GTK_WINDOW (song_info), _("Song Properties"));
 
 	/* get the widgets from the XML */
-	song_info->priv->notebook      = glade_xml_get_widget (xml, "song_info_vbox");
-	song_info->priv->title         = glade_xml_get_widget (xml, "song_info_title");
-	song_info->priv->track_cur     = glade_xml_get_widget (xml, "song_info_track_cur");
-	song_info->priv->bitrate       = glade_xml_get_widget (xml, "song_info_bitrate");
-	song_info->priv->duration      = glade_xml_get_widget (xml, "song_info_duration");
-	song_info->priv->location = glade_xml_get_widget (xml, "song_info_location");
-	song_info->priv->filesize = glade_xml_get_widget (xml, "song_info_filesize");
-	song_info->priv->date_added    = glade_xml_get_widget (xml, "song_info_dateadded");
-	song_info->priv->play_count    = glade_xml_get_widget (xml, "song_info_playcount");
-	song_info->priv->last_played   = glade_xml_get_widget (xml, "song_info_lastplayed");
-	song_info->priv->name = glade_xml_get_widget (xml, "song_info_name");
+	song_info->priv->notebook      = GTK_WIDGET (gtk_builder_get_object (builder, "song_info_vbox"));
+	song_info->priv->title         = GTK_WIDGET (gtk_builder_get_object (builder, "song_info_title"));
+	song_info->priv->track_cur     = GTK_WIDGET (gtk_builder_get_object (builder, "song_info_track_cur"));
+	song_info->priv->bitrate       = GTK_WIDGET (gtk_builder_get_object (builder, "song_info_bitrate"));
+	song_info->priv->duration      = GTK_WIDGET (gtk_builder_get_object (builder, "song_info_duration"));
+	song_info->priv->bpm           = GTK_WIDGET (gtk_builder_get_object (builder, "song_info_bpm"));
+	song_info->priv->location = GTK_WIDGET (gtk_builder_get_object (builder, "song_info_location"));
+	song_info->priv->filesize = GTK_WIDGET (gtk_builder_get_object (builder, "song_info_filesize"));
+	song_info->priv->date_added    = GTK_WIDGET (gtk_builder_get_object (builder, "song_info_dateadded"));
+	song_info->priv->play_count    = GTK_WIDGET (gtk_builder_get_object (builder, "song_info_playcount"));
+	song_info->priv->last_played   = GTK_WIDGET (gtk_builder_get_object (builder, "song_info_lastplayed"));
+	song_info->priv->name = GTK_WIDGET (gtk_builder_get_object (builder, "song_info_name"));
+	song_info->priv->comment = GTK_WIDGET (gtk_builder_get_object (builder, "song_info_comment"));
+	song_info->priv->comment_buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (song_info->priv->comment));
 
-	rb_glade_boldify_label (xml, "title_label");
-	rb_glade_boldify_label (xml, "trackn_label");
-	rb_glade_boldify_label (xml, "name_label");
-	rb_glade_boldify_label (xml, "location_label");
-	rb_glade_boldify_label (xml, "filesize_label");
-	rb_glade_boldify_label (xml, "date_added_label");
-	rb_glade_boldify_label (xml, "last_played_label");
-	rb_glade_boldify_label (xml, "play_count_label");
-	rb_glade_boldify_label (xml, "duration_label");
-	rb_glade_boldify_label (xml, "bitrate_label");
+	rb_builder_boldify_label (builder, "title_label");
+	rb_builder_boldify_label (builder, "trackn_label");
+	rb_builder_boldify_label (builder, "name_label");
+	rb_builder_boldify_label (builder, "location_label");
+	rb_builder_boldify_label (builder, "filesize_label");
+	rb_builder_boldify_label (builder, "date_added_label");
+	rb_builder_boldify_label (builder, "last_played_label");
+	rb_builder_boldify_label (builder, "play_count_label");
+	rb_builder_boldify_label (builder, "duration_label");
+	rb_builder_boldify_label (builder, "bitrate_label");
+	rb_builder_boldify_label (builder, "bpm_label");
+	rb_builder_boldify_label (builder, "comment_label");
 
 	/* whenever you press a mnemonic, the associated GtkEntry's text gets highlighted */
 	g_signal_connect_object (G_OBJECT (song_info->priv->title),
@@ -403,24 +412,27 @@ rb_song_info_construct_single (RBSongInfo *song_info, GladeXML *xml,
 				 "mnemonic-activate",
 				 G_CALLBACK (rb_song_info_mnemonic_cb),
 				 NULL, 0);
+	g_signal_connect_object (G_OBJECT (song_info->priv->comment),
+				 "mnemonic-activate",
+				 G_CALLBACK (rb_song_info_mnemonic_cb),
+				 NULL, 0);
 
 	gtk_editable_set_editable (GTK_EDITABLE (song_info->priv->title), editable);
 	gtk_editable_set_editable  (GTK_EDITABLE (song_info->priv->track_cur), editable);
+	gtk_text_view_set_editable (GTK_TEXT_VIEW (song_info->priv->comment), editable);
 
 	/* default focus */
 	gtk_widget_grab_focus (song_info->priv->title);
-
 }
 
 static void
-rb_song_info_construct_multiple (RBSongInfo *song_info, GladeXML *xml,
-				 gboolean editable)
+rb_song_info_construct_multiple (RBSongInfo *song_info, GtkBuilder *builder, gboolean editable)
 {
 	gtk_window_set_title (GTK_WINDOW (song_info),
 			      _("Multiple Song Properties"));
 	gtk_widget_grab_focus (song_info->priv->artist);
 
-	song_info->priv->notebook = glade_xml_get_widget (xml, "song_info_notebook");
+	song_info->priv->notebook = GTK_WIDGET (gtk_builder_get_object (builder, "song_info_notebook"));
 }
 
 static void
@@ -435,27 +447,25 @@ rb_song_info_add_completion (GtkEntry *entry, RhythmDBPropertyModel *propmodel)
 	g_object_unref (completion);
 }
 
-static GObject *
-rb_song_info_constructor (GType type, guint n_construct_properties,
-			  GObjectConstructParam *construct_properties)
+static void
+rb_song_info_constructed (GObject *object)
 {
 	RBSongInfo *song_info;
-	RBSongInfoClass *klass;
-	GladeXML *xml;
 	GList *selected_entries;
 	GList *tem;
 	gboolean editable = TRUE;
 	RBShell *shell;
 	AtkObject *lobj, *robj;
+	GtkBuilder *builder;
+	GtkWidget *content_area;
 
-	klass = RB_SONG_INFO_CLASS (g_type_class_peek (RB_TYPE_SONG_INFO));
+	RB_CHAIN_GOBJECT_METHOD (rb_song_info_parent_class, constructed, object);
 
-	song_info = RB_SONG_INFO (G_OBJECT_CLASS (rb_song_info_parent_class)
-			->constructor (type, n_construct_properties, construct_properties));
+	song_info = RB_SONG_INFO (object);
 
 	selected_entries = rb_entry_view_get_selected_entries (song_info->priv->entry_view);
 
-	g_return_val_if_fail (selected_entries != NULL, NULL);
+	g_return_if_fail (selected_entries != NULL);
 
 	for (tem = selected_entries; tem; tem = tem->next) {
 		if (!rhythmdb_entry_is_editable (song_info->priv->db,
@@ -478,40 +488,44 @@ rb_song_info_constructor (GType type, guint n_construct_properties,
 		song_info->priv->selected_entries = selected_entries;
 	}
 
+	content_area = gtk_dialog_get_content_area (GTK_DIALOG (song_info));
 	if (song_info->priv->current_entry) {
-		xml = rb_glade_xml_new ("song-info.glade",
-					"song_info_vbox",
-					song_info);
-		gtk_container_add (GTK_CONTAINER (GTK_DIALOG (song_info)->vbox),
-				   glade_xml_get_widget (xml, "song_info_vbox"));
+		builder = rb_builder_load ("song-info.ui", song_info);
+		gtk_container_add (GTK_CONTAINER (content_area),
+				   GTK_WIDGET (gtk_builder_get_object (builder, "song_info_vbox")));
 	} else {
-		xml = rb_glade_xml_new ("song-info-multiple.glade",
-					"song_info_notebook",
-					song_info);
-		gtk_container_add (GTK_CONTAINER (GTK_DIALOG (song_info)->vbox),
-				   glade_xml_get_widget (xml, "song_info_notebook"));
+		builder = rb_builder_load ("song-info-multiple.ui", song_info);
+		gtk_container_add (GTK_CONTAINER (content_area),
+				   GTK_WIDGET (gtk_builder_get_object (builder, "song_info_notebook")));
 	}
 
-	glade_xml_signal_autoconnect (xml);
+	song_info->priv->artist = GTK_WIDGET (gtk_builder_get_object (builder, "song_info_artist"));
+	song_info->priv->album = GTK_WIDGET (gtk_builder_get_object (builder, "song_info_album"));
+	song_info->priv->album_artist = GTK_WIDGET (gtk_builder_get_object (builder, "song_info_album_artist"));
+	song_info->priv->genre = GTK_WIDGET (gtk_builder_get_object (builder, "song_info_genre"));
+	song_info->priv->year = GTK_WIDGET (gtk_builder_get_object (builder, "song_info_year"));
+	song_info->priv->playback_error_box = GTK_WIDGET (gtk_builder_get_object (builder, "song_info_error_box"));
+	song_info->priv->playback_error_label = GTK_WIDGET (gtk_builder_get_object (builder, "song_info_error_label"));
+	song_info->priv->disc_cur = GTK_WIDGET (gtk_builder_get_object (builder, "song_info_disc_cur"));
 
-	song_info->priv->artist = glade_xml_get_widget (xml, "song_info_artist");
-	song_info->priv->album = glade_xml_get_widget (xml, "song_info_album");
-	song_info->priv->genre = glade_xml_get_widget (xml, "song_info_genre");
-	song_info->priv->year = glade_xml_get_widget (xml, "song_info_year");
-	song_info->priv->playback_error_box = glade_xml_get_widget (xml, "song_info_error_box");
-	song_info->priv->playback_error_label = glade_xml_get_widget (xml, "song_info_error_label");
-	song_info->priv->disc_cur = glade_xml_get_widget (xml, "song_info_disc_cur");
+	song_info->priv->artist_sortname = GTK_WIDGET (gtk_builder_get_object (builder, "song_info_artist_sortname"));
+	song_info->priv->album_sortname = GTK_WIDGET (gtk_builder_get_object (builder, "song_info_album_sortname"));
+	song_info->priv->album_artist_sortname = GTK_WIDGET (gtk_builder_get_object (builder, "song_info_album_artist_sortname"));
 
 	rb_song_info_add_completion (GTK_ENTRY (song_info->priv->genre), song_info->priv->genres);
 	rb_song_info_add_completion (GTK_ENTRY (song_info->priv->artist), song_info->priv->artists);
 	rb_song_info_add_completion (GTK_ENTRY (song_info->priv->album), song_info->priv->albums);
 
-	rb_glade_boldify_label (xml, "album_label");
-	rb_glade_boldify_label (xml, "artist_label");
-	rb_glade_boldify_label (xml, "genre_label");
-	rb_glade_boldify_label (xml, "year_label");
-	rb_glade_boldify_label (xml, "rating_label");
-	rb_glade_boldify_label (xml, "discn_label");
+	rb_builder_boldify_label (builder, "album_label");
+	rb_builder_boldify_label (builder, "artist_label");
+	rb_builder_boldify_label (builder, "album_artist_label");
+	rb_builder_boldify_label (builder, "genre_label");
+	rb_builder_boldify_label (builder, "year_label");
+	rb_builder_boldify_label (builder, "rating_label");
+	rb_builder_boldify_label (builder, "discn_label");
+	rb_builder_boldify_label (builder, "artist_sortname_label");
+	rb_builder_boldify_label (builder, "album_sortname_label");
+	rb_builder_boldify_label (builder, "album_artist_sortname_label");
 
 	g_signal_connect_object (G_OBJECT (song_info->priv->artist),
 				 "mnemonic-activate",
@@ -521,6 +535,10 @@ rb_song_info_constructor (GType type, guint n_construct_properties,
 				 "mnemonic-activate",
 				 G_CALLBACK (rb_song_info_mnemonic_cb),
 				 NULL, 0);
+	g_signal_connect_object (G_OBJECT (song_info->priv->album_artist),
+				 "mnemonic-activate",
+				 G_CALLBACK (rb_song_info_mnemonic_cb),
+				 NULL, 0);
 	g_signal_connect_object (G_OBJECT (song_info->priv->genre),
 				 "mnemonic-activate",
 				 G_CALLBACK (rb_song_info_mnemonic_cb),
@@ -533,18 +551,30 @@ rb_song_info_constructor (GType type, guint n_construct_properties,
 				 "mnemonic-activate",
 				 G_CALLBACK (rb_song_info_mnemonic_cb),
 				 NULL, 0);
+	g_signal_connect_object (G_OBJECT (song_info->priv->artist_sortname),
+				 "mnemonic-activate",
+				 G_CALLBACK (rb_song_info_mnemonic_cb),
+				 NULL, 0);
+	g_signal_connect_object (G_OBJECT (song_info->priv->album_sortname),
+				 "mnemonic-activate",
+				 G_CALLBACK (rb_song_info_mnemonic_cb),
+				 NULL, 0);
+	g_signal_connect_object (G_OBJECT (song_info->priv->album_artist_sortname),
+				 "mnemonic-activate",
+				 G_CALLBACK (rb_song_info_mnemonic_cb),
+				 NULL, 0);
 
 	/* this widget has to be customly created */
 	song_info->priv->rating = GTK_WIDGET (rb_rating_new ());
 	g_signal_connect_object (song_info->priv->rating, "rated",
 				 G_CALLBACK (rb_song_info_rated_cb),
 				 G_OBJECT (song_info), 0);
-	gtk_container_add (GTK_CONTAINER (glade_xml_get_widget (xml, "song_info_rating_container")),
+	gtk_container_add (GTK_CONTAINER (gtk_builder_get_object (builder, "song_info_rating_container")),
 			   song_info->priv->rating);
-	g_object_set (glade_xml_get_widget (xml, "rating_label"), "mnemonic-widget", song_info->priv->rating, NULL);
+	g_object_set (gtk_builder_get_object (builder, "rating_label"), "mnemonic-widget", song_info->priv->rating, NULL);
 
 	/* add relationship between the rating label and the rating widget */
-	lobj = gtk_widget_get_accessible (glade_xml_get_widget (xml, "rating_label"));
+	lobj = gtk_widget_get_accessible (GTK_WIDGET (gtk_builder_get_object (builder, "rating_label")));
 	robj = gtk_widget_get_accessible (song_info->priv->rating);
 
 	atk_object_add_relationship (lobj, ATK_RELATION_LABEL_FOR, robj);
@@ -552,16 +582,18 @@ rb_song_info_constructor (GType type, guint n_construct_properties,
 
 	gtk_editable_set_editable (GTK_EDITABLE (song_info->priv->artist), editable);
 	gtk_editable_set_editable (GTK_EDITABLE (song_info->priv->album), editable);
+	gtk_editable_set_editable (GTK_EDITABLE (song_info->priv->album_artist), editable);
 	gtk_editable_set_editable (GTK_EDITABLE (song_info->priv->genre), editable);
 	gtk_editable_set_editable (GTK_EDITABLE (song_info->priv->year), editable);
 	gtk_editable_set_editable (GTK_EDITABLE (song_info->priv->disc_cur), editable);
 
 	/* Finish construction */
 	if (song_info->priv->current_entry) {
-		rb_song_info_construct_single (song_info, xml, editable);
+
+		rb_song_info_construct_single (song_info, builder, editable);
 		rb_song_info_populate_dialog (song_info);
 	} else {
-		rb_song_info_construct_multiple (song_info, xml, editable);
+		rb_song_info_construct_multiple (song_info, builder, editable);
 		rb_song_info_populate_dialog_multiple (song_info);
 	}
 	g_object_get (G_OBJECT (song_info->priv->source), "shell", &shell, NULL);
@@ -575,8 +607,7 @@ rb_song_info_constructor (GType type, guint n_construct_properties,
 	gtk_dialog_set_default_response (GTK_DIALOG (song_info),
 					 GTK_RESPONSE_CLOSE);
 
-	g_object_unref (G_OBJECT (xml));
-	return G_OBJECT (song_info);
+	g_object_unref (builder);
 }
 
 static void
@@ -761,8 +792,12 @@ rb_song_info_new (RBSource *source, RBEntryView *entry_view)
 	RBSongInfo *song_info;
 
         g_return_val_if_fail (RB_IS_SOURCE (source), NULL);
-	if (!entry_view)
+	if (entry_view == NULL) {
 		entry_view = rb_source_get_entry_view (source);
+		if (entry_view == NULL) {
+			return NULL;
+		}
+	}
 
 	if (rb_entry_view_have_selection (entry_view) == FALSE)
 		return NULL;
@@ -892,21 +927,41 @@ rb_song_info_populate_num_field (GtkEntry *field, gulong num)
 	g_free (tmp);
 }
 
+static void
+rb_song_info_populate_dnum_field (GtkEntry *field, gdouble num)
+{
+	char *tmp;
+	if (num > 0)
+		tmp = g_strdup_printf ("%.2f", num);
+	else
+		tmp = g_strdup (_("Unknown"));
+	gtk_entry_set_text (field, tmp);
+	g_free (tmp);
+}
+
 static void
 rb_song_info_populate_dialog_multiple (RBSongInfo *song_info)
 {
 	gboolean mixed_artists = FALSE;
 	gboolean mixed_albums = FALSE;
+	gboolean mixed_album_artists = FALSE;
 	gboolean mixed_genres = FALSE;
 	gboolean mixed_years = FALSE;
 	gboolean mixed_disc_numbers = FALSE;
 	gboolean mixed_ratings = FALSE;
+	gboolean mixed_artist_sortnames = FALSE;
+	gboolean mixed_album_sortnames = FALSE;
+	gboolean mixed_album_artist_sortnames = FALSE;
 	const char *artist = NULL;
 	const char *album = NULL;
+	const char *album_artist = NULL;
 	const char *genre = NULL;
 	int year = 0;
 	int disc_number = 0;
 	double rating = 0.0; /* Zero is used for both "unrated" and "mixed ratings" too */
+	const char *artist_sortname = NULL;
+	const char *album_sortname = NULL;
+	const char *album_artist_sortname = NULL;
 	GList *l;
 
 	g_assert (song_info->priv->selected_entries);
@@ -915,24 +970,34 @@ rb_song_info_populate_dialog_multiple (RBSongInfo *song_info)
 		RhythmDBEntry *entry;
 		const char *entry_artist;
 		const char *entry_album;
+		const char *entry_album_artist;
 		const char *entry_genre;
 		int entry_year;
 		int entry_disc_number;
 		double entry_rating;
+		const char *entry_artist_sortname;
+		const char *entry_album_sortname;
+		const char *entry_album_artist_sortname;
 
 		entry = (RhythmDBEntry*)l->data;
 		entry_artist = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_ARTIST);
 		entry_album = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_ALBUM);
+		entry_album_artist = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_ALBUM_ARTIST);
 		entry_genre = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_GENRE);
 		entry_year = rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_YEAR);
 		entry_disc_number = rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_DISC_NUMBER);
 		entry_rating = rhythmdb_entry_get_double (entry, RHYTHMDB_PROP_RATING);
+		entry_artist_sortname = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_ARTIST_SORTNAME);
+		entry_album_sortname = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_ALBUM_SORTNAME);
+		entry_album_artist_sortname = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_ALBUM_ARTIST_SORTNAME);
 
 		/* grab first valid values */
 		if (artist == NULL)
 			artist = entry_artist;
 		if (album == NULL)
 			album = entry_album;
+		if (album_artist == NULL)
+			album_artist = entry_album_artist;
 		if (genre == NULL)
 			genre = entry_genre;
 		if (year == 0)
@@ -941,12 +1006,20 @@ rb_song_info_populate_dialog_multiple (RBSongInfo *song_info)
 			disc_number = entry_disc_number;
 		if (fabs(rating) < EPSILON)
 			rating = entry_rating;
+		if (artist_sortname == NULL)
+			artist_sortname = entry_artist_sortname;
+		if (album_sortname == NULL)
+			album_sortname = entry_album_sortname;
+		if (album_artist_sortname == NULL)
+			album_artist_sortname = entry_album_artist_sortname;
 
 		/* locate mixed values */
 		if (artist != entry_artist)
 			mixed_artists = TRUE;
 		if (album != entry_album)
 			mixed_albums = TRUE;
+		if (album_artist != entry_album_artist)
+			mixed_album_artists = TRUE;
 		if (genre != entry_genre)
 			mixed_genres = TRUE;
 		if (year != entry_year)
@@ -955,10 +1028,17 @@ rb_song_info_populate_dialog_multiple (RBSongInfo *song_info)
 			mixed_disc_numbers = TRUE;
 		if (fabs(rating - entry_rating) >= EPSILON)
 			mixed_ratings = TRUE;
+		if (artist_sortname != entry_artist_sortname)
+			mixed_artist_sortnames = TRUE;
+		if (album_sortname != entry_album_sortname)
+			mixed_album_sortnames = TRUE;
+		if (album_artist_sortname != entry_album_artist_sortname)
+			mixed_album_artist_sortnames = TRUE;
 
 		/* don't continue search if everything is mixed */
 		if (mixed_artists && mixed_albums && mixed_genres &&
-		    mixed_years && mixed_disc_numbers && mixed_ratings)
+		    mixed_years && mixed_disc_numbers && mixed_ratings &&
+		    mixed_artist_sortnames && mixed_album_sortnames)
 			break;
 	}
 
@@ -966,6 +1046,8 @@ rb_song_info_populate_dialog_multiple (RBSongInfo *song_info)
 		gtk_entry_set_text (GTK_ENTRY (song_info->priv->artist), artist);
 	if (!mixed_albums && album != NULL)
 		gtk_entry_set_text (GTK_ENTRY (song_info->priv->album), album);
+	if (!mixed_album_artists && album_artist != NULL)
+		gtk_entry_set_text (GTK_ENTRY (song_info->priv->album_artist), album_artist);
 	if (!mixed_genres && genre != NULL)
 		gtk_entry_set_text (GTK_ENTRY (song_info->priv->genre), genre);
 	if (!mixed_years && year != 0)
@@ -974,6 +1056,12 @@ rb_song_info_populate_dialog_multiple (RBSongInfo *song_info)
 		rb_song_info_populate_num_field (GTK_ENTRY (song_info->priv->disc_cur), disc_number);
 	if (!mixed_ratings && fabs(rating) >= EPSILON)
 		g_object_set (G_OBJECT (song_info->priv->rating), "rating", rating, NULL);
+	if (!mixed_artist_sortnames && artist_sortname != NULL)
+		gtk_entry_set_text (GTK_ENTRY (song_info->priv->artist_sortname), artist_sortname);
+	if (!mixed_album_sortnames && album_sortname != NULL)
+		gtk_entry_set_text (GTK_ENTRY (song_info->priv->album_sortname), album_sortname);
+	if (!mixed_album_artist_sortnames && album_artist_sortname != NULL)
+		gtk_entry_set_text (GTK_ENTRY (song_info->priv->album_artist_sortname), album_artist_sortname);
 }
 
 static void
@@ -982,6 +1070,7 @@ rb_song_info_populate_dialog (RBSongInfo *song_info)
 	const char *text;
 	char *tmp;
 	gulong num;
+	gdouble dnum;
 
 	g_assert (song_info->priv->current_entry);
 
@@ -999,6 +1088,8 @@ rb_song_info_populate_dialog (RBSongInfo *song_info)
 	gtk_entry_set_text (GTK_ENTRY (song_info->priv->artist), text);
 	text = rhythmdb_entry_get_string (song_info->priv->current_entry, RHYTHMDB_PROP_ALBUM);
 	gtk_entry_set_text (GTK_ENTRY (song_info->priv->album), text);
+	text = rhythmdb_entry_get_string (song_info->priv->current_entry, RHYTHMDB_PROP_ALBUM_ARTIST);
+	gtk_entry_set_text (GTK_ENTRY (song_info->priv->album_artist), text);
 	text = rhythmdb_entry_get_string (song_info->priv->current_entry, RHYTHMDB_PROP_GENRE);
 	gtk_entry_set_text (GTK_ENTRY (song_info->priv->genre), text);
 
@@ -1006,6 +1097,10 @@ rb_song_info_populate_dialog (RBSongInfo *song_info)
 	rb_song_info_populate_num_field (GTK_ENTRY (song_info->priv->track_cur), num);
 	num = rhythmdb_entry_get_ulong (song_info->priv->current_entry, RHYTHMDB_PROP_DISC_NUMBER);
 	rb_song_info_populate_num_field (GTK_ENTRY (song_info->priv->disc_cur), num);
+	dnum = rhythmdb_entry_get_double (song_info->priv->current_entry, RHYTHMDB_PROP_BPM);
+	rb_song_info_populate_dnum_field (GTK_ENTRY (song_info->priv->bpm), dnum);
+	text = rhythmdb_entry_get_string (song_info->priv->current_entry, RHYTHMDB_PROP_COMMENT);
+	gtk_text_buffer_set_text (song_info->priv->comment_buffer, text, -1);
 
 	rb_song_info_update_duration (song_info);
 	rb_song_info_update_location (song_info);
@@ -1017,6 +1112,13 @@ rb_song_info_populate_dialog (RBSongInfo *song_info)
 	rb_song_info_update_rating (song_info);
 	rb_song_info_update_year (song_info);
 	rb_song_info_update_playback_error (song_info);
+
+	text = rhythmdb_entry_get_string (song_info->priv->current_entry, RHYTHMDB_PROP_ARTIST_SORTNAME);
+	gtk_entry_set_text (GTK_ENTRY (song_info->priv->artist_sortname), text);
+	text = rhythmdb_entry_get_string (song_info->priv->current_entry, RHYTHMDB_PROP_ALBUM_SORTNAME);
+	gtk_entry_set_text (GTK_ENTRY (song_info->priv->album_sortname), text);
+	text = rhythmdb_entry_get_string (song_info->priv->current_entry, RHYTHMDB_PROP_ALBUM_ARTIST_SORTNAME);
+	gtk_entry_set_text (GTK_ENTRY (song_info->priv->album_artist_sortname), text);
 }
 
 static void
@@ -1098,8 +1200,9 @@ rb_song_info_update_location (RBSongInfo *song_info)
 	text = rhythmdb_entry_get_string (song_info->priv->current_entry, RHYTHMDB_PROP_LOCATION);
 
 	if (text != NULL) {
-		char *tmp, *tmp_utf8;
-		char *basename, *dir, *desktopdir;
+		char *tmp;
+		char *tmp_utf8;
+		char *basename;
 
 		basename = g_path_get_basename (text);
 		tmp = g_uri_unescape_string (basename, NULL);
@@ -1117,27 +1220,41 @@ rb_song_info_update_location (RBSongInfo *song_info)
 		}
 
 		g_free (tmp_utf8);
+		tmp_utf8 = NULL;
 
 		if (rb_uri_is_local (text)) {
+			const char *desktopdir;
+			char *dir;
+
+			/* for local files, convert to path, extract dirname, and convert to utf8 */
 			tmp = g_filename_from_uri (text, NULL, NULL);
-		}
-		if (tmp == NULL) {
-			tmp = g_strdup (text);
-		}
 
-		dir = g_path_get_dirname (tmp);
-		g_free (tmp);
-		tmp = g_uri_unescape_string (dir, NULL);
-		g_free (dir);
-		tmp_utf8 = g_filename_to_utf8 (tmp, -1, NULL, NULL, NULL);
-		g_free (tmp);
+			dir = g_path_get_dirname (tmp);
+			g_free (tmp);
+			tmp_utf8 = g_filename_to_utf8 (dir, -1, NULL, NULL, NULL);
+			g_free (dir);
 
-		desktopdir = g_build_filename (g_get_home_dir (), "Desktop", NULL);
-		if ((tmp_utf8 != NULL) && (strcmp (tmp_utf8, desktopdir) == 0)) {
-			g_free (tmp_utf8);
-			tmp_utf8 = g_strdup (_("On the desktop"));
+			/* special case for files on the desktop */
+			desktopdir = g_get_user_special_dir (G_USER_DIRECTORY_DESKTOP);
+			if (g_strcmp0 (tmp_utf8, desktopdir) == 0) {
+				g_free (tmp_utf8);
+				tmp_utf8 = g_strdup (_("On the desktop"));
+			}
+		} else {
+			GFile *file;
+			GFile *parent;
+			char *parent_uri;
+
+			/* get parent URI and unescape it */
+			file = g_file_new_for_uri (text);
+			parent = g_file_get_parent (file);
+			parent_uri = g_file_get_uri (parent);
+			g_object_unref (file);
+			g_object_unref (parent);
+
+			tmp_utf8 = g_uri_unescape_string (parent_uri, NULL);
+			g_free (parent_uri);
 		}
-		g_free (desktopdir);
 
 		if (tmp_utf8 != NULL) {
 			gtk_entry_set_text (GTK_ENTRY (song_info->priv->location),
@@ -1391,12 +1508,39 @@ rb_song_info_update_date_added (RBSongInfo *song_info)
 	gtk_label_set_text (GTK_LABEL (song_info->priv->date_added), str);
 }
 
+static gboolean
+sync_string_property (RBSongInfo *dialog, RhythmDBPropType property, GtkWidget *entry)
+{
+	const char *new_text;
+	GValue val = {0,};
+	GList *t;
+	gboolean changed = FALSE;
+
+	new_text = gtk_entry_get_text (GTK_ENTRY (entry));
+	if (strlen (new_text) == 0)
+		return FALSE;
+
+	g_value_init (&val, G_TYPE_STRING);
+	g_value_set_string (&val, new_text);
+	for (t = dialog->priv->selected_entries; t != NULL; t = t->next) {
+		const char *entry_value;
+		RhythmDBEntry *dbentry;
+
+		dbentry = (RhythmDBEntry *)t->data;
+		entry_value = rhythmdb_entry_get_string (dbentry, property);
+
+		if (g_strcmp0 (new_text, entry_value) == 0)
+			continue;
+		rhythmdb_entry_set (dialog->priv->db, dbentry, property, &val);
+		changed = TRUE;
+	}
+	g_value_unset (&val);
+	return changed;
+}
+
 static void
 rb_song_info_sync_entries_multiple (RBSongInfo *dialog)
 {
-	const char *genre = gtk_entry_get_text (GTK_ENTRY (dialog->priv->genre));
-	const char *artist = gtk_entry_get_text (GTK_ENTRY (dialog->priv->artist));
-	const char *album = gtk_entry_get_text (GTK_ENTRY (dialog->priv->album));
 	const char *year_str = gtk_entry_get_text (GTK_ENTRY (dialog->priv->year));
 	const char *discn_str = gtk_entry_get_text (GTK_ENTRY (dialog->priv->disc_cur));
 
@@ -1408,59 +1552,13 @@ rb_song_info_sync_entries_multiple (RBSongInfo *dialog)
 	gboolean changed = FALSE;
 	RhythmDBEntry *entry;
 
-	if (strlen (album) > 0) {
-		g_value_init (&val, G_TYPE_STRING);
-		g_value_set_string (&val, album);
-		for (tem = dialog->priv->selected_entries; tem; tem = tem->next) {
-			const char *entry_album;
-
-			entry = (RhythmDBEntry *)tem->data;
-			entry_album = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_ALBUM);
-
-			if (strcmp (album, entry_album) == 0)
-				continue;
-			rhythmdb_entry_set (dialog->priv->db, entry,
-					    RHYTHMDB_PROP_ALBUM, &val);
-			changed = TRUE;
-		}
-		g_value_unset (&val);
-	}
-
-	if (strlen (artist) > 0) {
-		g_value_init (&val, G_TYPE_STRING);
-		g_value_set_string (&val, artist);
-		for (tem = dialog->priv->selected_entries; tem; tem = tem->next) {
-			const char *entry_artist;
-
-			entry = (RhythmDBEntry *)tem->data;
-			entry_artist = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_ARTIST);
-
-			if (strcmp (artist, entry_artist) == 0)
-				continue;
-			rhythmdb_entry_set (dialog->priv->db, entry,
-					    RHYTHMDB_PROP_ARTIST, &val);
-			changed = TRUE;
-		}
-		g_value_unset (&val);
-	}
-
-	if (strlen (genre) > 0) {
-		g_value_init (&val, G_TYPE_STRING);
-		g_value_set_string (&val, genre);
-		for (tem = dialog->priv->selected_entries; tem; tem = tem->next) {
-			const char *entry_genre;
-
-			entry = (RhythmDBEntry *)tem->data;
-			entry_genre = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_GENRE);
-
-			if (strcmp (genre, entry_genre) == 0)
-				continue;
-			rhythmdb_entry_set (dialog->priv->db, entry,
-					    RHYTHMDB_PROP_GENRE, &val);
-			changed = TRUE;
-		}
-		g_value_unset (&val);
-	}
+	changed |= sync_string_property (dialog, RHYTHMDB_PROP_ALBUM, dialog->priv->album);
+	changed |= sync_string_property (dialog, RHYTHMDB_PROP_ARTIST, dialog->priv->artist);
+	changed |= sync_string_property (dialog, RHYTHMDB_PROP_ALBUM_ARTIST, dialog->priv->album_artist);
+	changed |= sync_string_property (dialog, RHYTHMDB_PROP_GENRE, dialog->priv->genre);
+	changed |= sync_string_property (dialog, RHYTHMDB_PROP_ARTIST_SORTNAME, dialog->priv->artist_sortname);
+	changed |= sync_string_property (dialog, RHYTHMDB_PROP_ALBUM_SORTNAME, dialog->priv->album_sortname);
+	changed |= sync_string_property (dialog, RHYTHMDB_PROP_ALBUM_ARTIST_SORTNAME, dialog->priv->album_artist_sortname);
 
 	if (strlen (year_str) > 0) {
 		GDate *date = NULL;
@@ -1523,27 +1621,44 @@ rb_song_info_sync_entry_single (RBSongInfo *dialog)
 	const char *genre;
 	const char *artist;
 	const char *album;
+	const char *album_artist;
 	const char *tracknum_str;
 	const char *discnum_str;
 	const char *year_str;
+	const char *artist_sortname;
+	const char *album_sortname;
+	const char *album_artist_sortname;
 	const char *entry_string;
+	const char *bpm_str;
+	char *comment = NULL;
 	char *endptr;
 	GType type;
 	gulong tracknum;
 	gulong discnum;
 	gulong year;
 	gulong entry_val;
+	gdouble bpm;
+	gdouble dentry_val;
 	GValue val = {0,};
 	gboolean changed = FALSE;
 	RhythmDBEntry *entry = dialog->priv->current_entry;
+	GtkTextIter start, end;
 
 	title = gtk_entry_get_text (GTK_ENTRY (dialog->priv->title));
 	genre = gtk_entry_get_text (GTK_ENTRY (dialog->priv->genre));
 	artist = gtk_entry_get_text (GTK_ENTRY (dialog->priv->artist));
 	album = gtk_entry_get_text (GTK_ENTRY (dialog->priv->album));
+	album_artist = gtk_entry_get_text (GTK_ENTRY (dialog->priv->album_artist));
 	tracknum_str = gtk_entry_get_text (GTK_ENTRY (dialog->priv->track_cur));
 	discnum_str = gtk_entry_get_text (GTK_ENTRY (dialog->priv->disc_cur));
 	year_str = gtk_entry_get_text (GTK_ENTRY (dialog->priv->year));
+	artist_sortname = gtk_entry_get_text (GTK_ENTRY (dialog->priv->artist_sortname));
+	album_sortname = gtk_entry_get_text (GTK_ENTRY (dialog->priv->album_sortname));
+	album_artist_sortname = gtk_entry_get_text (GTK_ENTRY (dialog->priv->album_artist_sortname));
+
+	/* Get comment text (string is allocated) */
+	gtk_text_buffer_get_bounds (dialog->priv->comment_buffer, &start, &end);
+	comment = gtk_text_buffer_get_text (dialog->priv->comment_buffer, &start, &end, FALSE);
 
 	g_signal_emit (dialog, rb_song_info_signals[PRE_METADATA_CHANGE], 0,
 		       entry);
@@ -1602,9 +1717,20 @@ rb_song_info_sync_entry_single (RBSongInfo *dialog)
 		if (date)
 			g_date_free (date);
 	}
-
+	bpm_str = gtk_entry_get_text (GTK_ENTRY (dialog->priv->bpm));
+	bpm = g_strtod (bpm_str, &endptr);
+	dentry_val = rhythmdb_entry_get_double (entry, RHYTHMDB_PROP_BPM);
+	if ((endptr != bpm_str) && (bpm != dentry_val)) {
+		type = rhythmdb_get_property_type (dialog->priv->db,
+						   RHYTHMDB_PROP_BPM);
+		g_value_init (&val, type);
+		g_value_set_double (&val, bpm);
+		rhythmdb_entry_set (dialog->priv->db, entry, RHYTHMDB_PROP_BPM, &val);
+		g_value_unset (&val);
+		changed = TRUE;
+	}
 	entry_string = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_TITLE);
-	if (strcmp (title, entry_string)) {
+	if (g_strcmp0 (title, entry_string)) {
 		type = rhythmdb_get_property_type (dialog->priv->db,
 						   RHYTHMDB_PROP_TITLE);
 		g_value_init (&val, type);
@@ -1616,7 +1742,7 @@ rb_song_info_sync_entry_single (RBSongInfo *dialog)
 	}
 
 	entry_string = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_ALBUM);
-	if (strcmp (album, entry_string)) {
+	if (g_strcmp0 (album, entry_string)) {
 		type = rhythmdb_get_property_type (dialog->priv->db,
 						   RHYTHMDB_PROP_ALBUM);
 		g_value_init (&val, type);
@@ -1628,7 +1754,7 @@ rb_song_info_sync_entry_single (RBSongInfo *dialog)
 	}
 
 	entry_string = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_ARTIST);
-	if (strcmp (artist, entry_string)) {
+	if (g_strcmp0 (artist, entry_string)) {
 		type = rhythmdb_get_property_type (dialog->priv->db,
 						   RHYTHMDB_PROP_ARTIST);
 		g_value_init (&val, type);
@@ -1639,8 +1765,20 @@ rb_song_info_sync_entry_single (RBSongInfo *dialog)
 		changed = TRUE;
 	}
 
+	entry_string = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_ALBUM_ARTIST);
+	if (g_strcmp0 (album_artist, entry_string)) {
+		type = rhythmdb_get_property_type (dialog->priv->db,
+						   RHYTHMDB_PROP_ALBUM_ARTIST);
+		g_value_init (&val, type);
+		g_value_set_string (&val, album_artist);
+		rhythmdb_entry_set (dialog->priv->db, entry,
+				    RHYTHMDB_PROP_ALBUM_ARTIST, &val);
+		g_value_unset (&val);
+		changed = TRUE;
+	}
+
 	entry_string = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_GENRE);
-	if (strcmp (genre, entry_string)) {
+	if (g_strcmp0 (genre, entry_string)) {
 		type = rhythmdb_get_property_type (dialog->priv->db,
 						   RHYTHMDB_PROP_GENRE);
 		g_value_init (&val, type);
@@ -1651,6 +1789,54 @@ rb_song_info_sync_entry_single (RBSongInfo *dialog)
 		changed = TRUE;
 	}
 
+	entry_string = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_ARTIST_SORTNAME);
+	if (g_strcmp0 (artist_sortname, entry_string)) {
+		type = rhythmdb_get_property_type (dialog->priv->db,
+						   RHYTHMDB_PROP_ARTIST_SORTNAME);
+		g_value_init (&val, type);
+		g_value_set_string (&val, artist_sortname);
+		rhythmdb_entry_set (dialog->priv->db, entry,
+				    RHYTHMDB_PROP_ARTIST_SORTNAME, &val);
+		g_value_unset (&val);
+		changed = TRUE;
+	}
+
+	entry_string = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_ALBUM_SORTNAME);
+	if (g_strcmp0 (album_sortname, entry_string)) {
+		type = rhythmdb_get_property_type (dialog->priv->db,
+						   RHYTHMDB_PROP_ALBUM_SORTNAME);
+		g_value_init (&val, type);
+		g_value_set_string (&val, album_sortname);
+		rhythmdb_entry_set (dialog->priv->db, entry,
+				    RHYTHMDB_PROP_ALBUM_SORTNAME, &val);
+		g_value_unset (&val);
+		changed = TRUE;
+	}
+
+	entry_string = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_COMMENT);
+	if (g_strcmp0 (comment, entry_string)) {
+		type = rhythmdb_get_property_type (dialog->priv->db,
+						   RHYTHMDB_PROP_COMMENT);
+		g_value_init (&val, type);
+		g_value_set_string (&val, comment);
+		rhythmdb_entry_set (dialog->priv->db, entry,
+				    RHYTHMDB_PROP_COMMENT, &val);
+		g_value_unset (&val);
+		changed = TRUE;
+	}
+
+	entry_string = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_ALBUM_ARTIST_SORTNAME);
+	if (g_strcmp0 (album_artist_sortname, entry_string)) {
+		type = rhythmdb_get_property_type (dialog->priv->db,
+						   RHYTHMDB_PROP_ALBUM_ARTIST_SORTNAME);
+		g_value_init (&val, type);
+		g_value_set_string (&val, album_artist_sortname);
+		rhythmdb_entry_set (dialog->priv->db, entry,
+				    RHYTHMDB_PROP_ALBUM_ARTIST_SORTNAME, &val);
+		g_value_unset (&val);
+		changed = TRUE;
+	}
+
 	/* FIXME: when an entry is SYNCed, a changed signal is emitted, and
 	 * this signal is also emitted, aren't they redundant?
 	 */
@@ -1659,6 +1845,8 @@ rb_song_info_sync_entry_single (RBSongInfo *dialog)
 
 	if (changed)
 		rhythmdb_commit (dialog->priv->db);
+
+	g_free (comment);
 }
 
 static void
